/**
 *         ===============================
 *          JPatch open source 3D modeler
 *         ===============================
 *      Copyright (c)2003 JPatch developer team
 *         see http://jpatch.sourceforge.net
 *
 * This file is part of JPatch.
 *
 * JPatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JPatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JPatch; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Heiko Irrgang
 * @version $Id: Shear.java,v 1.2 2003/06/03 07:47:58 god64 Exp $ $Date: 2003/06/03 07:47:58 $
 * last changed by $Author: god64 $
 *
 * Class: Shear
 * Hungarian notation prefix: *not yet... my brain dont works in hungarian notation... will go over it when finished*
 *
 **/

import jpatch.*;
import jpatch.plugin.*;
import javax.vecmath.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import plugins.modifiers.*;

// extend modifier plugin
public class Shear extends jpatch.plugin.Modifier {
	public static String name=new String("Shear");
	private Point3f cursor;
	private int lastMouseX=-1;
	private int difx;

	// init super class
	public Shear(GraphicsWindow gw) {
		super(gw, name);
	}

	// this method gets executed when user selects the plugin
	public void exec() {
		// Start undo recorder (displayed with undo "name")
		// needed when using own mousehandler, otherwise
		// every step when moving the mouse will be recorded
		// as single undo-step
		requestUndo(name);
		// get the position (Point3f) of the yellow cursor
		cursor=getCursor();
		// request mouse handling (stops mousehandling of mainwindow)
		requestMouseListener();
	}

	public void mouseMoved(MouseEvent e) {
		lastMouseX=e.getX();
	}

	public void mouseDragged(MouseEvent e) {
		if (e.getButton()==3) {
			// if right button pressed, give
			// mousehandler back to mainwindow
			freeMouseListener();
			// save undo
			freeUndo();
			return;
		}
		if (lastMouseX==-1) {
			lastMouseX=e.getX();
			return;
		}

		// calculate mouse movement
		difx=e.getX()-lastMouseX;

		// execute method shear(ControlPoint) with all
		// ControlPoints, deactivating automatic undo recorder
		withAllControlPoints("shear", false);
		lastMouseX=e.getX();
	}

	public void mouseReleased(MouseEvent e) {
		// if right button, free mousehandler and save undo
		if (e.getButton()==3) {
			freeMouseListener();
			freeUndo();
		}
	}

	public void shear(ControlPoint cp) {
		float disty;
		// only process selected ControlPoints
		if ((cp.getHead()).isSelected) {
			disty=cursor.y-cp.position.y;
			cp.position.x-=disty/500*difx;
		}
	}
}
