/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;

public class AddSegmentML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected Mesh mesh;
	protected int button;
	protected int mode = 0;
	protected ControlPoint cp1;
	protected ControlPoint cp2;
	protected Curve curve;
	protected boolean reset;
	
	final static int NEW = 1;
	final static int ADD_START = 2;
	final static int ADD_END = 3;
	
	protected MouseMotionAdapter motionListener;
	protected MouseAdapter weldListener;
	
	//protected MoverMML moveListener;
	
	public AddSegmentML(GraphicsPanel gp,int button, boolean reset)
	{
		this.gp = gp;
		this.gw = gp.gw;
		this.mesh = gw.mesh;
		this.button = button;
		this.reset = reset;
		gp.removeAllMouseMotionListeners();
		//System.out.println("AddSegmentML");
	}	

	public void mousePressed(MouseEvent event)
	{
		UndoAction undoAction;
		if (button == event.getButton())
		{
			mesh.prepareUndoableTransform();
			mesh.startUndoRecorder("add segment");
			Point mouse = event.getPoint();
			gp.setPopupEnable(false);
			////System.out.println(mouseX + "/" + mouseY);
			
			
			float mouseX = event.getX();
			float mouseY = event.getY();
			Point3f c = new Point3f(gp.get3dCoords(mouseX,mouseY));
			if (!gp.snapToClosestPoint(c,8,null)) if (gw.snapToGrid) gp.snapToGrid(c);;
			gp.view.transform(c);
			mouseX = c.x;
			mouseY = c.y;
			ControlPoint closestPoint = gp.getClosestPoint(new Point2D.Float(mouseX,mouseY),8);
			
			//ControlPoint closestPoint = gp.getClosestPoint(mouse,8);
			//float mouseX = mouse.x;
			//float mouseY = mouse.y;
			////System.out.println("+++" + closestPoint);
			/**
			* start a new curve
			**/
			if (closestPoint == null)
			{
				mode = NEW;
				Vector3f coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
				//if (gp.snapToGrid) gp.snapToGrid(c);
				//Vector3f coord = new Vector3f(c);
				cp1 = new ControlPoint(coord,gw.peak,gw.activeLayer);
				coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
				cp2 = new ControlPoint(coord,gw.peak,gw.activeLayer);
				curve = new Curve();
				curve.add(cp1);
				curve.add(cp2);
				curve.validate();
				mesh.recordAtomicAction(new NewCurveAtomicAction(mesh,curve));
				mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp1));
				mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp2));
				mesh.add(curve);
				motionListener = new AddSegmentMML(gp,curve,cp2,cp1.position);
				gp.addMouseMotionListener(motionListener);
				weldListener = new WeldML(gp,MouseEvent.BUTTON3,cp2,GraphicsWindow.ADD_SEGMENT);
				gp.addMouseListener(weldListener);
			}
			else
			check:
			{
				curve = closestPoint.curve;
				if (curve == null)
				{
					//System.out.println("curve == null!!!!!!");
					mesh.validate();
				}
				int pointID = closestPoint.pointOnCurve;
				/**
				* append to end of existing curve
				**/
				if (pointID == curve.size()-1 && !curve.closed)
				{
					mode = ADD_END;
					Vector3f coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
					//cp1 = new ControlPoint(coord);
					//coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
					cp2 = new ControlPoint(coord,gw.peak,gw.activeLayer);
					curve.add(cp2);
					mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp2));
					//curve.validate();
					motionListener = new AddSegmentMML(gp,curve,cp2,closestPoint.position);
					gp.addMouseMotionListener(motionListener);
					weldListener = new WeldML(gp,MouseEvent.BUTTON3,cp2,GraphicsWindow.ADD_SEGMENT);
					gp.addMouseListener(weldListener);
					break check;
				}
				
				/**
				* append to start of existing curve
				**/
				if (pointID == 0 && !curve.closed)
				{
					mode = ADD_START;
					Vector3f coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
					//cp1 = new ControlPoint(coord);
					//coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
					cp2 = new ControlPoint(coord,gw.peak,gw.activeLayer);
					curve.add(0,cp2);
					mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp2));
					//curve.validate();
					motionListener = new AddSegmentMML(gp,curve,cp2,closestPoint.position);
					gp.addMouseMotionListener(motionListener);
					weldListener = new WeldML(gp,MouseEvent.BUTTON3,cp2,GraphicsWindow.ADD_SEGMENT);
					gp.addMouseListener(weldListener);
					break check;
				}
				
				/**
				* start a new curve welded to this curve
				**/
				if (curve.closed || (pointID != 0 && pointID != curve.size()-1))
				{
					mode = NEW;
					//Vector3f coord = new Vector3f(gp.get3dCoords(mouseX,mouseY));
					ControlPoint cp = closestPoint;
					cp1 = new ControlPoint(cp.position,gw.peak,gw.activeLayer);
					Vector3f coord = new Vector3f(cp1.position);
					cp2 = new ControlPoint(coord,gw.peak,gw.activeLayer);
					curve = new Curve();
					curve.add(cp1);
					curve.add(cp2);
					mesh.recordAtomicAction(new NewCurveAtomicAction(mesh,curve));
					mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp1));
					mesh.recordAtomicAction(new NewControlPointAtomicAction(curve,cp2));
					//curve.validate();
					mesh.add(curve);
					mesh.recordAtomicAction(new WeldUnweldControlPointAtomicAction(cp,cp.weldedTo,cp1));
					cp.weldedTo = cp1;
					mesh.validate();
					motionListener = new AddSegmentMML(gp,curve,cp2,cp1.position);
					gp.addMouseMotionListener(motionListener);
					weldListener = new WeldML(gp,MouseEvent.BUTTON3,cp2,GraphicsWindow.ADD_SEGMENT);
					gp.addMouseListener(weldListener);
					break check;
				}
			}
			gw.repaint();
		}
	}

	public void mouseReleased(MouseEvent event)
	{
		if (button == event.getButton())
		{
			/*
			if (cp1.position.equals(cp2.position))
			{
				switch(mode)
				{
				case NEW:
					mesh.remove(mesh.size()-1);
					//System.out.println("remove curve " + (mesh.size()-1));
					break;
				case ADD_START:
					//System.out.println("remove point 0");
					curve.remove(0);
					mesh.validate();
					break;
				case ADD_END:
					//System.out.println("remove point " + (curve.size()-1));
					curve.remove(curve.size()-1);
					mesh.validate();
					break;
				}
			}
			*/
			mesh.recordUndoableTransform();
			mesh.stopUndoRecorder();	
			gp.removeMouseMotionListener(motionListener);
			gp.removeMouseListener(weldListener);
			if(reset) gw.changeAction(GraphicsWindow.DEFAULT);
			gw.repaint();
			//gw.editMeshToolBar.selectPointButton.setSelected(true);
			gp.setPopupEnable(true);
		}
	}
	
	class AddSegmentMML
	extends MouseMotionAdapter
	{
		protected GraphicsPanel gp;
		protected Curve curve;
		protected ControlPoint cp;
		protected Vector3f A;
		
		public AddSegmentMML(GraphicsPanel gp, Curve curve, ControlPoint cp, Vector3f A)
		{
			this.gp = gp;
			this.curve = curve;
			this.cp = cp;
			this.A = A;
		}

		public void mouseDragged(MouseEvent event)
		{
			float mouseX = event.getX();
			float mouseY = event.getY();
			Point3f coord = new Point3f(gp.get3dCoords(mouseX,mouseY));
			if (!gp.snapToClosestPoint(coord,8,cp)) if (gw.snapToGrid) gp.snapToGrid(coord);
			if (gw.xLock) coord.x = A.x;
			if (gw.yLock) coord.y = A.y;
			if (gw.zLock) coord.z = A.z;
			cp.position.set(coord);
			curve.validate();
			gp.gw.repaint();
		}
	}	
}