package jpatch;

import java.util.*;
import javax.vecmath.*;

public class BezierTriangle
{
	Point3f p003;
	Point3f p102;
	Point3f p201;
	Point3f p300;
	Point3f p210;
	Point3f p120;
	Point3f p030;
	Point3f p021;
	Point3f p012;
	Point3f p111;
	
	Point3f[] cp = new Point3f[25];
	
	float r = 1f/3f;
	
	public BezierTriangle(Point3f p003, Point3f p102, Point3f p201, Point3f p300, Point3f p210, Point3f p120, Point3f p030, Point3f p021, Point3f p012, Point3f p111)
	{
		this.p003 = p003;
		this.p102 = p102;
		this.p201 = p201;
		this.p300 = p300;
		this.p210 = p210;
		this.p120 = p120;
		this.p030 = p030;
		this.p021 = p021;
		this.p012 = p012;
		this.p111 = p111;
	}
	
	public BezierTriangle(Point3f p003, Point3f p102, Point3f p201, Point3f p300, Point3f p210, Point3f p120, Point3f p030, Point3f p021, Point3f p012)
	{
		this(p003,p102,p201,p300,p210,p120,p030,p021,p012,new Point3f(0,0,0));
		setCentroid();
	}
	
	public Point3f computeCentroid()
	{
		Point3f p111 = new Point3f(0,0,0);
		p111.add(parallelogram(p003,p102,p012));
		p111.add(parallelogram(p030,p120,p021));
		p111.add(parallelogram(p300,p201,p210));
		p111.scale(r);
		return p111;
	}
	
	public void setCentroid()
	{
		p111.set(computeCentroid());
	}
	
	public Point3f parallelogram(Tuple3f a, Tuple3f b, Tuple3f c)
	{
		Point3f result = new Point3f(b);
		result.add(c);
		result.sub(a);
		return result;
	}
	
	public Point3f getPoint(float u, float v, float w)
	{
	 	float b300 = w*w*w;
	 	float b030 = u*u*u;
	 	float b003 = v*v*v;
	 	float b210 = 3*w*w*u;
	 	float b120 = 3*w*u*u;
	 	float b201 = 3*w*w*v;
	 	float b021 = 3*u*u*v;
	 	float b102 = 3*w*v*v;
	 	float b012 = 3*u*v*v;
	 	float b111 = 6*w*u*v;
	 	
	 	float x = p300.x*b300 + p030.x*b030 + p003.x*b003 + p210.x*b210 + p120.x*b120 + p201.x*b201 + p021.x*b021 + p102.x*b102 + p012.x*b012 + p111.x*b111;
	 	float y = p300.y*b300 + p030.y*b030 + p003.y*b003 + p210.y*b210 + p120.y*b120 + p201.y*b201 + p021.y*b021 + p102.y*b102 + p012.y*b012 + p111.y*b111;
	 	float z = p300.z*b300 + p030.z*b030 + p003.z*b003 + p210.z*b210 + p120.z*b120 + p201.z*b201 + p021.z*b021 + p102.z*b102 + p012.z*b012 + p111.z*b111;
	 	
	 	return new Point3f(x,y,z);
	}
	
	public Vector3f getNormal(float u, float v, float w)
	{
		Vector3f du = getPoint_du(u,v,w);
		Vector3f dv = getPoint_dv(u,v,w);
		Vector3f n = new Vector3f();
		n.cross(du,dv);
		n.normalize();
		return n;
	}
	
	public Vector3f getPoint_du(float u, float v, float w)
	{
		float dw = -1;
		float dw2 = -2 + 2*v + 2*u;
		float dw3 = -3 + 6*v - 6*u*v - 3*v*v + 6*u - 3*u*u;
		
	 	float b300 = dw3;
	 	float b030 = 2*u*u;
	 	float b003 = 0;
	 	float b210 = 3*(1 - 4*u - 2*v + 4*u*v + 3*u*u + v*v);
	 	float b120 = 3*(2*u - 3*u*u - 2*u*v);
	 	float b201 = 3*dw2*v;
	 	float b021 = 6*u*v;
	 	float b102 = 3*dw*v*v;
	 	float b012 = 3*v*v;
	 	float b111 = 6*(v - 2*u*v - v*v);
	 	
	 	float x = p300.x*b300 + p030.x*b030 + p003.x*b003 + p210.x*b210 + p120.x*b120 + p201.x*b201 + p021.x*b021 + p102.x*b102 + p012.x*b012 + p111.x*b111;
	 	float y = p300.y*b300 + p030.y*b030 + p003.y*b003 + p210.y*b210 + p120.y*b120 + p201.y*b201 + p021.y*b021 + p102.y*b102 + p012.y*b012 + p111.y*b111;
	 	float z = p300.z*b300 + p030.z*b030 + p003.z*b003 + p210.z*b210 + p120.z*b120 + p201.z*b201 + p021.z*b021 + p102.z*b102 + p012.z*b012 + p111.z*b111;
	 	
	 	return new Vector3f(x,y,z);
	}
	
	public Vector3f getPoint_dv(float u, float v, float w)
	{
		float dw = -1;
		float dw2 = -2 + 2*v + 2*u;
		float dw3 = -3 + 6*v - 6*u*v - 3*v*v + 6*u - 3*u*u;
		
	 	float b300 = dw3;
	 	float b030 = 0;
	 	float b003 = 2*v*v;
	 	float b210 = 3*dw2*u;
	 	float b120 = 3*dw*u*u;
	 	float b201 = 3*(1 - 4*v - 2*u + 4*u*v + 3*v*v + u*u);
	 	float b021 = 3*u*u;
	 	float b102 = 3*(2*v - 3*v*v - 2*u*v);
	 	float b012 = 6*u*v;
	 	float b111 = 6*(u - 2*u*v - u*u);
	 	
	 	float x = p300.x*b300 + p030.x*b030 + p003.x*b003 + p210.x*b210 + p120.x*b120 + p201.x*b201 + p021.x*b021 + p102.x*b102 + p012.x*b012 + p111.x*b111;
	 	float y = p300.y*b300 + p030.y*b030 + p003.y*b003 + p210.y*b210 + p120.y*b120 + p201.y*b201 + p021.y*b021 + p102.y*b102 + p012.y*b012 + p111.y*b111;
	 	float z = p300.z*b300 + p030.z*b030 + p003.z*b003 + p210.z*b210 + p120.z*b120 + p201.z*b201 + p021.z*b021 + p102.z*b102 + p012.z*b012 + p111.z*b111;
	 	
	 	return new Vector3f(x,y,z);
	}	
	
	public Point3f midPoint(Tuple3f a, Tuple3f b)
	{
		Point3f m = new Point3f(a);
		m.add(b);
		m.scale(0.5f);
		return m;
	}
	
	public void deCasteljau(ArrayList points)
	{
		Point3f p0 = (Point3f)points.get(0);
		Point3f p1 = (Point3f)points.get(1);
		Point3f p2 = (Point3f)points.get(2);
		Point3f p3 = (Point3f)points.get(3);
		Point3f p4 = midPoint(p0,p1);
		Point3f p5 = midPoint(p1,p2);
		Point3f p6 = midPoint(p2,p3);
		Point3f p7 = midPoint(p4,p5);
		Point3f p8 = midPoint(p5,p6);
		Point3f p9 = midPoint(p7,p8);
		points.add(p4);
		points.add(p5);
		points.add(p6);
		points.add(p7);
		points.add(p8);
		points.add(p9);
	}
	
	public void compute()
	{
		ArrayList a = new ArrayList();
		ArrayList b = new ArrayList();
		ArrayList c = new ArrayList();
		a.add(p003);
		a.add(p102);
		a.add(p201);
		a.add(p300);
		deCasteljau(a);
		b.add(p300);
		b.add(p210);
		b.add(p120);
		b.add(p030);
		deCasteljau(b);
		c.add(p030);
		c.add(p021);
		c.add(p012);
		c.add(p003);
		deCasteljau(c);
		cp[0] = (Point3f)a.get(0);
		cp[1] = (Point3f)a.get(4);
		cp[2] = (Point3f)a.get(7);
		cp[3] = (Point3f)a.get(9);
		cp[4] = (Point3f)a.get(8);
		cp[5] = (Point3f)a.get(6);
		cp[6] = (Point3f)a.get(3);
		cp[7] = (Point3f)b.get(4);
		cp[8] = (Point3f)b.get(7);
		cp[9] = (Point3f)b.get(9);
		cp[10] = (Point3f)b.get(8);
		cp[11] = (Point3f)b.get(6);
		cp[12] = (Point3f)b.get(3);
		cp[13] = (Point3f)c.get(4);
		cp[14] = (Point3f)c.get(7);
		cp[15] = (Point3f)c.get(9);
		cp[16] = (Point3f)c.get(8);
		cp[17] = (Point3f)c.get(6);
		
		Point3f center = getPoint(1f/3f,1f/3f,1f/3f);
		Point3f center_u = getPoint(0f,0.5f,0.5f);
		Point3f center_v = getPoint(0.5f,0f,0.5f);
		Point3f center_w = getPoint(0.5f,0.5f,0f);
		
		float len_u = center.distance(center_u);
		float len_v = center.distance(center_v);
		float len_w = center.distance(center_w);
		
		Vector3f c_u = new Vector3f(center); c_u.sub(center_u);
		Vector3f c_v = new Vector3f(center); c_v.sub(center_v);
		Vector3f c_w = new Vector3f(center); c_w.sub(center_w);
		
		Vector3f normal_u = getNormal(0f,0.5f,0.5f);
		Vector3f normal_v = getNormal(0.5f,0f,0.5f);
		Vector3f normal_w = getNormal(0.5f,0.5f,0f);
		
		Vector3f normal_center = getNormal(1f/3f,1f/3f,1f/3f);
		
		Vector3f xx;
		
		xx = new Vector3f();
		
		xx.cross(c_u,normal_u);
		Vector3f uc = new Vector3f();
		uc.cross(normal_u,xx);
		uc.normalize();
		uc.scale(len_u*r);
		
		xx.cross(c_u,normal_center);
		Vector3f cu = new Vector3f();
		cu.cross(normal_center,xx);
		cu.normalize();
		cu.scale(-len_u*r);		
		
		Point3f up0 = new Point3f(center_u);
		Point3f up1 = new Point3f(up0);up1.add(uc);
		Point3f up3 = new Point3f(center);
		Point3f up2 = new Point3f(up3);up2.add(cu);
		
		xx.cross(c_v,normal_v);
		Vector3f vc = new Vector3f();
		vc.cross(normal_v,xx);
		vc.normalize();
		vc.scale(len_v*r);
		
		xx.cross(c_v,normal_center);
		Vector3f cv = new Vector3f();
		cv.cross(normal_center,xx);
		cv.normalize();
		cv.scale(-len_v*r);		
		
		Point3f vp0 = new Point3f(center_v);
		Point3f vp1 = new Point3f(vp0);vp1.add(vc);
		Point3f vp3 = new Point3f(center);
		Point3f vp2 = new Point3f(vp3);vp2.add(cv);
		
		xx.cross(c_w,normal_w);
		Vector3f wc = new Vector3f();
		wc.cross(normal_w,xx);
		wc.normalize();
		wc.scale(len_w*r);
		
		xx.cross(c_w,normal_center);
		Vector3f cw = new Vector3f();
		cw.cross(normal_center,xx);
		cw.normalize();
		cw.scale(-len_w*r);		
		
		Point3f wp0 = new Point3f(center_w);
		Point3f wp1 = new Point3f(wp0);wp1.add(wc);
		Point3f wp3 = new Point3f(center);
		Point3f wp2 = new Point3f(wp3);wp2.add(cw);
		
		cp[18] = center;
		cp[19] = vp2;
		cp[20] = vp1;
		cp[21] = wp2;
		cp[22] = wp1;
		cp[23] = up2;
		cp[24] = up1;
	}			
}
	
		
		
		
		
		
