/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import javax.vecmath.*;
import java.util.*;

/**The ControlPoint class defines a JPatch control point.*/
public class ControlPoint
implements Comparable
{	
	public Curve curve;
	public int pointOnCurve;
	public Vector3f reference = new Vector3f(0,0,0);
	public Vector3f position = new Vector3f(0,0,0);
	public Vector3f helperPosition = new Vector3f(0,0,0);
	public Point3f[] viewPosition = new Point3f[4];
	public boolean isSelected = false;
	public ControlPoint weldedTo = null;
	public boolean peak = false;
	public float inCurvature = 1f/3f;
	public float outCurvature = 1f/3f;
	public Vector3f inTangent = new Vector3f(0,0,0);
	public Vector3f outTangent = new Vector3f(0,0,0);
	public Point3f[] viewInTangent = new Point3f[4];
	public Point3f[] viewOutTangent = new Point3f[4];
	public int joint = -1;
	public float jointParentFactor = 0;
	public float jointPosition = 0;
	public ArrayList tail = new ArrayList();
	public int number;
	public int layer = 0;
	
	/*
	public ControlPoint(Vector3f position)
	{
		this.position.set(position);
		this.helperPosition.set(position);
		for (int v = 0; v < 4; v++)
		{
			viewPosition[v] = new Point3f();
			viewInTangent[v] = new Point3f();
			viewOutTangent[v] = new Point3f();
		}	
	}
	*/
	
	public ControlPoint(Vector3f position, boolean peak, int layer)
	{
		this.peak = peak;
		this.layer = layer;
		this.position.set(position);
		this.helperPosition.set(position);
		for (int v = 0; v < 4; v++)
		{
			viewPosition[v] = new Point3f();
			viewInTangent[v] = new Point3f();
			viewOutTangent[v] = new Point3f();
		}	
	}
	
	public ControlPoint(ControlPoint cp)
	{
		//this.curve = cp.curve;
		//this.pointOnCurve = cp.pointOnCurve;
		this.reference = new Vector3f(cp.reference);
		this.position = new Vector3f(cp.position);
		//this.helperPosition = new Vector3f(cp.helperPosition);
		//
		this.layer = cp.layer;
		this.isSelected = cp.isSelected;
		this.peak = cp.peak;
		this.inCurvature = cp.inCurvature;
		this.outCurvature = cp.outCurvature;
		for (int v = 0; v < 4; v++)
		{
			viewPosition[v] = new Point3f();
			viewInTangent[v] = new Point3f();
			viewOutTangent[v] = new Point3f();
		}
	}
		
	public ControlPoint getHead()
	{
		ControlPoint cp = this;
		if (weldedTo != null) cp = weldedTo.getHead();
		return cp;
	}
	
	public int compareTo(Object cp)
	{
		return number - ((ControlPoint)cp).number;
	}
	
	public boolean headSelected()
	{
		return (getHead()).isSelected;
	}
	
	public void validate()
	{
		assert weldedTo != this : "illegal self-referenct; controlpoint welded to itself";
		/*
		if (weldedTo == this) 
		{
			weldedTo = null;
			//System.out.println("WARNING: illgal self-reference removed");
		}
		*/
		this.position.set(getHead().position);
		this.inTangent.set(0,0,0);
		this.outTangent.set(0,0,0);
	}
	
	public void debug()
	{
		//System.out.println("curve: " + curve);
		//System.out.println("pointOnCurve: " + pointOnCurve);
		//System.out.println("position: " + position);
		//System.out.println("curve size: " + curve.size());
	}
	
	public ControlPoint clone(ControlPoint head)
	{
		ControlPoint cp = new ControlPoint(this);
		cp.weldedTo = head;
		return cp;
	}	 
}
