/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.util.*;
import javax.vecmath.*;

/**The Curve class defines a JPatch curve point.*/
public class Curve
extends ArrayList
{
	public boolean closed;
	public int layer = -1;
	protected Mesh mesh;
	
	public int prev(int index)
	{
		int result = index - 1;
		if (result < 0)
		{
			if (closed) result = size()-1;
			else result = -1;
		}
		return result;
	}
	
	public int next(int index)
	{
		int result = index + 1;
		if (result >= size())
		{
			if (closed) result = 0;
			else result = -1;
		}
		return result;
	}
	
	/*
	protected Vector3f getWeldedPos(int index)
	{
		ControlPoint thisCp = (ControlPoint)get(index);
		if (!thisCp.isWelded) return thisCp.position;
		else return getWeldedPos(thisCp.weldedTo);
	}
	*/
	
	public ControlPoint getNextControlPoint(ControlPoint cp)
	{
		int index = indexOf(cp);
		if (index == -1) return null;
		int next = next(index);
		if (next == -1) return null;
		else return (ControlPoint)get(next);
	}
	
	public ControlPoint getPrevControlPoint(ControlPoint cp)
	{
		int index = indexOf(cp);
		if (index == -1) return null;
		int prev = prev(index);
		if (prev == -1) return null;
		else return (ControlPoint)get(prev);
	}
	
	public ControlPoint getNextControlPoint(int index)
	{
		int next = next(index);
		if (next == -1) return null;
		else return (ControlPoint)get(next);
	}
	
	public ControlPoint getPrevControlPoint(int index)
	{
		int prev = prev(index);
		if (prev == -1) return null;
		else return (ControlPoint)get(prev);
	}
	
	public void add(ControlPoint cp)
	{
		super.add(cp);
		cp.curve = this;
		cp.pointOnCurve = this.size() - 1;
		validate();
	}
	
	public void add(int index, ControlPoint cp)
	{
		super.add(index,cp);
		cp.curve = this;
		cp.pointOnCurve = this.size() - 1;
		validate();
	}
	
	public CurveRemoveControlPointAtomicAction remove(ControlPoint cp)
	{
		int index = indexOf(cp);
		CurveRemoveControlPointAtomicAction atomicAction = new CurveRemoveControlPointAtomicAction(this,cp,index);
		super.remove(cp);
		return atomicAction;
	}
		
	/*
	public boolean remove(ControlPoint cp)
	{
		super.remove(cp);
		if (size() < 2) 
		{
			for (int p = 0; p < size(); p++)
			{
				ControlPoint cp2 = (ControlPoint)get(p);
				mesh.removeAllCP(cp2);
			}	
		}
		else validate();
		//cp = null;
		return true;
	}
	*/
	/*
	public Curve cloneCurve()
	{
		Curve clonedCurve = new Curve();
		for (int cp = 0; cp < size(); cp++)
		{
			ControlPoint thisCp = (ControlPoint)get(cp);
			ControlPoint clonedCp = new ControlPoint(thisCp.position);
			thisCp.isSelected = false;
			clonedCp.isSelected = true;
			clonedCurve.add(clonedCp);
		}
		clonedCurve.closed = closed;
		return clonedCurve;
	}
	*/
	/*
	public boolean isSelected()
	{
		boolean result = true;
		for (int cp = 0; cp < size(); cp++)
		{
			ControlPoint thisCp = (ControlPoint)get(cp);
			if (!(thisCp.getHead().isSelected)) result = false;
		}
		return result;
	}
	*/
	
	public Vector3f midPoint(Vector3f A, Vector3f B)
	{
		Vector3f m = new Vector3f(A);
		m.add(B);
		m.scale(0.5f);
		return m;
	}
	
	public ControlPoint deCasteljau(ControlPoint A, ControlPoint B)
	{
		Vector3f p1 = new Vector3f(A.position);
		Vector3f p2 = new Vector3f(A.position);
		p2.add(A.outTangent);
		Vector3f p3 = new Vector3f(B.position);
		p3.add(B.inTangent);
		Vector3f p4 = new Vector3f(B.position);
		Vector3f l2 = midPoint(p1,p2);
		Vector3f h = midPoint(p2,p3);
		Vector3f r3 = midPoint(p3,p4);
		Vector3f l3 = midPoint(l2,h);
		Vector3f r2 = midPoint(h,r3);
		Vector3f c = midPoint(l3,r2);
		ControlPoint x = new ControlPoint(c,false,layer);
		return x;
	}
	
	public void insert(int index, boolean direction)
	{
		ControlPoint A;
		ControlPoint B;
		ControlPoint C;
		if (direction)
		{
			A = (ControlPoint)get(index);
			B = getNextControlPoint(index);
		}
		else
		{
			B = (ControlPoint)get(index);
			A = getPrevControlPoint(index);
		}
		if (A != null && B != null)
		{
			C = deCasteljau(A,B);
			if (direction) add(index + 1,C);
			else add(index,C);
		}
		validate();
	}
				
	public void validate()
	{
		/** assign positions to welded points */
		/*
		for (int cp = 0; cp < size(); cp++)
		{
			ControlPoint thisCp = (ControlPoint)get(cp);
			if (thisCp.isWelded) thisCp.position = getWeldedPos(thisCp.weldedTo);
		}
		*/
				
		/** calculate tangents */
		Vector3f dummy = new Vector3f();
		layer = -1;
		for (int cp = 0; cp < size(); cp++)
		{
			ControlPoint thisCp = (ControlPoint)get(cp);
			if (layer == -1 || layer > thisCp.layer) layer = thisCp.layer;
			thisCp.curve = this;
			thisCp.pointOnCurve = cp;
			if (prev(cp) == -1 && next(cp) > -1)
			{
				Vector3f nextCpPos = ((ControlPoint)get(next(cp))).position;
				thisCp.outTangent.set(nextCpPos);
				thisCp.outTangent.sub(thisCp.position);
				thisCp.outTangent.scale(thisCp.outCurvature);
				thisCp.inTangent.set(0,0,0);
			}
			if (next(cp) == -1 && prev(cp) > -1)
			{
				Vector3f prevCpPos = ((ControlPoint)get(prev(cp))).position;
				thisCp.inTangent.set(prevCpPos);
				thisCp.inTangent.sub(thisCp.position);
				thisCp.inTangent.scale(thisCp.inCurvature);
				thisCp.outTangent.set(0,0,0);
			}
			if (prev(cp) != -1 && next(cp) != -1)
			{
				Vector3f prevCpPos = ((ControlPoint)get(prev(cp))).position;
				Vector3f nextCpPos = ((ControlPoint)get(next(cp))).position;
				if (thisCp.peak)
				{
					thisCp.inTangent.set(prevCpPos);
					thisCp.inTangent.sub(thisCp.position);
					thisCp.inTangent.scale(thisCp.inCurvature);
					thisCp.outTangent.set(nextCpPos);
					thisCp.outTangent.sub(thisCp.position);
					thisCp.outTangent.scale(thisCp.outCurvature);
				}
				else
				{
					Vector3f tangent = new Vector3f(nextCpPos);
					tangent.sub(prevCpPos);
					if (tangent.length() != 0)
					{
						tangent.normalize();
						dummy.set(thisCp.position);
						dummy.sub(prevCpPos);
						float inLength = dummy.length();
						dummy.set(thisCp.position);
						dummy.sub(nextCpPos);
						float outLength = dummy.length();
						thisCp.inTangent.set(tangent);
						thisCp.outTangent.set(tangent);
						thisCp.inTangent.scale(-inLength*thisCp.inCurvature);
						thisCp.outTangent.scale(outLength*thisCp.outCurvature);
						if (dummy.length() == 0)
						{
							thisCp.inTangent.set(0,0,0);
							thisCp.outTangent.set(0,0,0);
						}
					}
					else
					{
						thisCp.inTangent.set(0,0,0);
						thisCp.outTangent.set(0,0,0);
					}
				}
			}
		}
	}
}
	