/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.vecmath.*;
//import java.net.*;

public class EditMeshToolBar
extends JToolBar
implements ActionListener, KeyListener
{
	public JToggleButton selectPointButton;
	protected JToggleButton magnetButton;
	protected JToggleButton addSegmentButton;
	protected JToggleButton addMultiButton;
	protected JToggleButton scaleButton;
	protected JToggleButton rotateButton;
	protected JToggleButton cpButton;
	protected JButton insertButton;
	protected JToggleButton peakButton;
	protected JToggleButton roundButton;
	protected JButton deleteButton;
	protected JButton detachButton;
	protected JButton deleteSegmentButton;
	protected JButton cloneButton;
	protected JButton extrudeButton;
	protected JButton latheButton;
	protected JFrame frame;

	protected ButtonGroup peakRoundButtonGroup;	
	protected GraphicsWindow graphicsWindow;
	
	public EditMeshToolBar(JFrame frame,GraphicsWindow graphicsWindow, ButtonGroup actionButtonGroup)
	{
		super("Control Panel");
		this.graphicsWindow = graphicsWindow;
		this.frame = frame;
		//ClassLoader loader = getClass().getClassLoader();
		//URL fileLocation = loader.getResource("images/selectmove.png");
		selectPointButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/selectmove.png")));
		selectPointButton.setSelected(true);
		magnetButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/magnet.png")));
		magnetButton.setEnabled(false);
		addSegmentButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/add.png")));
		addMultiButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/addmultiple.png")));
		scaleButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/scale.png")));
		rotateButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/rot.png")));
		cpButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/cp.png")));
		//cpButton.setEnabled(false);
		peakButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/peak.png")));
		roundButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/round.png")));
		roundButton.setSelected(true);
		deleteButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/delete.png")));
		detachButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/detach.png")));
		insertButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/insert.png")));
		deleteSegmentButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/deletesegment.png")));
		cloneButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/clone.png")));
		extrudeButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/extrude.png")));
		latheButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/lathe.png")));
		
		add(selectPointButton);
		add(magnetButton);
		add(addSegmentButton);
		add(addMultiButton);
		add(scaleButton);
		add(rotateButton);
		add(cpButton);
		addSeparator(new Dimension(10,10));
		add(deleteButton);
		add(detachButton);
		add(insertButton);
		add(deleteSegmentButton);
		addSeparator(new Dimension(10,10));
		add(peakButton);
		add(roundButton);
		addSeparator(new Dimension(10,10));
		add(cloneButton);
		add(extrudeButton);
		add(latheButton);
		
		selectPointButton.setActionCommand("selectMove");
		magnetButton.setActionCommand("magnetMove");
		addSegmentButton.setActionCommand("addSegment");
		addMultiButton.setActionCommand("addMulti");
		scaleButton.setActionCommand("scale");
		rotateButton.setActionCommand("rotate");
		cpButton.setActionCommand("cp");
		peakButton.setActionCommand("peak");
		roundButton.setActionCommand("round");
		deleteButton.setActionCommand("delete");
		detachButton.setActionCommand("detach");
		insertButton.setActionCommand("insert");
		deleteSegmentButton.setActionCommand("deleteSegment");
		cloneButton.setActionCommand("clone");
		extrudeButton.setActionCommand("extrude");
		latheButton.setActionCommand("lathe");
		
		//selectPointButton.setMnemonic(KeyEvent.VK_S);
		//addSegmentButton.setMnemonic(KeyEvent.VK_A);
		
		selectPointButton.addActionListener(this);
		magnetButton.addActionListener(this);
		addSegmentButton.addActionListener(this);
		addMultiButton.addActionListener(this);
		scaleButton.addActionListener(this);
		rotateButton.addActionListener(this);
		cpButton.addActionListener(this);
		peakButton.addActionListener(this);
		roundButton.addActionListener(this);
		deleteButton.addActionListener(this);
		detachButton.addActionListener(this);
		insertButton.addActionListener(this);
		deleteSegmentButton.addActionListener(this);
		cloneButton.addActionListener(this);
		extrudeButton.addActionListener(this);
		latheButton.addActionListener(this);
		
		selectPointButton.setToolTipText("select/move points [ ESC ]");
		magnetButton.setToolTipText("magnet move tool");
		scaleButton.setToolTipText("scale tool [ S ]");
		rotateButton.setToolTipText("rotate tool [ R ]");
		cpButton.setToolTipText("tangent tool/select curve [ T ]");
		peakButton.setToolTipText("peak [ P ]");
		roundButton.setToolTipText("round [ O ]");
		addSegmentButton.setToolTipText("add curve segment [ A ]");
		addMultiButton.setToolTipText("add multiple curve segments");
		deleteButton.setToolTipText("delete selected points [ DEL ]");
		detachButton.setToolTipText("detach selected points");
		insertButton.setToolTipText("insert point");
		deleteSegmentButton.setToolTipText("delete curve segment");
		cloneButton.setToolTipText("clone selected points [ C ]");
		extrudeButton.setToolTipText("extrude tool [ E ]");
		latheButton.setToolTipText("lathe tool [ L ]");
		
		//actionButtonGroup = new ButtonGroup();
		actionButtonGroup.add(selectPointButton);
		actionButtonGroup.add(magnetButton);
		actionButtonGroup.add(addSegmentButton);
		actionButtonGroup.add(addMultiButton);
		actionButtonGroup.add(scaleButton);
		actionButtonGroup.add(rotateButton);
		actionButtonGroup.add(cpButton);
		
		peakRoundButtonGroup = new ButtonGroup();
		peakRoundButtonGroup.add(peakButton);
		peakRoundButtonGroup.add(roundButton);
		
		graphicsWindow.changeAction(GraphicsWindow.SELECT_MOVE);
		//graphicsWindow.graphicsPanels[0].addKeyListener(this);
		//graphicsWindow.graphicsPanels[0].requestFocus();
		//System.out.println(KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner());
		selectPointButton.addKeyListener(this);
	}
	public void actionPerformed(ActionEvent event)
	{
		if (event.getActionCommand() == "selectMove") graphicsWindow.changeAction(GraphicsWindow.SELECT_MOVE);
		if (event.getActionCommand() == "addSegment") graphicsWindow.changeAction(GraphicsWindow.ADD_SEGMENT);
		if (event.getActionCommand() == "addMulti") graphicsWindow.changeAction(GraphicsWindow.ADD_MULTI);
		if (event.getActionCommand() == "scale") graphicsWindow.changeAction(GraphicsWindow.SCALE);
		if (event.getActionCommand() == "rotate") graphicsWindow.changeAction(GraphicsWindow.ROTATE);
		if (event.getActionCommand() == "delete")
		{
			graphicsWindow.mesh.deleteSelected();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "clone")
		{
			graphicsWindow.mesh.cloneSelected();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "lathe")
		{
			LatheOptions.Values latheOptions = LatheOptions.getOptions(frame);
			if(latheOptions.ok && latheOptions.num > 2 && latheOptions.curvature >= 0) graphicsWindow.mesh.latheSelected(new Point3f(0,0,0),latheOptions.num,latheOptions.curvature,latheOptions.peak);
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "extrude")
		{
			graphicsWindow.mesh.extrudeSelected();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "detach")
		{
			graphicsWindow.mesh.detachSelected();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "peak")
		{
			graphicsWindow.setPeakMode(true);
			graphicsWindow.mesh.peakSelected(true);
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "round")
		{
			graphicsWindow.setPeakMode(false);
			graphicsWindow.mesh.peakSelected(false);
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "insert")
		{
			graphicsWindow.mesh.insertPoint();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "deleteSegment")
		{
			graphicsWindow.mesh.deleteSegment();
			graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		if (event.getActionCommand() == "cp")
		{
			graphicsWindow.mesh.computeTails();
			graphicsWindow.mesh.unselectAll();
			graphicsWindow.changeAction(GraphicsWindow.TANGENT);
			//graphicsWindow.changeAction(GraphicsWindow.DEFAULT);
		}
		graphicsWindow.repaint();
		graphicsWindow.graphicsPanels[0].requestFocus();
	}
	public void keyPressed(KeyEvent event)
	{
		int key = event.getKeyCode();
		//System.out.println("keyPressed " + key);
		if (key == KeyEvent.VK_A)
		{
			addSegmentButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"addSegment"));
		}
		if (key == KeyEvent.VK_DELETE)
		{
			deleteButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"delete"));
		}
		if (key == KeyEvent.VK_L)
		{
			latheButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"lathe"));
		}
		if (key == KeyEvent.VK_E)
		{
			extrudeButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"extrude"));
		}
		if (key == KeyEvent.VK_C)
		{
			extrudeButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"clone"));
		}
		if (key == KeyEvent.VK_ESCAPE)
		{
			selectPointButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"selectMove"));
		}
		if (key == KeyEvent.VK_P)
		{
			peakButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"peak"));
		}
		if (key == KeyEvent.VK_O)
		{
			roundButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"round"));
		}
		if (key == KeyEvent.VK_S)
		{
			scaleButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"scale"));
		}
		if (key == KeyEvent.VK_R)
		{
			rotateButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"rotate"));
		}
		if (key == KeyEvent.VK_T)
		{
			cpButton.setSelected(true);
			actionPerformed(new ActionEvent(this,0,"cp"));
		}
		
		
	}
	public void keyTyped(KeyEvent event)
	{
	}
	public void keyReleased(KeyEvent event)
	{
	}	
}
