/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*@author developer on changes : Dan Bishop
*@version $Id: GraphicsPanel.java,v 1.10 2003/06/04 07:17:38 god64 Exp $ $Date: 2003/06/04 07:17:38 $
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.vecmath.*;
import java.awt.geom.*;
import java.util.*;
import java.io.File;
import javax.swing.filechooser.*;
import java.awt.image.*;
import java.lang.*;

public class GraphicsPanel
extends JPanel
{
	protected Image img;
	protected Image img2;
	protected Image backImage;
	public Mesh mesh;
	protected PopupML popup;
	Matrix4f view = new Matrix4f();
	protected Matrix4f iView = new Matrix4f();
	protected GraphicsWindow gw;
	protected int number;
	public ViewDefinition viewDef = new ViewDefinition(FRONT);
	public ViewDefinition[] viewDefs = new ViewDefinition[8];
	
	protected float windowScale = 1;
	
	public boolean showGrid = true;
	public boolean axisView = true;
	public boolean showCurves = true;
	public boolean showControlPoints = true;
	public float fRotoscopeX = 1.0f;
	public float fRotoscopeY = 1.0f;


	//public boolean snapToGrid = false;
	
	static final int FRONT = 1;
	static final int BACK = 2;
	static final int LEFT = 3;
	static final int RIGHT = 4;
	static final int TOP = 5;
	static final int BOTTOM = 6;
	static final int BIRDSEYE = 7;
	
	static final String[] viewNames = { "","front","back","left side","right side","top","bottom","bird's eye" };
	
	public int viewMode = FRONT;
	
	public float gridSpacing = 1;
	protected int display;
	
	protected SelectorRectangle selectorRectangle = new SelectorRectangle();

	private boolean pluginHasMouse=false;
	
	/*
	protected Image rotoscope;
	protected Image scaledRotoscope;
	protected int rotoscopeXoffset = 0;
	protected int rotoscopeYoffset = 0;
	protected float rotoscopeScale = 10f;
	protected int rotoscopePixelXoffset;
	protected int rotoscopePixelYoffset;
	protected float rotoscopePixelScale;
	protected boolean showRotoscope = true;	       
	*/
	public GraphicsPanel(Mesh mesh, int display, GraphicsWindow gw, int number)
	{
		//System.out.println("***");
		this.display = display;
		this.mesh = mesh;
		this.gw = gw;
		this.number = number;
		//add(new Label("test"));
		
		//setup();
		//view.setIdentity();
		////System.out.println("setup done\n" + view);
		//setLayout(new GridLayout(1,1));
		
		
		//setup();
		setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED,Config.border1Color,Config.border2Color,Config.border3Color,Config.border4Color));
		setBackground(Config.backgroundColor);
		
		addMouseListener(new SetCursorML(this,MouseEvent.BUTTON2));
		for (int v = 1; v < 8; v++)
		{
			viewDefs[v] = new ViewDefinition(v);
		}
		setPopupEnable(true);
		ImageIcon r = new ImageIcon("f:\\test.png");
		//rotoscope = r.getImage();
	}
	
	/*
	public void validateRotoscope()
	{
		if (showRotoscope)
		{
			float translateX = viewDef.translateX;
			float translateY = viewDef.translateY;
			//float rotateX = viewDef.rotateX;
			//float rotateY = viewDef.rotateY;
			float scale = viewDef.scale;
			float width = getWidth();
			float height = getHeight();
			float xcenter = width/2;
			float ycenter = height/2;
			float size = xcenter;
			if (ycenter < size) size = ycenter;
			
			windowScale = size * scale;
			rotoscopePixelScale = rotoscopeScale * windowScale;
			rotoscopePixelOffsetX = rotosceopOffsetX * windowScale + xcenter + viewDef.translateX;
			rotoscopePixelOffsetY = rotosceopOffsetY * windowScale + ycenter + viewDef.translateY;
			scaledRotoscope = rotoscope.getScaledInstance(rotoscope.getWidth(),300,Image.SCALE_DEFAULT);
	*/		
	public void removeAllMouseMotionListeners()
	{
		MouseMotionListener[] MMLs = getMouseMotionListeners();
		for (int m = 0; m < MMLs.length; m++)
		{
			removeMouseMotionListener(MMLs[m]);
		}
	}
	
	public void removeAllMouseListeners()
	{
		MouseListener[] MLs = getMouseListeners();
		for (int m = 0; m < MLs.length; m++)
		{
			removeMouseListener(MLs[m]);
		}
		//setPopupEnable(true);

		if (!pluginHasMouse) {
			addMouseListener(new SetCursorML(this,MouseEvent.BUTTON2));
			addMouseListener(new FocusML());
		}
	}
	
	void setPopupEnable(boolean pop)
	{
		if (pop)
		{
			removeMouseListener(popup);
			popup = new PopupML(this,MouseEvent.BUTTON3);
			addMouseListener(popup);
		}
		else
		{
			removeMouseListener(popup);
		}
	}
		
	void setView(int viewMode)
	{
		viewDefs[this.viewMode] = new ViewDefinition(viewDef);
		this.viewMode = viewMode;
		viewDef = new ViewDefinition(viewDefs[viewMode]);
		if (viewMode != BIRDSEYE) axisView = true;
		else axisView = false;
		//setup();
		//repaint();
	}

	class SelectorRectangle
	{
		float x1;
		float x2;
		float y1;
		float y2;
		boolean visible;
	}
		
	public class ViewDefinition
	{
		public float rotateX;
		public float rotateY;
		public float translateX = 0;
		public float translateY = 0;
		public float scale = 0.1f;
		
		ViewDefinition(ViewDefinition d)
		{
			this.rotateX = d.rotateX;
			this.rotateY = d.rotateY;
			this.translateX = d.translateX;
			this.translateY = d.translateY;
			this.scale = d.scale;
		}
		ViewDefinition(int v)
		{
			switch(v)
			{
				case FRONT:
					rotateX = 0;
					rotateY = 0;
					break;
				case BACK:
					rotateX = 0;
					rotateY = (float)Math.PI;
					break;
				case LEFT:
					rotateX = 0;
					rotateY = -(float)Math.PI/2f;
					break;
				case RIGHT:
					rotateX = 0;
					rotateY = (float)Math.PI/2f;
					break;
				case TOP:
					rotateX = -(float)Math.PI/2f;
					rotateY = 0;
					break;
				case BOTTOM:
					rotateX = (float)Math.PI/2f;
					rotateY = 0;
					break;
				case BIRDSEYE:
					rotateX = -(float)Math.PI/4;
					rotateY = (float)Math.PI/4;
				break;
			}
		}
	
	}

	public ControlPoint getClosestPoint(Point2D.Float target, float minDistance)
	{
		//Point2D.Float target = new Point2D.Float(mouse.x,mouse.y);
		//if (snapToGrid) minDistance = Math.max(minDistance,windowScale*gridSpacing*(float)Math.sqrt(2));
		
		float distance = minDistance*minDistance;
		ControlPoint result = null;
		for (int m = 0; m < mesh.size(); m++)
		{
			Curve curve = (Curve)mesh.get(m);
			for (int c = 0; c < curve.size(); c++)
			{
				ControlPoint cp = (ControlPoint)curve.get(c);
				if (cp.weldedTo == null && (cp.layer == gw.activeLayer || cp.isSelected))
				{
					Point2D.Float vertex = new Point2D.Float(cp.viewPosition[display].x,cp.viewPosition[display].y);
					float d = (float)vertex.distanceSq(target);
					if (cp.isSelected) d *= 0.99;
					////System.out.print(cp.position);
					////System.out.print(" " + d);
					if (d < distance)
					{
						distance = d;
						result = cp;
						//System.out.print("*");
					}
					////System.out.println();
				}
			}
		}
		////System.out.println();
		if (result != null)
		{
			try
			{
				Point mouse = new Point();
				Robot robot = new Robot();
				mouse.setLocation((int)result.viewPosition[display].x,(int)result.viewPosition[display].y);
				Point screenPos = new Point(mouse);
				SwingUtilities.convertPointToScreen(screenPos,this);
				if (result != null) robot.mouseMove(screenPos.x,screenPos.y);
				target.setLocation(mouse);
			}
			catch (AWTException x)
			{
			}
		}	
		/*
		if (result != null)
		{
			//System.out.println(result.position);
			//System.out.println(result.weldedTo);
		}
		*/	
		return result;
	}

	public ControlPoint getClosestPoint(Point2D.Float target, float minDistance, ControlPoint notCp)
	{
		//Point2D.Float target = new Point2D.Float(mouse.x,mouse.y);
		//if (snapToGrid) minDistance = Math.max(minDistance,windowScale*gridSpacing*(float)Math.sqrt(2));
		float distance = minDistance*minDistance;
		ControlPoint result = null;
		for (int m = 0; m < mesh.size(); m++)
		{
			Curve curve = (Curve)mesh.get(m);
			for (int c = 0; c < curve.size(); c++)
			{
				ControlPoint cp = (ControlPoint)curve.get(c);
				if (cp.weldedTo == null && cp != notCp && (cp.layer == gw.activeLayer || cp.isSelected))
				{
					Point2D.Float vertex = new Point2D.Float(cp.viewPosition[display].x,cp.viewPosition[display].y);
					float d = (float)vertex.distanceSq(target);
					if (cp.isSelected) d *= 0.99;
					if (d < distance)
					{
						distance = d;
						result = cp;
					}
				}
			}
		}
		/*
		if (result != null)
		{
			try
			{
				Robot robot = new Robot();
				mouse.setLocation((int)result.viewPosition[display].x,(int)result.viewPosition[display].y);
				Point screenPos = new Point(mouse);
				SwingUtilities.convertPointToScreen(screenPos,this);
				if (result != null) robot.mouseMove(screenPos.x,screenPos.y);
			}
			catch (AWTException x)
			{
			}
		}
		*/		
		return result;
	}
		
	public void selectRectangle(boolean clear)
	{
		float minx = selectorRectangle.x1;
		float miny = selectorRectangle.y1;
		float maxx = selectorRectangle.x2;
		float maxy = selectorRectangle.y2;
		if (minx > maxx)
		{
			maxx = selectorRectangle.x1;
			minx = selectorRectangle.x2;
		}
		if (miny > maxy)
		{
			maxy = selectorRectangle.y1;
			miny = selectorRectangle.y2;
		}
		for (int m = 0; m < mesh.size(); m++)
		{
			Curve curve = (Curve)mesh.get(m);
			for (int c = 0; c < curve.size(); c++)
			{
				ControlPoint cp = (ControlPoint)curve.get(c);
				if (cp.weldedTo == null && (gw.layerVisible(cp.layer) || gw.activeLayer == cp.layer))
				{
					float px = cp.viewPosition[display].x;
					float py = cp.viewPosition[display].y;
					if (px >= minx && px <= maxx && py >= miny && py <= maxy) cp.isSelected = !cp.isSelected;
					//else if (clear) cp.isSelected = false;
				}
			}
		}
	}
					
					
						
	public void clearSelector()
	{
		selectorRectangle.visible = false;
	}
	public void setSelectorA(float X, float Y)
	{
		selectorRectangle.x1 = X;
		selectorRectangle.y1 = Y;
		selectorRectangle.visible = false;
		//img = this.createVolatileImage(getWidth(),getHeight());
		//img2 = this.createVolatileImage(getWidth(),getHeight());
		//img = new.image()
		img = createVolatileImage(getWidth(),getHeight());
		img2 = createVolatileImage(getWidth(),getHeight());
		paintComponent(img.getGraphics());
		paintBorder(img.getGraphics());
	}
	
	public void setSelectorB(float X, float Y)
	{
		float rectX;
		float rectY;
		float height;
		float width;
		selectorRectangle.x2 = X;
		selectorRectangle.y2 = Y;
		Graphics g = getGraphics();
		Graphics i = img.getGraphics();
		Graphics i2 = img2.getGraphics();
		//g.drawImage(img,0,0,null);
		paintBorder(g);
		//g.setXORMode(Config.backgroundColor);
		//if (selectorRectangle.visible)
		//{
		if (selectorRectangle.x1 < selectorRectangle.x2) rectX = selectorRectangle.x1; else rectX = selectorRectangle.x2;
		if (selectorRectangle.y1 < selectorRectangle.y2) rectY = selectorRectangle.y1; else rectY = selectorRectangle.y2;
		width = Math.abs(selectorRectangle.x1 - selectorRectangle.x2);
		height = Math.abs(selectorRectangle.y1 - selectorRectangle.y2);
		
		selectorRectangle.visible = (width != 0 && height != 0);
		//if (selectorRectangle.visible) g.drawRect((int)rectX,(int)rectY,(int)width,(int)height);
		i2.drawImage(img,0,0,null);
		//i2.setColor(Config.selectorBGColor);
		//i2.fillRect((int)rectX,(int)rectY,(int)width,(int)height);
		i2.setColor(Config.selectorColor);
		i2.drawRect((int)rectX,(int)rectY,(int)width,(int)height);
		//i.setPaintMode();
		g.drawImage(img2,0,0,null);
		
		//}
	}	
	
	void snapToGrid(Point3f point)
	{
		if (viewMode != BIRDSEYE)
		{
			float x = point.x;
			float y = point.y;
			float z = point.z;
			x = Math.round(x/gridSpacing)*gridSpacing;
			y = Math.round(y/gridSpacing)*gridSpacing;
			z = Math.round(z/gridSpacing)*gridSpacing;
			point.set(x,y,z);
		}
	}
	
	boolean snapToClosestPoint(Point3f point, float dist, ControlPoint notCp)
	{
		Point3f viewPoint = new Point3f(point);
		view.transform(viewPoint);
		ControlPoint ncp = getClosestPoint(new Point2D.Float(viewPoint.x,viewPoint.y),dist,notCp);
		if (ncp != null)
		{
			point.set(ncp.position);
			return true;
		}
		else return false;
	}
	
	Point3f get3dCoords (float X, float Y)
	{
		Point3f c = new Point3f(gw.cursor);
		view.transform(c);
		Point3f n = new Point3f(X,Y,c.z);
		iView.set(view);
		iView.invert();
		iView.transform(n);
		return n;
	}
	
	
	/*
	public void setSelectorB(float X, float Y)
	{
		float rectX;
		float rectY;
		float height;
		float width;
		
		Graphics g = getGraphics();
		g.setColor(Config.selectorColor);
		g.setXORMode(Config.backgroundColor);
		if (selectorRectangle.visible)
		{
			if (selectorRectangle.x1 < selectorRectangle.x2) rectX = selectorRectangle.x1; else rectX = selectorRectangle.x2;
			if (selectorRectangle.y1 < selectorRectangle.y2) rectY = selectorRectangle.y1; else rectY = selectorRectangle.y2;
			width = Math.abs(selectorRectangle.x1 - selectorRectangle.x2);
			height = Math.abs(selectorRectangle.y1 - selectorRectangle.y2);
			g.drawRect((int)rectX,(int)rectY,(int)width,(int)height);
		}
			
		selectorRectangle.x2 = X;
		selectorRectangle.y2 = Y;
		
		if (selectorRectangle.x1 < selectorRectangle.x2) rectX = selectorRectangle.x1; else rectX = selectorRectangle.x2;
		if (selectorRectangle.y1 < selectorRectangle.y2) rectY = selectorRectangle.y1; else rectY = selectorRectangle.y2;
		width = Math.abs(selectorRectangle.x1 - selectorRectangle.x2);
		height = Math.abs(selectorRectangle.y1 - selectorRectangle.y2);
		
		g.drawRect((int)rectX,(int)rectY,(int)width,(int)height);	
		selectorRectangle.visible = true;
	}	
	*/
	
	protected void paintComponent(Graphics g)
	{	
		super.paintComponent(g);
		float translateX = viewDef.translateX;
		float translateY = viewDef.translateY;
		//float rotateX = viewDef.rotateX;
		//float rotateY = viewDef.rotateY;
		float scale = viewDef.scale;
		float width = getWidth();
		float height = getHeight();
		float xcenter = width/2;
		float ycenter = height/2;
		float size = xcenter;
		if (ycenter < size) size = ycenter;
		
		windowScale = size * scale;
		//view.setIdentity();
		
		Matrix4f transform = new Matrix4f();
		/*
		transform.setIdentity();
		transform.setScale(windowScale);
		view.mul(transform);
		transform.setIdentity();
		transform.setTranslation(new Vector3f(translateX + xcenter/windowScale,translateY + ycenter/windowScale,0));
		view.mul(transform);
		transform.setIdentity();
		transform.rotX(rotateX);
		view.mul(transform);
		transform.setIdentity();
		transform.setRotation(new AxisAngle4f(0,1,0,rotateY));
		view.mul(transform);
		*/
		switch(viewMode)
		{
			case FRONT:
				view.set(new Matrix4f( 1, 0, 0, 0,
						       0,-1, 0, 0,
						       0, 0, 1, 0,
						       0, 0, 0, 1));
				//viewDef.rotateX = 0;
				//viewDef.rotateY = 0;
				break;
			case BACK:
				view.set(new Matrix4f(-1, 0, 0, 0,
						       0,-1, 0, 0,
						       0, 0,-1, 0,
						       0, 0, 0, 1));
				//viewDef.rotateX = 0;
				//viewDef.rotateY = (float)Math.PI;
				break;
			case LEFT:
				view.set(new Matrix4f( 0, 0,-1, 0,
						       0,-1, 0, 0,
						       1, 0, 0, 0,
						       0, 0, 0, 1));
				//viewDef.rotateX = 0;
				//viewDef.rotateY = -(float)Math.PI/2f;
				break;
			case RIGHT:
				view.set(new Matrix4f( 0, 0, 1, 0,
			 			       0,-1, 0, 0,
						      -1, 0, 0, 0,
						       0, 0, 0, 1));
				//viewDef.rotateX = 0;
				//viewDef.rotateY = (float)Math.PI/2f;
				break;
			case TOP:
				view.set(new Matrix4f( 1, 0, 0, 0,
						       0, 0,-1, 0,
						       0,-1, 0, 0,
						       0, 0, 0, 1));
				//rotateX = (float)Math.PI/2f;
				//rotateY = 0;
				break;
			case BOTTOM:
				view.set(new Matrix4f( 1, 0, 0, 0,
						       0, 0, 1, 0,
						       0, 1, 0, 0,
						       0, 0, 0, 1));
				//rotateX = -(float)Math.PI/2f;
				//rotateY = 0;
				break;
			case BIRDSEYE:
				view.set(new Matrix4f( 1, 0, 0, 0,
						       0,-1, 0, 0,
						       0, 0, 1, 0,
						       0, 0, 0, 1));
				transform.rotX(viewDef.rotateX);
				view.mul(transform);
				transform.rotY(viewDef.rotateY);
				view.mul(transform);
			break;
		}
		view.setScale(windowScale);
		view.setTranslation(new Vector3f(translateX*windowScale + xcenter,translateY*windowScale + ycenter,0));
		
		
		Graphics2D g2 = (Graphics2D)g;
		
		// Display the background image
		if( gw.backgroundInfo[viewMode].backgroundImage != null)
		{
			if(gw.backgroundInfo[viewMode].displayBackgroundImage)
			{
				//Change to allow image to zoom and translate. Made by Dan Bishop
				//Added the Affine Transformation
				//Mar 30,2003
				float imgWidth = gw.backgroundInfo[viewMode].fWidth;
				float imgHeight = gw.backgroundInfo[viewMode].fHeight;
				float ftotalScale = windowScale*gw.backgroundInfo[viewMode].fscale;
				float frotoXoffset = gw.backgroundInfo[viewMode].frotoscopeXoffset;
				float frotoYoffset = gw.backgroundInfo[viewMode].frotoscopeYoffset;
				
				AffineTransform imgTransform = new AffineTransform(
					ftotalScale*0.05,0,0,ftotalScale*0.05,translateX*windowScale + frotoXoffset*windowScale + xcenter - ftotalScale*.05*imgWidth/2 
					,translateY*windowScale + frotoYoffset*windowScale + ycenter - ftotalScale*.05*imgHeight/2);
				g2.drawImage(gw.backgroundInfo[viewMode].backgroundImage,imgTransform,this);
			}
		}
			
		g2.setColor(new Color(255,255,0));
		/**
		* set to true if you want to have normals and patches displayed in edit-window
		**/
		if (false)
		{
			ArrayList polys = new ArrayList();
			for (int p = 0; p < mesh.patches.size(); p++)
			{
				Patch patch = (Patch)mesh.patches.get(p);
				ArrayList bcpl = patch.getBicubicPatches();
				for (int b = 0; b < bcpl.size(); b++)
				{
					Vector3f[] cps = (Vector3f[])bcpl.get(b);
					//Point3f displayCP = new Point3f();
					/*
					for (int c = 0; c < 16; c++)
					{
						displayCP.set(cps[c]);
						//System.out.println(displayCP);
						view.transform(displayCP);
						//System.out.println(displayCP);
						g2.fillRect((int)displayCP.x - 1, (int)displayCP.y - 1,3,3);
					}*/
					int uSteps = 4;
					int vSteps = 4;
				
					int[][] posX = new int[uSteps + 1][vSteps + 1];
					int[][] posY = new int[uSteps + 1][vSteps + 1];
					float[][] posZ = new float[uSteps + 1][vSteps + 1];
					
					g2.setColor(new Color(255,255,255));
					ArrayList bicubicPatches = patch.getBicubicPatches();
					for (int bp = 0; bp < bicubicPatches.size(); bp++)
					{
						Vector3f[] bcp = (Vector3f[])bicubicPatches.get(bp);
						Point3f displayCP = new Point3f();
						for (int c = 0; c < 16; c++)
						{
							displayCP.set(bcp[c]);
							view.transform(displayCP);
							g2.fillRect((int)displayCP.x-1,(int)displayCP.y-1,3,3);
						}
					}
					
					for (int u = 0; u <= uSteps; u++)
					{
						float uu = (float)u/(float)uSteps;
						for (int v = 0; v <= vSteps; v++)
						{
							float vv = (float)v/(float)vSteps;
							Point3f displayCP = patch.getPoint(uu,vv,cps);
							Vector3f normal = patch.getNormal(uu,vv,cps);
							Point3f normalCP = new Point3f(displayCP);
							normalCP.add(normal);
							view.transform(displayCP);
							view.transform(normalCP);
							posX[u][v] = (int)displayCP.x;
							posY[u][v] = (int)displayCP.y;
							posZ[u][v] = displayCP.z;
							g2.drawLine((int)displayCP.x,(int)displayCP.y,(int)normalCP.x,(int)normalCP.y);
							//g2.drawLine((int)displayCP.x,(int)displayCP.y,(int)displayCP.x,(int)displayCP.y);
							//g2.fillRect((int)displayCP.x - 1, (int)displayCP.y - 1,3,3);
						}
					}
					Color color = Config.layerColors[patch.getLayer()].getColor();
					for (int u = 0; u < uSteps; u++)
					{
						for (int v = 0; v < vSteps; v++)
						{
							int[] xPoints = new int[] { posX[u][v], posX[u + 1][v], posX[u + 1][v + 1], posX[u][v + 1] };
							int[] yPoints = new int[] { posY[u][v], posY[u + 1][v], posY[u + 1][v + 1], posY[u][v + 1] };
							ZBufferPolygon zbp = new ZBufferPolygon(xPoints,yPoints,4,posZ[u][v],color);
							polys.add(zbp);
						}
					}
							
							
							
				}
			}
			/*
			Object[] zbps = polys.toArray();
			Arrays.sort(zbps);
			for (int p = 0; p < zbps.length; p++)
			{
				ZBufferPolygon zbp = (ZBufferPolygon)zbps[p];
				g2.setColor(zbp.color);
				//g2.fillPolygon(zbp);
			}
			*/
		}
		
		//System.out.println(view);        
		//System.out.println(KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner());
		
		
		//g2.drawImage(scaledRotoscope,0,0,null);
		
		if (showGrid && axisView)
		{
			float gridScreenSpacing = windowScale*gridSpacing;
			float dx = translateX*windowScale;
			float dy = translateY*windowScale;
			//while (gridScreenSpacing < 8) gridScreenSpacing *= 5;
			
			if (gridScreenSpacing > 5)
			{
				
				int start = Math.round((- dx - xcenter)/gridScreenSpacing);
				int end = Math.round((- dx + xcenter)/gridScreenSpacing);
				for (int x = start; x <= end; x++)
				{
					if (x % 5 == 0) g.setColor(Config.gridColor);
					else g.setColor(Config.minorGridColor);
					g.drawLine((int)(xcenter + x*gridScreenSpacing + dx),0,(int)(xcenter + x*gridScreenSpacing + dx),(int)height);
				}
				start = Math.round((- dy - ycenter)/gridScreenSpacing);
				end = Math.round((- dy + ycenter)/gridScreenSpacing);
				for (int y = start; y <= end; y++)
				{
					if (y % 5 == 0) g.setColor(Config.gridColor);
					else g.setColor(Config.minorGridColor);
					g.drawLine(0,(int)(ycenter + y*gridScreenSpacing + dy),(int)width,(int)(ycenter + y*gridScreenSpacing + dy));
				}
				//g.setColor(Config.gridZeroColor);
				g2.setColor(Config.gridColor);
				g2.drawLine((int)(xcenter + dx - 1),0,(int)(xcenter + dx - 1),(int)(height));
				g2.drawLine((int)(xcenter + dx + 1),0,(int)(xcenter + dx + 1),(int)(height));
				g2.drawLine(0,(int)(ycenter + dy - 1),(int)width,(int)(ycenter + dy - 1));
				g2.drawLine(0,(int)(ycenter + dy + 1),(int)width,(int)(ycenter + dy + 1));
			}
			g2.setColor(Config.gridColor);
			g2.drawLine((int)(xcenter + dx),0,(int)(xcenter + dx),(int)(height));
			g2.drawLine(0,(int)(ycenter + dy),(int)width,(int)(ycenter + dy));
		}
		ControlPoint cp;
		ControlPoint cpB;
		for (int m = 0; m < mesh.size(); m++)
		{
			Curve curve = (Curve)mesh.get(m);
			for (int c = 0; c < curve.size(); c++)
			{
				cp = (ControlPoint)curve.get(c);
				//if (!cp.isWelded)
				//{
					cp.viewPosition[display].set(cp.position);
					cp.viewInTangent[display].set(cp.inTangent);
					cp.viewOutTangent[display].set(cp.outTangent);
					cp.viewInTangent[display].add(cp.viewPosition[display]);
					cp.viewOutTangent[display].add(cp.viewPosition[display]);
					view.transform(cp.viewPosition[display]);
					view.transform(cp.viewInTangent[display]);
					view.transform(cp.viewOutTangent[display]);
				//}
			}
			/*
			for (int c = 0; c < curve.size(); c++)
			{
				cp = (ControlPoint)curve.get(c);
				if (cp.isWelded)
				{
					//Curve wCurve = (Curve)mesh.get(cp.weldedToLastCurve);
					//ControlPoint wCp = (ControlPoint)curve.get(cp.weldedToLastPoint);
					ControlPoint wCp = mesh.getHeadCP(cp);
					cp.viewPosition[display].set(wCp.viewPosition[display]);
					cp.viewInTangent[display].set(wCp.viewInTangent[display]);
					cp.viewOutTangent[display].set(wCp.viewOutTangent[display]);
				}
			}
			*/
			if (showCurves)
			{
				GeneralPath bezierCurve = new GeneralPath();
				if (gw.visibleLayers[curve.layer] || gw.activeLayer == curve.layer)
				{
					if (gw.activeLayer == curve.layer) g2.setColor(Config.activeCurveColor);
					else g2.setColor(Config.curveColor);
					cp = (ControlPoint)curve.get(0);
					bezierCurve.moveTo(cp.viewPosition[display].x,cp.viewPosition[display].y);
					for (int c = 1; c < curve.size(); c++)
					{
						cpB = cp;
						cp = (ControlPoint)curve.get(c);
						bezierCurve.curveTo(cpB.viewOutTangent[display].x,cpB.viewOutTangent[display].y,cp.viewInTangent[display].x,cp.viewInTangent[display].y,cp.viewPosition[display].x,cp.viewPosition[display].y);
					}
					if (curve.closed)
					{
						cpB = cp;
						cp = (ControlPoint)curve.get(0);
						bezierCurve.curveTo(cpB.viewOutTangent[display].x,cpB.viewOutTangent[display].y,cp.viewInTangent[display].x,cp.viewInTangent[display].y,cp.viewPosition[display].x,cp.viewPosition[display].y);
					}
					g2.draw(bezierCurve);
				}
				
			}
			if (showControlPoints)
			{
				g2.setColor(Config.controlPointColor);
				for (int c = 0; c < curve.size(); c++)
				{
					cp = (ControlPoint)curve.get(c);
					if (cp.layer == gw.activeLayer && cp.weldedTo == null)
					{
						if (cp.isSelected)
						{
							g2.setColor(Config.selectedControlPointColor);
							g2.fillRect((int)cp.viewPosition[display].x - 2,(int)cp.viewPosition[display].y - 2,5,5);
						}
						else
						{
							g2.setColor(Config.controlPointColor);
							g2.fillRect((int)cp.viewPosition[display].x - 1, (int)cp.viewPosition[display].y - 1,3,3);
						}
					}
					if (gw.visibleLayers[cp.layer] && cp.isSelected)
					{
						g2.setColor(Config.selectedControlPointColor);
						g2.fillRect((int)cp.viewPosition[display].x - 1, (int)cp.viewPosition[display].y - 1,3,3);
					}
				}
			}
		
		}
		
		
		
		if (gw.activeCP != null && gw.activeCP.layer == gw.activeLayer)
		{
			
			g2.setColor(Config.tangentColor);
			g2.fillRect((int)gw.activeCP.viewInTangent[display].x - 1,(int)gw.activeCP.viewInTangent[display].y - 1,3,3);
			g2.fillRect((int)gw.activeCP.viewOutTangent[display].x - 1,(int)gw.activeCP.viewOutTangent[display].y - 1,3,3);
			g2.drawLine((int)gw.activeCP.viewPosition[display].x,(int)gw.activeCP.viewPosition[display].y,(int)gw.activeCP.viewInTangent[display].x,(int)gw.activeCP.viewInTangent[display].y);
			g2.drawLine((int)gw.activeCP.viewPosition[display].x,(int)gw.activeCP.viewPosition[display].y,(int)gw.activeCP.viewOutTangent[display].x,(int)gw.activeCP.viewOutTangent[display].y);
			ControlPoint endCP;
			if (gw.curveSelector) endCP = gw.activeCP.curve.getNextControlPoint(gw.activeCP);
			else endCP = gw.activeCP.curve.getPrevControlPoint(gw.activeCP);
			if (endCP != null)
			{
				g2.setColor(Config.activeSegmentColor);
				if (gw.curveSelector) g2.draw(new CubicCurve2D.Float(gw.activeCP.viewPosition[display].x,gw.activeCP.viewPosition[display].y,gw.activeCP.viewOutTangent[display].x,gw.activeCP.viewOutTangent[display].y,endCP.viewInTangent[display].x,endCP.viewInTangent[display].y,endCP.viewPosition[display].x,endCP.viewPosition[display].y));
				else g2.draw(new CubicCurve2D.Float(gw.activeCP.viewPosition[display].x,gw.activeCP.viewPosition[display].y,gw.activeCP.viewInTangent[display].x,gw.activeCP.viewInTangent[display].y,endCP.viewOutTangent[display].x,endCP.viewOutTangent[display].y,endCP.viewPosition[display].x,endCP.viewPosition[display].y));
			}
			g2.setColor(Config.semiTransparentDark);
			g2.fillRect((int)gw.activeCP.viewPosition[display].x + 6,(int)gw.activeCP.viewPosition[display].y - 8 ,35,15);
			g2.setColor(Config.tangentColor);
			g2.drawString(Float.toString(Math.round(gw.activeCP.inCurvature*1000f)/1000f),(int)gw.activeCP.viewPosition[display].x + 8,(int)gw.activeCP.viewPosition[display].y + 4);
		}
		
		
		float len = 10f/windowScale;
		
		g2.setColor(Config.cursorColor);
		Point3f p1 = new Point3f(gw.cursor);
		Point3f p2 = new Point3f(gw.cursor);
		p1.add(new Point3f(len,0,0));
		p2.add(new Point3f(-len,0,0));
		view.transform(p1);
		view.transform(p2);
		g2.drawLine((int)p1.x,(int)p1.y,(int)p2.x,(int)p2.y);
		p1.set(gw.cursor);
		p2.set(gw.cursor);
		p1.add(new Point3f(0,len,0));
		p2.add(new Point3f(0,-len,0));
		view.transform(p1);
		view.transform(p2);
		g2.drawLine((int)p1.x,(int)p1.y,(int)p2.x,(int)p2.y);
		p1.set(gw.cursor);
		p2.set(gw.cursor);
		p1.add(new Point3f(0,0,len));
		p2.add(new Point3f(0,0,-len));
		view.transform(p1);
		view.transform(p2);
		g2.drawLine((int)p1.x,(int)p1.y,(int)p2.x,(int)p2.y);
		
		
		g2.setColor(Config.semiTransparentLight);
		g2.fillRect(4,4,11,11);
		
		
		g2.drawString(viewNames[viewMode] + " view",20,14);
		g2.setColor(Config.backgroundColor);
		g2.drawString(" " + (number + 1),3,14);
		//g.drawString(viewNames[viewMode] + " view",3,28);
	}	

	public void setPluginHasMouse(boolean state) {
		this.pluginHasMouse=state;
	}

	public void setSelected() {
		this.gw.setSelected(this.number);
	}
}
