/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*@author developer on changes : Dan Bishop
*@version $Id: GraphicsWindow.java,v 1.7 2003/06/04 07:17:38 god64 Exp $ $Date: 2003/06/04 07:17:38 $
**/

package jpatch;
import jpatch.plugin.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.vecmath.*;

public class GraphicsWindow
extends JPanel
{
	protected boolean peak = false;
	public Point3f cursor = new Point3f();
	public static final int SINGLE = 1;
	public static final int HORIZONTAL_SPLIT = 2;
	public static final int VERTICAL_SPLIT = 3;
	public static final int QUAD = 4;
	public static final int VIEW_TRANSLATE = 1;
	public static final int VIEW_ZOOM = 2;
	public static final int VIEW_ROTATE = 3;
	public static final int SELECT_MOVE = 4;
	public static final int ADD_SEGMENT = 5;
	public static final int ADD_MULTI = 6;
	public static final int SCALE = 7;
	public static final int ROTATE = 8;
	public static final int TANGENT = 9;
	public static final int DEFAULT = 0;
	public int action = VIEW_TRANSLATE;
	public int mode = SINGLE;
	public int activePanel = 0;
	public int selectedPanel = 0;
	public boolean[] visibleLayers = new boolean[Config.layers];
	public int activeLayer = 0;
	public BackgroundInfo[] backgroundInfo = new BackgroundInfo[8];
	
	public boolean xLock = false;
	public boolean yLock = false;
	public boolean zLock = false;
	
	public ControlPoint activeCP = null;
	public boolean curveSelector = true;
	
	public boolean snapToGrid = false;
	
	protected Mesh mesh;
	public EditMeshToolBar editMeshToolBar;
	
	public GraphicsPanel[] graphicsPanels = new GraphicsPanel[4];
	public PluginManager pluginManager=new PluginManager(this);
	
	public GraphicsWindow(Mesh mesh)
	{
		this.mesh = mesh;
		mesh.gw = this;
		this.editMeshToolBar = editMeshToolBar;
		for(int i = 1; i<8; i++)
		{
			backgroundInfo[i] = new BackgroundInfo(i);
		}
		for (int l = 0; l < Config.layers; l++) visibleLayers[l] = true;
		for (int gp = 0; gp < 4; gp++)
		{
			graphicsPanels[gp] = new GraphicsPanel(mesh,gp,this,gp);
			//graphicsPanels[gp].addKeyListener(editMeshToolBar);
		}
		graphicsPanels[0].setView(GraphicsPanel.FRONT);
		graphicsPanels[1].setView(GraphicsPanel.LEFT);
		graphicsPanels[2].setView(GraphicsPanel.TOP);
		graphicsPanels[3].setView(GraphicsPanel.BIRDSEYE);
		

	}
	
	public void setEditMeshToolBar(EditMeshToolBar editMeshToolBar)
	{
		this.editMeshToolBar = editMeshToolBar;
		for (int gp = 0; gp < 4; gp++)
		{
			//graphicsPanels[gp] = new GraphicsPanel(mesh,gp,this,gp);
			graphicsPanels[gp].addKeyListener(editMeshToolBar);
		}
		
	}
	
	public boolean layerVisible(int layer)
	{
		return visibleLayers[layer];
	}
	
	public void arrange()
	{
		removeAll();
		switch(mode)
		{
			case SINGLE:
				setLayout(new GridLayout(1,1));
				add(graphicsPanels[activePanel]);
				break;
			case HORIZONTAL_SPLIT:
				setLayout(new GridLayout(1,2));
				add(graphicsPanels[0]);
				add(graphicsPanels[1]);
				break;
			case VERTICAL_SPLIT:
				setLayout(new GridLayout(2,1));
				add(graphicsPanels[0]);
				add(graphicsPanels[2]);
				break;
			case QUAD:
				setLayout(new GridLayout(2,2));
				add(graphicsPanels[0]);
				add(graphicsPanels[1]);
				add(graphicsPanels[2]);
				add(graphicsPanels[3]);
				break;	
		}
		validate();	
	}
	public void changeMode(int mode)
	{
		activeCP = null;
		this.mode = mode;
		arrange();
	}
	public void setPeakMode(boolean peak)
	{
		this.peak = peak;
	}
	public void changeAction(int action)
	{
		if (this.action == TANGENT) activeCP = null;
		this.action = action;
		graphicsPanels[0].requestFocus();
		for (int gp = 0; gp < 4; gp++)
		{
			graphicsPanels[gp].removeAllMouseListeners();
			graphicsPanels[gp].removeAllMouseMotionListeners();
			graphicsPanels[gp].setPopupEnable(true);
			switch(action)
			{
				case VIEW_TRANSLATE:
					graphicsPanels[gp].addMouseListener(new TranslateViewML(graphicsPanels[gp],MouseEvent.BUTTON1));
					break;
				case VIEW_ZOOM:
					graphicsPanels[gp].addMouseListener(new ZoomViewML(graphicsPanels[gp],MouseEvent.BUTTON1));
					break;
				case VIEW_ROTATE:
					graphicsPanels[gp].addMouseListener(new RotateViewML(graphicsPanels[gp],MouseEvent.BUTTON1));
					break;
				case SELECT_MOVE:
					graphicsPanels[gp].addMouseListener(new SelectMoveML(this,graphicsPanels[gp],MouseEvent.BUTTON1));
					break;
				case ADD_SEGMENT:
					graphicsPanels[gp].addMouseListener(new AddSegmentML(graphicsPanels[gp],MouseEvent.BUTTON1,true));
					break;
				case ADD_MULTI:
					graphicsPanels[gp].addMouseListener(new AddSegmentML(graphicsPanels[gp],MouseEvent.BUTTON1,false));
					break;
				case SCALE:
					graphicsPanels[gp].addMouseListener(new TransformML(graphicsPanels[gp],MouseEvent.BUTTON1,TransformML.SCALE));
					break;
				case ROTATE:
					graphicsPanels[gp].addMouseListener(new TransformML(graphicsPanels[gp],MouseEvent.BUTTON1,TransformML.ROTATE));
					break;
				case TANGENT:
					graphicsPanels[gp].addMouseListener(new TangentML(graphicsPanels[gp],MouseEvent.BUTTON1));
					break;
				case DEFAULT:
					graphicsPanels[gp].addMouseListener(new SelectMoveML(this,graphicsPanels[gp],MouseEvent.BUTTON1));
					editMeshToolBar.selectPointButton.setSelected(true);
					break;
			}
		}
	
	}

	public void setSelected(int num) {
		this.selectedPanel=num;
	}
}
