package jpatch;

import java.util.*;
import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.*;
import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.universe.*; 
import com.sun.j3d.utils.geometry.ColorCube;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.*;
import javax.swing.*;

public class Java3DFrame
extends JFrame
implements MouseListener, MouseMotionListener
{
	TransformGroup transformGroup = new TransformGroup();
	Transform3D trans = new Transform3D();
	Matrix4f matrix = new Matrix4f();
	Matrix4f view = new Matrix4f();
	static final int ROTATE = 1;
	static final int SCALE = 2;
	int X;
	int Y;
	float rotX = 0f;
	float rotY = (float)Math.PI;
	float scale = 0.1f;
	int mode = 0;
	
	public void mousePressed(MouseEvent event)
	{
		X = event.getX();
		Y = event.getY();
		if (event.getButton() == MouseEvent.BUTTON1) mode = ROTATE;
		if (event.getButton() == MouseEvent.BUTTON3) mode = SCALE;
	}

	public void mouseReleased(MouseEvent event)
	{
		mode = 0;
	}
	
	public void mouseClicked(MouseEvent event)
	{
	}
	
	public void mouseEntered(MouseEvent event)
	{
	}
	
	public void mouseExited(MouseEvent event)
	{
	}
						
	public void mouseDragged(MouseEvent event)
	{
		int dx = event.getX() - X;
		int dy = event.getY() - Y;
		X = event.getX();
		Y = event.getY();
		//System.out.println("mouseDragged" + X + " " + Y + mode);
		if (mode == ROTATE)
		{			
			rotX += (float)dy/200f;
			rotY += (float)dx/200f;
		}	
		if (mode == SCALE)
		{
			scale = scale*(1+dx/200f);
			if (scale < 0.001f) scale = 0.001f;
		}
		view.rotX(rotX);
		matrix.rotY(rotY);
		view.mul(matrix);
		view.setScale(scale);
		trans.set(view);
		transformGroup.setTransform(trans);
	}

	public void mouseMoved(MouseEvent event)
	{
	}
	
	public Java3DFrame(ArrayList patches)
	{
		super("JPatch Java3D preview window");
		Shape3D shape = new J3DMesh(patches);
        	getContentPane().setLayout(new BorderLayout());
        	GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();
		Canvas3D canvas3D = new Canvas3D(config);
	        canvas3D.setSize(640,480);
		getContentPane().add("Center", canvas3D);
		validate();
		pack();
		setVisible(true);
		SimpleUniverse simpleU = new SimpleUniverse(canvas3D);
		simpleU.getViewingPlatform().setNominalViewingTransform();
		simpleU.addBranchGraph(sceneGraph(shape));
		canvas3D.addMouseListener(this);
		canvas3D.addMouseMotionListener(this);
	}
	public BranchGroup sceneGraph(Shape3D shape)
	{
		BranchGroup contentRoot = new BranchGroup();
		transformGroup = new TransformGroup();
		transformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		contentRoot.addChild(transformGroup);

		transformGroup.addChild(shape);
		
		//Alpha rotationAlpha = new Alpha(-1, 10000);
		//RotationInterpolator rotator = new RotationInterpolator(rotationAlpha, objSpin);
		BoundingSphere bounds = new BoundingSphere();
		bounds.setRadius(50);
		//rotator.setSchedulingBounds(bounds);
		//objSpin.addChild(rotator);

		/*
		AmbientLight lightA = new AmbientLight();
		lightA.setInfluencingBounds(bounds);
		contentRoot.addChild(lightA);
		*/
		
		DirectionalLight lightD1 = new DirectionalLight();
		lightD1.setInfluencingBounds(bounds);
		Vector3f direction1 = new Vector3f(-1.0f,-1.0f,-1.0f);
		direction1.normalize();
		lightD1.setDirection(direction1);
		lightD1.setColor(new Color3f(1.0f, 1.0f, 1.0f));
		contentRoot.addChild(lightD1);
		
		DirectionalLight lightD2 = new DirectionalLight();
		lightD2.setInfluencingBounds(bounds);
		Vector3f direction2 = new Vector3f(1.0f,1.0f,1.0f);
		direction2.normalize();
		lightD2.setDirection(direction2);
		lightD2.setColor(new Color3f(1.0f, 1.0f, 1.0f));
		contentRoot.addChild(lightD2);
		
		Background bg = new Background();
		bg.setColor(0.5f, 0.5f, 0.5f);
		bg.setApplicationBounds(bounds);
		contentRoot.addChild(bg);

		contentRoot.compile();
		
		view.rotX(rotX);
		matrix.rotY(rotY);
		view.mul(matrix);
		view.setScale(scale);
		trans.set(view);
		transformGroup.setTransform(trans);
		
		return contentRoot;
	}	
}

