/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.vecmath.*;
import java.awt.geom.*;

public class LatheOptions
extends JDialog
implements ActionListener, CaretListener
{
	static LatheOptions latheOptions;
	static Values values = new Values();
	JTextField numField = new JTextField("4");
	JTextField curvatureField = new JTextField("0.38737077");
	JButton okButton = new JButton("OK");
	JButton cancelButton = new JButton("Cancel");	
	JToggleButton peakButton = new JToggleButton("peak");
	JButton roundButton = new JButton("round");
	public static class Values
	{
		boolean ok = false;
		boolean peak = false;
		public Point3f pivot = new Point3f(0,0,0);
		public int num = 4;
		public float curvature = 0.33f;
	}
	
	public LatheOptions(Frame owner)
	{
		super(owner, "Lathe options", true);
		values.num = 4;
		values.curvature = 0.38737077f;
		getContentPane().setLayout(new GridLayout(4,2));
		getContentPane().add(new JLabel("Number of segments:"));
		getContentPane().add(numField);
		getContentPane().add(new JLabel("Curvature:"));
		getContentPane().add(curvatureField);
		getContentPane().add(peakButton);
		getContentPane().add(roundButton);
		getContentPane().add(okButton);
		getContentPane().add(cancelButton);
		numField.addCaretListener(this);
		curvatureField.addCaretListener(this);
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
		peakButton.addActionListener(this);
		roundButton.addActionListener(this);
		pack();
		setLocationRelativeTo(owner);
		show();
	}
	public void actionPerformed(ActionEvent event)
	{
		if (event.getSource() == okButton)
		{
			values.ok = true;
			dispose();
		}
		if (event.getSource() == cancelButton)
		{
			dispose();
		}
		if (event.getSource() == roundButton)
		{
			int segments = values.num;
			if ( segments > 2)
			{
				curvatureField.setText(Float.toString(Utils.optimumCurvature(segments)));
			}
		}
		if (event.getSource() == peakButton)	
		{
			values.peak = peakButton.isSelected();
			//curvatureField.setText(Float.toString(values.curvature));
		}	

	}
	
	public void caretUpdate(CaretEvent event)
	{
		if (event.getSource() == numField)
		{
			try
			{
				values.num = (new Integer(numField.getText())).intValue();
			}
			catch(NumberFormatException exception)
			{
				values.num = -1;
			}
		}
		if (event.getSource() == curvatureField)
		{
			try
			{
				values.curvature = (new Float(curvatureField.getText())).floatValue();
			}
			catch(NumberFormatException exception)
			{
				values.curvature = -1;
			}
		}	
	}
		 	
	public static Values getOptions(Frame owner)
	{
		latheOptions = new LatheOptions(owner);
		return values;
	}
}