/**
 *    Copyright (c)2003 Sascha Ledinsky
 *
 *    This file is part of JPatch.
 *
 *    JPatch is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JPatch is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JPatch; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jpatch;

import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;

/**
 *  Description of the Class
 *
 * @author on latest changes    dbishop
 * @created    March 31, 2003
 */
public class MainMenu
		 extends JMenuBar
		 implements ActionListener {
	/**
	 *  Description of the Fields
	 */
	protected JFileChooser fc = new JFileChooser();
	protected JMenu file;
	protected JMenu fileImport;
	protected JMenu fileExport;
	protected JMenu plaf;
	protected JMenu help;
	
	// New menu items added for javascript and Java3d
	// @author Dan Bishop
	// Mar 31, 2003
	// New menu item Scripts with sub-menu Shapes
	// @author Sascha Ledinsky
	// Apr 1, 2003 
	protected JMenu jmnuShapes;
	protected JMenu jmnuRender;
	protected JMenu jmnuScript;
	
	protected JMenuItem menuItem;
	protected JRadioButtonMenuItem radioButtonMenuItem;
	protected JFrame jPatch;
	protected ButtonGroup plafGroup;
	protected Mesh mesh;
	protected GraphicsWindow gw;


	/**
	 *Constructor for the MainMenu object
	 *
	 * @param  jPatch  Description of the Parameter
	 * @param  mesh    Description of the Parameter
	 * @param  gw      Description of the Parameter
	 */
	public MainMenu(JFrame jPatch, Mesh mesh, GraphicsWindow gw) {
		this.jPatch = jPatch;
		this.mesh = mesh;
		this.gw = gw;

		file = new JMenu("File");
		menuItem = new JMenuItem("New");
		menuItem.addActionListener(this);
		file.add(menuItem);
		menuItem = new JMenuItem("Open");
		menuItem.setEnabled(false);
		menuItem.addActionListener(this);
		file.add(menuItem);
		menuItem = new JMenuItem("Save");
		menuItem.setEnabled(false);
		menuItem.addActionListener(this);
		file.add(menuItem);
		menuItem = new JMenuItem("Save as...");
		menuItem.setEnabled(false);
		menuItem.addActionListener(this);
		file.add(menuItem);
		file.addSeparator();

		fileImport = new JMenu("Import");
		menuItem = new JMenuItem("Animation Master (.mdl)");
		menuItem.setEnabled(false);
		menuItem.addActionListener(this);
		fileImport.add(menuItem);
		menuItem = new JMenuItem("sPatch (.spt)");
		menuItem.setActionCommand("import sPatch");
		menuItem.addActionListener(this);
		fileImport.add(menuItem);

		file.add(fileImport);

		fileExport = new JMenu("Export");
		menuItem = new JMenuItem("sPatch (.spt)");
		menuItem.setActionCommand("export sPatch");
		menuItem.addActionListener(this);
		fileExport.add(menuItem);
		fileExport.addSeparator();
		menuItem = new JMenuItem("POV-Ray (.pov)");
		menuItem.setActionCommand("export pov");
		menuItem.addActionListener(this);
		fileExport.add(menuItem);
		menuItem = new JMenuItem("Renderman (.rib)");
		menuItem.setActionCommand("export rib");
		menuItem.addActionListener(this);
		fileExport.add(menuItem);

		file.add(fileExport);
		file.addSeparator();

		menuItem = new JMenuItem("Exit");
		menuItem.addActionListener(this);
		file.add(menuItem);
		add(file);
		String lookAndFeel = (UIManager.getLookAndFeel()).getName();
		UIManager.LookAndFeelInfo[] lookAndFeelInfos = UIManager.getInstalledLookAndFeels();

		plaf = new JMenu("Look&Feel");
		plafGroup = new ButtonGroup();
		for (int laf = 0; laf < lookAndFeelInfos.length; laf++) {
			radioButtonMenuItem = new JRadioButtonMenuItem(lookAndFeelInfos[laf].getName());
			radioButtonMenuItem.setActionCommand("switchPlaf::" + lookAndFeelInfos[laf].getClassName());
			radioButtonMenuItem.addActionListener(this);
			radioButtonMenuItem.setSelected(lookAndFeelInfos[laf].getName() == lookAndFeel);
			plaf.add(radioButtonMenuItem);
			plafGroup.add(radioButtonMenuItem);
		}
		add(plaf);

		// New menuitems added for Rhino Javascript
		// @Author: Dan Bishop
		// Mar 31, 2003
		
		// added circle and grid scripts and made Shapes a submenu of Script
		// @Author: Sascha Ledinsky
		// Apr 1, 2003
		
		jmnuScript = new JMenu("Script");
		jmnuShapes = new JMenu("Shapes");
		menuItem = new JMenuItem("Circle");
		menuItem.addActionListener(this);
		jmnuShapes.add(menuItem);
		menuItem = new JMenuItem("Grid");
		menuItem.addActionListener(this);
		jmnuShapes.add(menuItem);
		menuItem = new JMenuItem("Sphere");
		menuItem.addActionListener(this);
		jmnuShapes.add(menuItem);
		jmnuScript.add(jmnuShapes);
		// Commented out for release version
		// (Sascha Ledinsky)
		
	//	menuItem = new JMenuItem("Choose...");
	//	menuItem.addActionListener(this);
	//	jmnuScript.add(menuItem);
	//	menuItem = new JMenuItem("Console...");
	//	menuItem.addActionListener(this);
	//	jmnuScript.add(menuItem);
		add(jmnuScript);

		// New menuitems added for Java3D Rendering
		// @Author: Dan Bishop
		// Mar 31, 2003
		jmnuRender = new JMenu("Render");
		menuItem = new JMenuItem("Java 3D");
		menuItem.addActionListener(this);
		jmnuRender.add(menuItem);
		add(jmnuRender);

		// Changed "Help" to "Info" since there is no help ;-)
		// @Author: Sascha Ledinsky
		// Apr 24, 2003
		help = new JMenu("Info");
		menuItem = new JMenuItem("About...");
		menuItem.addActionListener(this);
		help.add(menuItem);
		// New menuitems added for credits
		// @Author: Sascha Ledinsky
		// Apr 24, 2003
		menuItem = new JMenuItem("Credits...");
		menuItem.addActionListener(this);
		help.add(menuItem);

		add(help);
	}


	/**
	 *  Description of the Method
	 *
	 * @param  event  Description of the Parameter
	 */
	public void actionPerformed(ActionEvent event) {
		String cmd = event.getActionCommand();
		//System.out.println(cmd);
		if (cmd.startsWith("switchPlaf::")) {
			String lookAndFeelClassName = cmd.substring(12);
			try {
				UIManager.setLookAndFeel(lookAndFeelClassName);
				SwingUtilities.updateComponentTreeUI(jPatch);
			} catch (Exception e) {
			}
		}
		//System.out.println(cmd);
		if (cmd == "Exit") {
			jPatch.dispose();
			System.exit(0);
		}
		if (cmd == "New") {
			//System.out.println("***");
			mesh.NEW();
			gw = new GraphicsWindow(mesh);
			jPatch.repaint();
		}
		if (cmd == "export sPatch") {
			if (Config.dir == "") {
				fc = new JFileChooser();
			} else {
				fc = new JFileChooser(Config.dir);
			}
			fc.addChoosableFileFilter(new SPatchFilter());
			if (fc.showSaveDialog(jPatch) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.sPatchExport(filename);
				if (errorString != "") {
					JOptionPane.showMessageDialog(jPatch, errorString, "IO Error", JOptionPane.ERROR_MESSAGE);
				} else {
					Config.dir = file.getParent();
				}
			}
		}
		if (cmd == "export pov") {
			if (Config.dir == "") {
				fc = new JFileChooser();
			} else {
				fc = new JFileChooser(Config.dir);
			}
			if (fc.showSaveDialog(jPatch) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.povExport(filename);
				if (errorString != "") {
					JOptionPane.showMessageDialog(jPatch, errorString, "IO Error", JOptionPane.ERROR_MESSAGE);
				} else {
					Config.dir = file.getParent();
				}
			}
		}
		if (cmd == "export rib") {
			if (Config.dir == "") {
				fc = new JFileChooser();
			} else {
				fc = new JFileChooser(Config.dir);
			}
			if (fc.showSaveDialog(jPatch) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.ribExport(filename);
				if (errorString != "") {
					JOptionPane.showMessageDialog(jPatch, errorString, "IO Error", JOptionPane.ERROR_MESSAGE);
				} else {
					Config.dir = file.getParent();
				}
			}
		}
		if (cmd == "import sPatch") {
			if (Config.dir == "") {
				fc = new JFileChooser();
			} else {
				fc = new JFileChooser(Config.dir);
			}
			fc.addChoosableFileFilter(new SPatchFilter());
			if (fc.showOpenDialog(jPatch) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.sPatchImport(filename);
				if (errorString != "") {
					JOptionPane.showMessageDialog(jPatch, errorString, "IO Error", JOptionPane.ERROR_MESSAGE);
				} else {
					Config.dir = file.getParent();
				}
			}
			jPatch.repaint();
		}
		
		// Menu cmds for Rhino Javascript
		// @Author: Dan Bishop
		// Mar 31, 2003
		
		// added circle and grid menuItems
		// @Author: Sascha Ledinsky
		// Apr 1, 2003
		if(cmd == "Circle")
		{
			try {
				TestScript.load(mesh, "scripts/circle.js", true);
			} catch (Exception e) {
				System.out.println(e);
				System.out.println("Javascript aborted");
			}
		}
		if(cmd == "Grid")
		{
			try {
				TestScript.load(mesh, "scripts/grid.js", true);
			} catch (Exception e) {
				System.out.println(e);
				System.out.println("Javascript aborted");
			}
		}
		if(cmd == "Sphere")
		{
			try {
				TestScript.load(mesh, "scripts/sphere.js", true);
			} catch (Exception e) {
				System.out.println(e);
				System.out.println("Javascript aborted");
			}
		}
		if (cmd == "Choose...") {
			if (Config.dir == "") {
				fc = new JFileChooser();
			} else {
				fc = new JFileChooser(Config.dir);
			}
			if (fc.showOpenDialog(mesh.frame) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				Config.dir = file.getParent();
				String filename = file.getPath();
				try {
					TestScript.load(mesh, filename, false);
				} catch (Exception e) {
					System.out.println(e);
					System.out.println("Javascript aborted");
				}
			}
		}
		if (cmd == "Console...")
		{
			try
			{
				TestScript.run(mesh);
			}
			catch(Exception e)
			{
				System.out.println(e);
				System.out.println("Javascript aborted");
			}
		}
		
		// Rendering Java3D
		// @Author: Dan Bishop
		// Mar 31,2003
		if (cmd == "Java 3D") {
			//Shap scene  = gp.mesh.sceneGraph();
			//mesh.j3dFrame = new Java3DFrame(mesh.getShape());
			//mesh.j3dFrame.setVisible(true);
			
			mesh.computePatches();
			mesh.j3dFrame = new Java3DFrame(mesh.patches);
			mesh.j3dFrame.setVisible(true);
		}
		if (cmd == "About...") {
			JOptionPane.showMessageDialog(jPatch, Config.about, "About...", JOptionPane.PLAIN_MESSAGE, new ImageIcon(getClass().getClassLoader().getResource("images/jpatchlogo.png")));
		}
		if (cmd == "Credits...") {
			JOptionPane.showMessageDialog(jPatch, Config.team, "Credits...", JOptionPane.PLAIN_MESSAGE, new ImageIcon(getClass().getClassLoader().getResource("images/jpatchlogo.png")));
		}
		jPatch.repaint();
	}
}
