/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;

public class MainToolBar
extends JToolBar
implements ActionListener
{
	protected Mesh mesh;
	protected JButton newButton;
	protected JButton openButton;
	protected JButton saveButton;
	protected JButton undoButton;
	protected JButton redoButton;
	protected JToggleButton singleViewButton;
	protected JToggleButton horizontalSplitViewButton;
	protected JToggleButton verticalSplitViewButton;
	protected JToggleButton quadViewButton;
	protected JToggleButton translateViewButton;
	protected JToggleButton zoomViewButton;
	protected JToggleButton rotateViewButton;
	protected JToggleButton meshModeButton;
	protected JToggleButton muscleModeButton;
	protected JToggleButton boneModeButton;
	protected JToggleButton xLockButton;
	protected JToggleButton yLockButton;
	protected JToggleButton zLockButton;
	protected ButtonGroup viewButtonGroup;
	protected ButtonGroup modeButtonGroup;
	//protected ButtonGroup actionButtonGroup;		
	protected GraphicsWindow graphicsWindow;
	public MainToolBar(GraphicsWindow graphicsWindow, ButtonGroup actionButtonGroup)
	{
		super("Control Panel");
		this.graphicsWindow = graphicsWindow;
		this.mesh = graphicsWindow.mesh;
		newButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/new.png")));
		openButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/open.png")));
		saveButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/save.png")));
		undoButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/undo.png")));
		redoButton = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/redo.png")));
		singleViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/single.png")));
		horizontalSplitViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/horizontal.png")));
		verticalSplitViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/vertical.png")));
		quadViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/quad.png")));
		translateViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/move.png")));
		zoomViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/zoom.png")));
		rotateViewButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/rotate.png")));
		meshModeButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/mesh.png")));
		muscleModeButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/muscle.png")));
		boneModeButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/bone.png")));
		xLockButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/xunlocked.png")));
		xLockButton.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("images/xlocked.png")));
		yLockButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/yunlocked.png")));
		yLockButton.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("images/ylocked.png")));
		zLockButton = new JToggleButton(new ImageIcon(getClass().getClassLoader().getResource("images/zunlocked.png")));
		zLockButton.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("images/zlocked.png")));
		//openButton.setEnabled(false);
		//saveButton.setEnabled(false);
		meshModeButton.setSelected(true);
		muscleModeButton.setEnabled(false);
		boneModeButton.setEnabled(false);
		
		add(newButton);
		add(openButton);
		add(saveButton);
		addSeparator(new Dimension(10,10));
		add(undoButton);
		add(redoButton);
		addSeparator(new Dimension(10,10));
		add(singleViewButton);
		add(horizontalSplitViewButton);
		add(verticalSplitViewButton);
		add(quadViewButton);
		addSeparator(new Dimension(10,10));
		add(translateViewButton);
		add(zoomViewButton);
		add(rotateViewButton);
		addSeparator(new Dimension(10,10));
		add(xLockButton);
		add(yLockButton);
		add(zLockButton);
		addSeparator(new Dimension(10,10));
		add(meshModeButton);
		add(muscleModeButton);
		add(boneModeButton);
		
		newButton.setActionCommand("new");
		openButton.setActionCommand("open");
		saveButton.setActionCommand("save");
		undoButton.setActionCommand("undo");
		redoButton.setActionCommand("redo");
		singleViewButton.setActionCommand("singleView");
		horizontalSplitViewButton.setActionCommand("horizontalSplitView");
		verticalSplitViewButton.setActionCommand("verticalSplitView");
		quadViewButton.setActionCommand("quadView");
		translateViewButton.setActionCommand("translateView");
		zoomViewButton.setActionCommand("zoomView");
		rotateViewButton.setActionCommand("rotateView");
		meshModeButton.setActionCommand("meshMode");
		muscleModeButton.setActionCommand("muscleMode");
		boneModeButton.setActionCommand("boneMode");
		xLockButton.setActionCommand("xLock");
		yLockButton.setActionCommand("yLock");
		zLockButton.setActionCommand("zLock");
		
		//translateViewButton.setMnemonic(KeyEvent.VK_M);
		//zoomViewButton.setMnemonic(KeyEvent.VK_Z);
		//rotateViewButton.setMnemonic(KeyEvent.VK_R);
		
		newButton.addActionListener(this);
		openButton.addActionListener(this);
		saveButton.addActionListener(this);
		undoButton.addActionListener(this);
		redoButton.addActionListener(this);
		singleViewButton.addActionListener(this);
		horizontalSplitViewButton.addActionListener(this);
		verticalSplitViewButton.addActionListener(this);
		quadViewButton.addActionListener(this);
		translateViewButton.addActionListener(this);
		zoomViewButton.addActionListener(this);
		rotateViewButton.addActionListener(this);
		meshModeButton.addActionListener(this);
		muscleModeButton.addActionListener(this);
		boneModeButton.addActionListener(this);
		xLockButton.addActionListener(this);
		yLockButton.addActionListener(this);
		zLockButton.addActionListener(this);
			
		newButton.setToolTipText("new model");
		openButton.setToolTipText("open model");
		saveButton.setToolTipText("save model");
		undoButton.setToolTipText("undo");
		redoButton.setToolTipText("redo");		
		singleViewButton.setToolTipText("single view");
		horizontalSplitViewButton.setToolTipText("split view horizontally");
		verticalSplitViewButton.setToolTipText("split view vertically");
		quadViewButton.setToolTipText("quad view");
		translateViewButton.setToolTipText("move view");
		zoomViewButton.setToolTipText("zoom view");
		rotateViewButton.setToolTipText("rotate view");
		meshModeButton.setToolTipText("mesh mode");
		muscleModeButton.setToolTipText("muscle mode");
		boneModeButton.setToolTipText("bone mode");
		xLockButton.setToolTipText("lock/unlock X");
		yLockButton.setToolTipText("lock/unlock Y");
		zLockButton.setToolTipText("lock/unlock Z");
		
		singleViewButton.setSelected(graphicsWindow.mode == GraphicsWindow.SINGLE);
		horizontalSplitViewButton.setSelected(graphicsWindow.mode == GraphicsWindow.HORIZONTAL_SPLIT);
		verticalSplitViewButton.setSelected(graphicsWindow.mode == GraphicsWindow.VERTICAL_SPLIT);
		quadViewButton.setSelected(graphicsWindow.mode == GraphicsWindow.QUAD);
		
		viewButtonGroup = new ButtonGroup();
		viewButtonGroup.add(singleViewButton);
		viewButtonGroup.add(horizontalSplitViewButton);
		viewButtonGroup.add(verticalSplitViewButton);
		viewButtonGroup.add(quadViewButton);
		
		//actionButtonGroup = new ButtonGroup();
		actionButtonGroup.add(translateViewButton);
		actionButtonGroup.add(zoomViewButton);
		actionButtonGroup.add(rotateViewButton);
		
		modeButtonGroup = new ButtonGroup();
		modeButtonGroup.add(meshModeButton);
		modeButtonGroup.add(muscleModeButton);
		modeButtonGroup.add(boneModeButton);
	}
	/*
	public unselect()
	{
		translateViewButton.setSelected(flase);
		zoomViewButton.setSelected(flase);
		rotateViewButton.setSelected(flase);
	}
	*/
	public void actionPerformed(ActionEvent event)
	{
		String cmd = event.getActionCommand();
		//System.out.println(cmd);
		if (cmd == "new")
		{
			mesh.NEW();
			mesh.frame.repaint();
		}
		if (cmd == "save")
		{
			JFileChooser fc;
			if (Config.dir == "") fc = new JFileChooser();
			else fc = new JFileChooser(Config.dir);
			fc.addChoosableFileFilter(new SPatchFilter());
			if ( fc.showSaveDialog(mesh.frame) == JFileChooser.APPROVE_OPTION)
			{
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.sPatchExport(filename);
				if (errorString != "") JOptionPane.showMessageDialog(mesh.frame,errorString,"IO Error",JOptionPane.ERROR_MESSAGE);
				else Config.dir = file.getParent();
			}
		}
		if (cmd == "open")
		{
			JFileChooser fc;
			if (Config.dir == "") fc = new JFileChooser();
			else fc = new JFileChooser(Config.dir);
			fc.addChoosableFileFilter(new SPatchFilter());
			if ( fc.showOpenDialog(mesh.frame) == JFileChooser.APPROVE_OPTION)
			{
				File file = fc.getSelectedFile();
				String filename = file.getPath();
				String errorString = mesh.sPatchImport(filename);
				if (errorString != "") JOptionPane.showMessageDialog(mesh.frame,errorString,"IO Error",JOptionPane.ERROR_MESSAGE);
				else Config.dir = file.getParent();
			}
			mesh.frame.repaint();
		}
		if (cmd == "undo")
		{
			mesh.undo();
			mesh.frame.repaint();
		}
		if (cmd == "redo")
		{
			mesh.redo();
			mesh.frame.repaint();
		}
		if (event.getActionCommand() == "singleView") graphicsWindow.changeMode(GraphicsWindow.SINGLE);
		if (event.getActionCommand() == "singleView") graphicsWindow.changeMode(GraphicsWindow.SINGLE);
		if (event.getActionCommand() == "horizontalSplitView") graphicsWindow.changeMode(GraphicsWindow.HORIZONTAL_SPLIT);
		if (event.getActionCommand() == "verticalSplitView") graphicsWindow.changeMode(GraphicsWindow.VERTICAL_SPLIT);
		if (event.getActionCommand() == "quadView") graphicsWindow.changeMode(GraphicsWindow.QUAD);
		if (event.getActionCommand() == "translateView") graphicsWindow.changeAction(GraphicsWindow.VIEW_TRANSLATE);
		if (event.getActionCommand() == "zoomView") graphicsWindow.changeAction(GraphicsWindow.VIEW_ZOOM);
		if (event.getActionCommand() == "rotateView") graphicsWindow.changeAction(GraphicsWindow.VIEW_ROTATE);
		if (cmd == "xLock") graphicsWindow.xLock = !graphicsWindow.xLock;
		if (cmd == "yLock") graphicsWindow.yLock = !graphicsWindow.yLock;
		if (cmd == "zLock") graphicsWindow.zLock = !graphicsWindow.zLock;
		graphicsWindow.graphicsPanels[0].requestFocus();
	}	
}