/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.io.*;
import java.util.*;
import javax.vecmath.*;
import java.math.*;
import javax.swing.*;
//import javax.media.j3d.*;
//import com.sun.j3d.utils.universe.*; 
//import com.sun.j3d.utils.geometry.*;
import java.awt.*;

public class Mesh
extends ArrayList
{
	protected UndoStack undoStack = new UndoStack(Config.undoStackCapacity);
	protected UndoAction undoAction;
	public ArrayList patches = new ArrayList();
	public GraphicsWindow gw;
	//ArrayList[] undoBuffer = new ArrayList[20];
	int undoPosition = 0;
	int undoFill = 0;
	public JFrame frame;
	public JFrame j3dFrame;
	static final int undoMax = 20;
	
	public static final int DIRECTION_X = 0;
	public static final int DIRECTION_Y = 1;
	public static final int DIRECTION_Z = 2;
	/*
	ControlPoint getCP(int curveID, int pointID)
	{
		Curve curve = (Curve)get(curveID);
		return (ControlPoint)curve.get(pointID);
	}
	
	ControlPoint getHeadCP(ControlPoint cp)
	{
		if (cp.isWelded) cp = getHeadCP(getCP(cp.weldedToCurve,cp.weldedToPoint));
		return cp;
	}
	*/
	
	public void NEW()
	{
		clear();
		patches.clear();
		undoStack.clear();
		gw.cursor.set(0,0,0);
		gw.changeAction(GraphicsWindow.DEFAULT);
	}
	
	public void add(Curve curve)
	{
		super.add(curve);
		curve.mesh = this;
		curve.validate();
	}
	
	public void startUndoRecorder(String name)
	{
		undoAction = new UndoAction(name);
		//System.out.println("undoRecorder started - record " + name);
	}
	
	public void stopUndoRecorder()
	{
		//System.out.println("undoRecorder stopped - record " + undoAction.getName());
		undoStack.add(undoAction);
		undoAction = null;
	}
		
	public void stopUndoRecorder(String name)
	{
		undoAction.rename(name);
		stopUndoRecorder();
	}
	
	public void recordAtomicAction(AtomicAction atomicAction)
	{
		undoAction.add(atomicAction);
	}
	
	public void undo()
	{
		undoStack.undo();
		validate();
	}
	
	public void redo()
	{
		undoStack.redo();
		validate();
	}
	
	public ArrayList getWeldedCPs(ControlPoint cp)
	{
		ArrayList result = new ArrayList();
		ControlPoint head = cp.getHead();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint testCP = (ControlPoint)curve.get(p);
				if (testCP.getHead() == head) result.add(testCP);
			}
		}
		return result;
	}
	
	public ArrayList getTailCPs(ControlPoint cp)
	{
		ArrayList result = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint testCP = (ControlPoint)curve.get(p);
				if (testCP.getHead() == cp) result.add(testCP);
			}
		}
		return result;
	}
	
	public ArrayList getAllControlPoints()
	{
		ArrayList points = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				points.add(cp);
			}
		}
		return points;
	}
	
	public void computeTails()
	{
		ArrayList points = getAllControlPoints();
		for (int p = 0; p < points.size(); p++)
		{
			ControlPoint cp = (ControlPoint)points.get(p);
			cp.tail.clear();
		}
		for (int p = 0; p < points.size(); p++)
		{
			ControlPoint cp = (ControlPoint)points.get(p);
			if (cp.weldedTo != null) (cp.weldedTo.getHead()).tail.add(cp);
		}
	}
		
	public void removeSingleCP(ControlPoint cp)
	{
		//System.out.println("removeCP called");
		ArrayList weldedCPs = getTailCPs(cp);
		//System.out.println("weldedCPs size = " + weldedCPs.size());
		for (int w = 0; w < weldedCPs.size(); w++)
		{
			ControlPoint rcp = (ControlPoint)weldedCPs.get(w);
			rcp.weldedTo = null;
		}
		cp.curve.remove(cp);
	}
	
	public ChangeCurvesAtomicAction removeCurveSegment(Curve curve, int pointOnCurve)
	{
		if (curve.closed)
		{
			Curve newCurve = new Curve();
			for (int p = 0; p < curve.size(); p++)
			{
				int pp = (p + pointOnCurve + 1)%curve.size();
				newCurve.add(curve.get(pp));
			}
			newCurve.closed = false;
			ArrayList newCurves = new ArrayList();
			ArrayList oldCurves = new ArrayList();
			oldCurves.add(curve);
			newCurves.add(newCurve);
			remove(curve);
			add(newCurve);
			validate();
			return new ChangeCurvesAtomicAction(this,oldCurves,newCurves);
		}
		else
		{
			Curve newCurve1 = new Curve();
			Curve newCurve2 = new Curve();
			for (int p = 0; p <= pointOnCurve; p++) newCurve1.add(curve.get(p));
			for (int p = pointOnCurve + 1; p < curve.size(); p++) newCurve2.add(curve.get(p));
			newCurve1.closed = false;
			newCurve2.closed = false;
			ArrayList newCurves = new ArrayList();
			ArrayList oldCurves = new ArrayList();
			oldCurves.add(curve);
			remove(curve);
			if (newCurve1.size() > 1)
			{
				add(newCurve1);
				newCurves.add(newCurve1);
			}
			if (newCurve2.size() > 1)
			{
				add(newCurve2);
				newCurves.add(newCurve1);
			}
			validate();
			return new ChangeCurvesAtomicAction(this,oldCurves,newCurves);
		}	
	}
	
	public void deleteSegment()
	{
		if (gw.activeCP != null)
		{
			startUndoRecorder("delete curve segment");
			Curve curve = gw.activeCP.curve;
			int point = gw.activeCP.pointOnCurve;
			if (!gw.curveSelector) point--;
			recordAtomicAction(removeCurveSegment(curve,point));
			stopUndoRecorder();
			gw.activeCP = null;
		}
	}
	/*
	public WeldCurvesAtomicAction weldCurves(Curve curve1, Curve curve2, int pointOnCurve1, int pointOnCurve2)
	{	
		//System.out.println(pointOnCurve1 + " " + pointOnCurve2);
		Curve newCurve = new Curve();
		if (pointOnCurve1 == 0) for (int p = curve1.size() - 1; p >= 1; p--) newCurve.add((ControlPoint)curve1.get(p));
		else for (int p = 0; p < curve1.size() - 1; p++) newCurve.add((ControlPoint)curve1.get(p));
		if (pointOnCurve2 == 0) for (int p = 0; p < curve2.size(); p++) newCurve.add((ControlPoint)curve2.get(p));
		else for (int p = curve2.size() - 1; p >= 0; p--) newCurve.add((ControlPoint)curve2.get(p));
		WeldCurvesAtomicAction atomicAction = new WeldCurvesAtomicAction(this,curve1,curve2,newCurve);
		remove(curve1);
		remove(curve2);
		add(newCurve);
		return atomicAction;
	}
	*/
	public ChangeCurvesAtomicAction weldCurves(Curve curve1, Curve curve2, int pointOnCurve1, int pointOnCurve2)
	{	
		//System.out.println(pointOnCurve1 + " " + pointOnCurve2);
		Curve newCurve = new Curve();
		if (pointOnCurve1 == 0) for (int p = curve1.size() - 1; p >= 1; p--) newCurve.add((ControlPoint)curve1.get(p));
		else for (int p = 0; p < curve1.size() - 1; p++) newCurve.add((ControlPoint)curve1.get(p));
		if (pointOnCurve2 == 0) for (int p = 0; p < curve2.size(); p++) newCurve.add((ControlPoint)curve2.get(p));
		else for (int p = curve2.size() - 1; p >= 0; p--) newCurve.add((ControlPoint)curve2.get(p));
		ArrayList oldCurves = new ArrayList();
		ArrayList newCurves = new ArrayList();
		oldCurves.add(curve1);
		oldCurves.add(curve2);
		newCurves.add(newCurve);
		ChangeCurvesAtomicAction atomicAction = new ChangeCurvesAtomicAction(this,oldCurves,newCurves);
		remove(curve1);
		remove(curve2);
		add(newCurve);
		return atomicAction;
	}
								
	public void validate()
	{
		int num = 0;
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				((ControlPoint)curve.get(p)).validate();
				num++;
			}
			curve.validate();
		}
		//System.out.println("Mesh.validate() " + num + " ControlPoints counted");
	}
	
	public void unselectAll()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				cp.isSelected = false;
			}
		}
	}
	public void selectLayer()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.layer == gw.activeLayer) cp.isSelected = true;
				else cp.isSelected = false;
			}
		}
	}
	public void selectAll()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (gw.layerVisible(cp.layer)) cp.isSelected = true;
				else cp.isSelected = false;
			}
		}
	}

	public void invertSelection()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				cp.isSelected = (!cp.isSelected && gw.layerVisible(cp.layer));
			}
		}
		
	}
	public void layerSelected(int layer)
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if(cp.getHead().isSelected) cp.layer = layer;
			}
		}
		validate();
	}
	public void moveSelected(Point3f vector)
	{
		if (gw.xLock) vector.x = 0;
		if (gw.yLock) vector.y = 0;
		if (gw.zLock) vector.z = 0;
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if(cp.isSelected) cp.position.add(vector);
			}
		}
		validate();
	}
	public void initHelper()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				cp.helperPosition.set(cp.position);
			}
		}
	}
	
	public void prepareUndoableTransform()
	{
		initHelper();
	}
	
	public void recordUndoableTransform()
	{
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (!cp.position.equals(cp.helperPosition))
				{
					PositionControlPointAtomicAction atomicAction = new PositionControlPointAtomicAction(cp,cp.helperPosition,cp.position);
					undoAction.add(atomicAction);
				}
			}
		}
	}
	
	public void peakSelected(boolean peak)
	{
		String name;
		if (peak) name = "peak";
		else name = "round";
		startUndoRecorder(name);
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if(cp.getHead().isSelected) 
				{
					recordAtomicAction(new TangentChangeAtomicAction(cp,peak,cp.inCurvature));
					cp.peak = peak;
				}
			}
		}
		if (gw.activeCP != null)
		{
			recordAtomicAction(new TangentChangeAtomicAction(gw.activeCP,peak,gw.activeCP.inCurvature));
			gw.activeCP.peak = peak;
		}
		stopUndoRecorder();
		validate();
	}
					
	public void transformSelected(Matrix4f matrix)
	{
		Point3f p3 = new Point3f();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if(cp.isSelected) 
				{
					p3.set(cp.helperPosition);
					//cp.position.set(cp.helperPosition);
					matrix.transform(p3);
					cp.position.set(p3);
					if (gw.xLock) cp.position.x = cp.helperPosition.x;
					if (gw.yLock) cp.position.y = cp.helperPosition.y;
					if (gw.zLock) cp.position.z = cp.helperPosition.z;
				}
			}
		}
		validate();
	}
	
	protected ArrayList getClonedCurves(ArrayList parentCPs, ArrayList clonedCPs)
	{
		return getClonedCurves(parentCPs,clonedCPs,false);
	}
	
	protected ArrayList getClonedCurves(ArrayList parentCPs, ArrayList clonedCPs, boolean headOnly)
	{
		ArrayList clonedCurves = new ArrayList();
		int num = 0;
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			Curve newCurve = new Curve();
			newCurve.closed = curve.closed;
			ArrayList dummyParents = new ArrayList();
			ArrayList dummyClones = new ArrayList();
			int start = 0;
			if (curve.closed)
			{
				for (int p = 0; p < curve.size(); p++)
				{
					ControlPoint cp = (ControlPoint)curve.get(p);
					if (!cp.headSelected()) start = p;
				}
			}
			for (int pp = 0; pp < curve.size(); pp++)
			{
				int s = pp + start;
				int p = s%curve.size();
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.headSelected() && (cp.weldedTo == null || !headOnly))
				{
					//System.out.print("clone... selected");
					ControlPoint newCp = new ControlPoint(cp.position,cp.peak,cp.layer);
					newCp.inCurvature = cp.inCurvature;
					newCp.outCurvature = cp.outCurvature;
					//newCp.isSelected = true;
					//cp.isSelected = false;
					newCp.weldedTo = cp.weldedTo;
					newCurve.add(newCp);
					dummyParents.add(cp);
					dummyClones.add(newCp);
				}
				else
				{
					newCurve.closed = false;
					if (newCurve.size() > 1)
					{
						num += newCurve.size();
						clonedCurves.add(newCurve);
						parentCPs.addAll(dummyParents);
						clonedCPs.addAll(dummyClones);
					}
					dummyParents.clear();
					dummyClones.clear();
					newCurve = new Curve();
					//newCurve.closed = false;
				}
				
			}
			if (newCurve.size() > 1)
			{
				num += newCurve.size();
				clonedCurves.add(newCurve);
				parentCPs.addAll(dummyParents);
				clonedCPs.addAll(dummyClones);
			}
		}
		/*
		for (int c = 0; c < clonedCurves.size(); c++)
		{
			add(clonedCurves.get(c));
		}
		*/
		/**
		* reconstruct weldingpoints in clone
		**/
		
		for (int p = 0; p < clonedCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)clonedCPs.get(p);
			ControlPoint weld = cp.weldedTo;
			if (weld != null)
			{
				int index = parentCPs.indexOf(weld);
				if (index != -1 && index < clonedCPs.size()) cp.weldedTo = (ControlPoint)clonedCPs.get(index);
				else cp.weldedTo = null;
				if (cp.weldedTo == cp) cp.weldedTo = null;
			}
			//if (cp.weldedTo == null) cp.isSelected = true;
		}
		/*
		for (int p = 0; p < parentCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)parentCPs.get(p);
			(cp.getHead()).isSelected = false;
			
		}
		*/
		//System.out.println (num + " controlPoints cloned");
		return clonedCurves;
	}

	/*
	protected ArrayList getClonedCurves(ArrayList parentCPs, ArrayList clonedCPs)
	{
		ArrayList clonedCurves = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			Curve dummyCurve = new Curve();
			Curve newCurve = new Curve();
			newCurve.closed = curve.closed;
			ArrayList dummyParents = new ArrayList();
			ArrayList dummyClones = new ArrayList();
			ControlPoint dummyCurveParent = null;
			ControlPoint dummyCurveClone = null;
			boolean start = false;
			boolean end = false;
			boolean gap = false;
			boolean first= false;
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.headSelected())
				{
					if (p == 0) start = true;
					if (p == curve.size() - 1) end = true;
					////System.out.print("clone... selected");
					ControlPoint newCp = new ControlPoint(cp.position);
					//newCp.isSelected = true;
					//cp.isSelected = false;
					newCp.weldedTo = cp.weldedTo;
					newCurve.add(newCp);
					dummyParents.add(cp);
					dummyClones.add(newCp);
				}
				else
				{
					newCurve.closed = false;
					//if (newCurve.size() > 1)
					//{
						//clonedCurves.add(newCurve);
					gap = true;
					if (start)
					{
						start = false;
						first= true;
						dummyCurve = new Curve();
						for (int d = 0; d < newCurve.size(); d++)
						{
							dummyCurve.add(newCurve.get(d));
						}
						if (dummyCurve.size() > 1)
						{
							parentCPs.addAll(dummyParents);
							clonedCPs.addAll(dummyClones);
						}
						else
						{
							dummyCurveParent = (ControlPoint)dummyParents.get(0);
							dummyCurveClone = (ControlPoint)dummyClones.get(0);
						}
					}
					else if (newCurve.size() > 1)
					{
						clonedCurves.add(newCurve);
						parentCPs.addAll(dummyParents);
						clonedCPs.addAll(dummyClones);
					}
					
					//}
					dummyParents.clear();
					dummyClones.clear();
					newCurve = new Curve();
					//newCurve.closed = false;
				}
				
			}
			//if (newCurve.size() > 1)
			//{
				if (gap && first)
				{
					if (curve.closed && end)
					{
						for (int d = 0; d < dummyCurve.size(); d++)
						{
							newCurve.add(dummyCurve.get(d));
						}
						if (dummyCurve.size() == 1)
						{
							parentCPs.add(dummyCurveParent);
							clonedCPs.add(dummyCurveClone);
						}
					}
				}
				else if (dummyCurve.size() > 1) clonedCurves.add(dummyCurve);
				
				if (newCurve.size() > 1) clonedCurves.add(newCurve);
				if (newCurve.size() > 1 || dummyCurve.size() > 1)
				{
					parentCPs.addAll(dummyParents);
					clonedCPs.addAll(dummyClones);
				}
			//}
		}
		/*
		for (int c = 0; c < clonedCurves.size(); c++)
		{
			add(clonedCurves.get(c));
		}
		
		/**
		* reconstruct weldingpoints in clone
		
		
		for (int p = 0; p < clonedCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)clonedCPs.get(p);
			ControlPoint weld = cp.weldedTo;
			if (weld != null)
			{
				int index = parentCPs.indexOf(weld);
				if (index != -1 && index < clonedCPs.size()) cp.weldedTo = (ControlPoint)clonedCPs.get(index);
				else cp.weldedTo = null;
				if (cp.weldedTo == cp) cp.weldedTo = null;
			}
			//if (cp.weldedTo == null) cp.isSelected = true;
		}
		/*
		for (int p = 0; p < parentCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)parentCPs.get(p);
			(cp.getHead()).isSelected = false;
			
		}
		
		return clonedCurves;
	}
	*/
	public void cloneSelected()
	{
		startUndoRecorder("clone");
		ArrayList parentCPs = new ArrayList();
		ArrayList clonedCPs = new ArrayList();
		ArrayList clonedCurves = getClonedCurves(parentCPs,clonedCPs);
		for (int p = 0; p < clonedCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)clonedCPs.get(p);
			if (cp.weldedTo == null) cp.isSelected = true;
		}
		/*
		for (int p = 0; p < parentCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)parentCPs.get(p);
			(cp.getHead()).isSelected = false;
			
		}
		*/
		unselectAll();
		for (int c = 0; c < clonedCurves.size(); c++)
		{
			Curve curve = (Curve)clonedCurves.get(c);
			add(curve);
			recordAtomicAction(new NewCurveAtomicAction(this,curve));
		}
		stopUndoRecorder();
		validate();
	}
	
	public void extrudeSelected()
	{
		startUndoRecorder("extrude");
		ArrayList clonedCPs = new ArrayList();
		ArrayList parentCPs = new ArrayList();
		ArrayList clonedCurves = new ArrayList();
		clonedCurves = getClonedCurves(parentCPs,clonedCPs);
		for (int c = 0; c < clonedCurves.size(); c++)
		{
			Curve curve = (Curve)clonedCurves.get(c);
			add(curve);
			recordAtomicAction(new NewCurveAtomicAction(this,curve));
		}
		for (int p = 0; p < clonedCPs.size(); p++)
		{
			//System.out.println("p=" + p);
			ControlPoint pcp = (ControlPoint)parentCPs.get(p); // parent
		    if (pcp.weldedTo == null)
		    {	
			ControlPoint ccp = (ControlPoint)clonedCPs.get(p); // clone
			ArrayList weldedClones = getWeldedCPs(ccp);
			//System.out.println(weldedClones.size());
			ControlPoint ecp = new ControlPoint(ccp.position,ccp.peak,ccp.layer); // welded to clone
			ecp.weldedTo = ccp;
			boolean newCurve = true;
			ArrayList weldedCPs = getWeldedCPs(pcp);
			check:
			for (int w = 0; w < weldedCPs.size(); w++)
			{
				//System.out.println("w=" + w);
				ControlPoint wcp = (ControlPoint)weldedCPs.get(w);
				//if (wcp != pcp && !wcp.curve.closed && (wcp.curve != pcp.curve || wcp.pointOnCurve != pcp.pointOnCurve))
				//if (wcp != pcp && !wcp.curve.closed)
				if (parentCPs.indexOf(wcp) == -1 && !wcp.curve.closed)
				{
					//System.out.println("check1");
					if (wcp.pointOnCurve == 0)
					{
						//System.out.println("start");
						wcp.curve.add(0,ecp);
						recordAtomicAction(new NewControlPointAtomicAction(wcp.curve,ecp));
						newCurve = false;
						break check;
					}
					if (wcp.pointOnCurve == (wcp.curve.size() - 1))
					{
						//System.out.println("end");
						wcp.curve.add(ecp);
						recordAtomicAction(new NewControlPointAtomicAction(wcp.curve,ecp));
						newCurve = false;
						break check;
					}
				}
			}
			if (newCurve)
			{
				//System.out.println("newcurve");
				Curve curve = new Curve();
				ControlPoint wcp = new ControlPoint(pcp.position,pcp.peak,pcp.layer);
				wcp.weldedTo = pcp;
				curve.add(wcp);
				curve.add(ecp);
				add(curve);
				recordAtomicAction(new NewCurveAtomicAction(this,curve));
			}
		    }
		}
		unselectAll();	
		for (int p = 0; p < clonedCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)clonedCPs.get(p);
			if (cp.weldedTo == null) cp.isSelected = true;
			//else cp.isSelected = false;
		}
		/*
		for (int p = 0; p < parentCPs.size(); p++)
		{
			ControlPoint cp = (ControlPoint)parentCPs.get(p);
			(cp.getHead()).isSelected = false;
			
		}
		*/
		stopUndoRecorder();
		validate();
	}
				
	public void latheSelected(Point3f pivot, int num, float curvature, boolean peak)
	{
		startUndoRecorder("lathe");
		ArrayList[] clonedCPs = new ArrayList[num];
		ArrayList parentCPs = new ArrayList();
		clonedCPs[0] = new ArrayList();
		ArrayList clonedCurves = new ArrayList();
		Matrix4f rotation = new Matrix4f();
		for (int n = 1; n < num; n++)
		{
			clonedCPs[n] = new ArrayList();
			if (n == 1) clonedCurves = getClonedCurves(clonedCPs[0],clonedCPs[1]);
			else clonedCurves = getClonedCurves(parentCPs,clonedCPs[n]);
			rotation.setIdentity();
			rotation.rotY((float)(Math.PI*2*n/num));
			for (int c = 0; c < clonedCurves.size(); c++)
			{
				Curve curve = (Curve)clonedCurves.get(c);
				add(curve);
				recordAtomicAction(new NewCurveAtomicAction(this,curve));
			}
			for (int p = 0; p < clonedCPs[n].size(); p++)
			{
				ControlPoint cp = (ControlPoint)clonedCPs[n].get(p);
				rotation.transform(cp.position);
			}
			//transformSelected(rotation);
		}
		
		for (int p = 0; p < clonedCPs[0].size(); p++)
		{
			Curve curve = new Curve();
			curve.closed = true;
			for (int s = 0; s < num; s++)			
			{
				
				ControlPoint cp = (ControlPoint)clonedCPs[s].get(p);
				ControlPoint newCP = new ControlPoint(cp.position,cp.peak,cp.layer);
				if (cp.weldedTo == null) cp.isSelected = true;
				newCP.weldedTo = cp;
				newCP.inCurvature = curvature;
				newCP.outCurvature = curvature;
				newCP.peak = peak;
				curve.add(newCP);
			}
			add(curve);
			recordAtomicAction(new NewCurveAtomicAction(this,curve));
		}
		stopUndoRecorder();		
		validate();
	}
			
	/*
	public void cloneSelectedCurves()
	{
		ArrayList curvesToClone = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			if (curve.isSelected()) curvesToClone.add(curve);
		}
		unselectAll();
		
		for (int c = 0; c < curvesToClone.size(); c++)
		{
			Curve curve = (Curve)curvesToClone.get(c);
			Curve clone = curve.cloneCurve();
			add(clone);
		}
		
	}
	*/
	public void detachSelected()
	{
		//System.out.println("detach");
		startUndoRecorder("detach");
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				ControlPoint head = cp.getHead();
				if(head.isSelected)
				{
					ArrayList tailCPs = getTailCPs(head);
					for (int t = 0; t < tailCPs.size(); t++)
					{
						ControlPoint tcp = (ControlPoint)tailCPs.get(t);
						recordAtomicAction(new WeldUnweldControlPointAtomicAction(tcp,tcp.weldedTo,null));
						tcp.weldedTo = null;
						tcp.layer = head.layer;
					}
				}
			}
		}
		stopUndoRecorder();
		validate();
	}
	
	/*
	public void removeCP(ControlPoint cp)
	{
		//System.out.println("removeCP called");
		ArrayList weldedCPs = getWeldedCPs(cp);
		//System.out.println("weldedCPs size = " + weldedCPs.size());
		for (int w = 0; w < weldedCPs.size(); w++)
		{
			ControlPoint rcp = (ControlPoint)weldedCPs.get(w);
			rcp.curve.remove(rcp);
			rcp = null;
		}
	}
	*/
	/*
	public void removeCP(ControlPoint cp)
	{
		cp.curve.remove(cp);
	}
	
	public void removeAllCP(ControlPoint cp)
	{
		ArrayList weldedCPs = getWeldedCPs(cp);
		for (int p = 0; p < weldedCPs.size(); p++)
		{
			ControlPoint cp2 = (ControlPoint)weldedCPs.get(p);
			cp2.curve.remove(cp2);
		}
	}
	*/			
	public void deleteSelected()
	{
		startUndoRecorder("delete");
		ArrayList toDelete = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if ((cp.getHead()).isSelected)
				{
					ArrayList weldedCPs = getWeldedCPs(cp);
					toDelete.addAll(weldedCPs);	
				}
			}
		}
		removePoints(toDelete);
		
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			recordAtomicAction(new CurveCloseOpenAtomicAction(curve,curve.closed,false));
			if (curve.size() < 3) curve.closed = false;
		}
		ArrayList toRemove = new ArrayList();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			curve.validate();
			if (curve.size() < 2) toRemove.add(curve);
		}
		for (int c = 0; c < toRemove.size(); c++)
		{
			Curve curve = (Curve)toRemove.get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.weldedTo == null)
				{
					ArrayList tail = getTailCPs(cp);
					for (int t = 0; t < tail.size(); t++)
					{
						ControlPoint tp = (ControlPoint)tail.get(t);
						recordAtomicAction(new WeldUnweldControlPointAtomicAction(tp,tp.weldedTo,null));
						tp.weldedTo = null;
					}
				}
			}
		}
		for (int c = 0; c < toRemove.size(); c++)
		{
			Curve curve = (Curve)toRemove.get(c);
			recordAtomicAction(new RemoveCurveAtomicAction(this,curve));
			remove(curve);
		}
		stopUndoRecorder();
		validate();
	}
	
	public void insertPoint()
	{
		if (gw.activeCP != null)
		{
			gw.activeCP.curve.insert(gw.activeCP.pointOnCurve,gw.curveSelector);
		}
	}
	
	public void removePoints(ArrayList points)
	{
		for (int p = 0; p < points.size(); p++)
		{
			ControlPoint cp = (ControlPoint)points.get(p);
			cp.isSelected = false;
			//recordAtomicAction(new RemoveControlPointAtomicAction(cp.curve,cp));
			recordAtomicAction(cp.curve.remove(cp));
		}
	}
	
	String toString(boolean b)
	{
		if (b) return "1";
		else return "0";
	}
	
	String toString(float f)
	{
		BigInteger i = BigInteger.valueOf((long)(f*1e6));
		
		BigDecimal d = new BigDecimal(i);
		d = d.movePointLeft(6);
		return d.toString();
	}
	
	String sPatchImport(String filename)
	{
		BufferedReader file;
		String errorString = "";
		String line;
		Vector3f[] points = new Vector3f[0];
		int[] layers = new int[0];
		ControlPoint[] heads = new ControlPoint[0];
		boolean[] intToBoolean = { false,true };
		int version = 1;
		try
		{
			file = new BufferedReader(new FileReader(filename));
			while ((line = file.readLine()) != null)
			{
				if (line.startsWith("version"))
				{
					String[] part = line.split("\\s");
					version = (new Integer(part[1])).intValue();
				}
				else if (line.startsWith("num_points"))
				{
					String[] part = line.split("\\s");
					int numPoints = (new Integer(part[1])).intValue();
					points = new Vector3f[numPoints];
					layers = new int[numPoints];
					heads = new ControlPoint[numPoints];
					for (int p = 0;p < numPoints;p++)
					{
						line = file.readLine().replaceFirst("\\s*","");
						part = line.split("\\s");
						float x = (new Float(part[0])).floatValue();
						float y = (new Float(part[1])).floatValue();
						float z = (new Float(part[2])).floatValue();
						line = file.readLine().replaceFirst("\\s*","");
						int layer = (new Integer(line)).intValue();
						line = file.readLine().replaceFirst("\\s*","");
						if (version == 2)
						{
							int visible = (new Integer(line)).intValue();
							line = file.readLine().replaceFirst("\\s*","");
						}
						int numSplines = (new Integer(line)).intValue();
						for (int s = 0;s < numSplines;s++)
						{
							line = file.readLine().replaceFirst("\\s*","");
							part = line.split("\\s");
							int splineID = (new Integer(part[0])).intValue();
							int pointOnSpline = (new Integer(part[1])).intValue();
						}
						points[p] = new Vector3f(x,y,-z);
						layers[p] = layer;
						//heads[p] = null;
				 	}
				}
				else if (line.startsWith("num_curves"))
				{
					String[] part = line.split("\\s");
					int numCurves = (new Integer(part[1])).intValue();
					//curves = new SCurve[numCurves];
					for (int c = 0;c < numCurves;c++)
					{
						line = file.readLine().replaceFirst("\\s*","");
						part = line.split("\\s");
						int pointsOnSpline = (new Integer(part[0])).intValue();
						Curve curve = new Curve();
						curve.closed = intToBoolean[(new Integer(part[1])).intValue()];
						for (int p = 0;p < pointsOnSpline;p++)
						{
							line = file.readLine().replaceFirst("\\s*","");
							part = line.split("\\s");
							int pointID = (new Integer(part[0])).intValue();
							ControlPoint cp = new ControlPoint(points[pointID],false,0);
							cp.peak = !intToBoolean[(new Integer(part[1])).intValue()];
							cp.layer = layers[pointID];
							if (heads[pointID] == null) heads[pointID] = cp;
							else cp.weldedTo = heads[pointID];
							float curvature = (new Float(part[2])).floatValue();
							
							//remove this!!!
							//
							//curvature = 1f/3f;
							//
							//
							
							cp.inCurvature = curvature;
							cp.outCurvature = curvature;
							curve.add(cp);
						}
						add(curve);
					}
				}
			}
			file.close();
		}
		catch (IOException e)
		{
			errorString = e.getMessage();
		}
		return errorString;
	}		
			
	String sPatchExport(String filename)
	{
		class HeadPoint
		{
			Point3f position = new Point3f();
			ArrayList splinePoints = new ArrayList();
			int layer;
		}
		class SplinePoint
		{
			int splineID;
			int pointID;
			SplinePoint(int splineID, int pointID)
			{
				this.splineID = splineID;
				this.pointID = pointID;
			}			
		}		
		HeadPoint headPoint;
		SplinePoint splinePoint;
		ArrayList headPoints = new ArrayList();
		ArrayList helperCPs = new ArrayList();
		String tab = "\t";
		/**
		* first we make a list of all headpoints
		**/
		//System.out.print("building headpoints");
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.weldedTo == null)
				{
					/**
					* this is a headpoint
					**/
					headPoint = new HeadPoint();
					headPoint.position.set(cp.position);
					headPoint.layer = cp.layer;
					int splineID = indexOf(curve);
					int pointID = cp.pointOnCurve;
					splinePoint = new SplinePoint(splineID,pointID);
					headPoint.splinePoints.add(splinePoint);
					headPoints.add(headPoint);
					helperCPs.add(cp);
				}
			}
		}
		//System.out.println("...done");
		/**
		* second we add all tails to the headpoints
		**/
		//System.out.print("adding tails to headpoints");
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.weldedTo != null)
				{
					/**
					* this is a welded Point
					**/
					ControlPoint hp = cp.getHead();
					int index = helperCPs.indexOf(hp);
					headPoint = (HeadPoint)headPoints.get(index);
					int splineID = indexOf(curve);
					int pointID = cp.pointOnCurve;
					splinePoint = new SplinePoint(splineID,pointID);
					headPoint.splinePoints.add(splinePoint);
				}
			}
		}
		//System.out.println("...done");
		String errorString = "";
		BufferedWriter file;
		try
		{
			file = new BufferedWriter(new FileWriter(filename));
			file.write("sPatches");file.newLine();
			file.write("version 2");file.newLine();
			file.write("visibility FALSE");file.newLine();
			file.write("selection FALSE");file.newLine();
			file.write("num_points " + headPoints.size());file.newLine();
			/**
			* save all headpoint data
			**/
			//System.out.print("saving headpoint data");
			for (int p = 0; p < headPoints.size(); p++)
			{
				HeadPoint hp = (HeadPoint)headPoints.get(p);
				file.write(tab + toString(hp.position.x) + " " + toString(hp.position.y) + " " + toString(-hp.position.z));file.newLine();
				file.write(tab + tab + hp.layer);file.newLine();
				file.write(tab + tab + "0");file.newLine();
				file.write(tab + tab + hp.splinePoints.size());file.newLine(); // number of splines
				for (int s = 0; s < hp.splinePoints.size(); s++)
				{
					SplinePoint sp = (SplinePoint)hp.splinePoints.get(s);
					file.write(tab + tab + sp.splineID + " " + sp.pointID);file.newLine();
				}
			}
			//System.out.println("...done");
                	
			/**
			* save all curve data
			**/
			//System.out.print("saving curve data");
			file.newLine();
			file.write("num_curves " + size());file.newLine(); // number of curves;
			for (int c = 0; c < size(); c++)
			{
				Curve curve = (Curve)get(c);
				file.write(tab + curve.size() + " " + toString(curve.closed));file.newLine(); //number of points on spline, open/closed
				for (int p = 0; p < curve.size(); p++)
				{
					ControlPoint cp = (ControlPoint)curve.get(p);
					ControlPoint hp = cp.getHead();
					int index = helperCPs.indexOf(hp);
					file.write(tab + tab + index + " " + toString(!cp.peak) + " " + toString(cp.inCurvature));file.newLine(); // pointID, peak/round, curvature
				}
			}
			//System.out.println("...done");
			file.write("end_patches");file.newLine();
			file.close();
		}
		catch (IOException e)
		{
			errorString = e.getMessage();
		}
		return errorString;
	}
	
	public void computePatches()
	{
		patches.clear();
		/**
		* build some nice tables for fast point neighborhood information lookup
		**/
		//System.out.println("computePatches() called");
		computeTails();
		//System.out.println("computeTails() finished");
		class NeighborDetailEntry
		implements Comparable
		{
			ControlPoint headPoint;
			ControlPoint trueA;
			ControlPoint trueB;
			NeighborDetailEntry(ControlPoint headPoint, ControlPoint trueA, ControlPoint trueB)
			{
				this.headPoint = headPoint;
				this.trueA = trueA;
				this.trueB = trueB;
			}
			public int compareTo(Object ob)
			{
				NeighborDetailEntry nd = (NeighborDetailEntry)ob;
				return this.headPoint.compareTo(nd.headPoint);
			}
		}
		ArrayList pointList = getAllControlPoints();
		ArrayList headPointList = new ArrayList();
		int num = 0;
		for (int p = 0; p < pointList.size(); p++)
		{
			ControlPoint cp = (ControlPoint)pointList.get(p);
			if (cp.weldedTo == null)
			{
				cp.number = num++;
				headPointList.add(cp);
			}
		}
		ArrayList yHeadPointList = new ArrayList();	//stores neighbor headpoint lists for each headpoint
		ArrayList yDetailList = new ArrayList();	//stores neighbor details for each headpoint
		//TreeSet xHeadPointList;	// a list of neighbor headpoints
		//TreeSet xDetailList;		// a list of neighbor details
		Object[] xHeadPointArray;
		Object[] xDetailArray;
		for (int h = 0; h < headPointList.size(); h++)
		{
			ControlPoint cp = (ControlPoint)headPointList.get(h);
			ArrayList xHeadPointList = new ArrayList();
			ArrayList xDetailList = new ArrayList();
			ControlPoint neighbor;
			//xHeadPointList.add(cp);
			//xDetailList.add(new NeighborDetailEntry(cp,null,null));
			neighbor = cp.curve.getPrevControlPoint(cp);
			if (neighbor != null)
			{
				xHeadPointList.add(neighbor.getHead());
				xDetailList.add(new NeighborDetailEntry(neighbor.getHead(),cp,neighbor));
			}
			neighbor = cp.curve.getNextControlPoint(cp);
			if (neighbor != null)
			{
				xHeadPointList.add(neighbor.getHead());
				xDetailList.add(new NeighborDetailEntry(neighbor.getHead(),cp,neighbor));
			}
			ArrayList weldedTo = cp.tail;
			//System.out.println(cp.number + ":" + cp.tail.size());
			for (int w = 0; w < weldedTo.size(); w++)
			{
				ControlPoint wcp = (ControlPoint)weldedTo.get(w);
				neighbor = wcp.curve.getPrevControlPoint(wcp);
				if (neighbor != null)
				{
					xHeadPointList.add(neighbor.getHead());
					xDetailList.add(new NeighborDetailEntry(neighbor.getHead(),wcp,neighbor));
				}
				neighbor = wcp.curve.getNextControlPoint(wcp);
				if (neighbor != null)
				{
					xHeadPointList.add(neighbor.getHead());
					xDetailList.add(new NeighborDetailEntry(neighbor.getHead(),wcp,neighbor));
				}
			}
			xHeadPointArray = xHeadPointList.toArray();
			Arrays.sort(xHeadPointArray);
			yHeadPointList.add(xHeadPointArray);
			xDetailArray = xDetailList.toArray();
			Arrays.sort(xDetailArray);
			yDetailList.add(xDetailArray);
		}
		//System.out.println("neighborLists for " + headPointList.size() + " headPoints set up");
	
		/**
		* find patches
		**/
		int patch3 = 0;
		int patch4 = 0;
		ControlPoint hp;
		ControlPoint xHP;
		ControlPoint zHP;
		ControlPoint wHP;
		int index;
		int indexX;
		int indexZ;
		int w;
		int x;
		int y;
		int z;
		NeighborDetailEntry A;
		NeighborDetailEntry B;
		NeighborDetailEntry C;
		NeighborDetailEntry D;
		Object[] wHeadPointArray;
		Object[] wDetailArray;
		Curve c;
		for (y = 0; y < yHeadPointList.size(); y++)
		{
			hp = (ControlPoint)headPointList.get(y);
			//System.out.print(y + " <" + hp.position.x + "," + hp.position.y + "," + hp.position.z + "> : ");
			xHeadPointArray = (Object[])yHeadPointList.get(y);
			xDetailArray = (Object[])yDetailList.get(y);
			for (x = 0; x < xHeadPointArray.length; x++)
			{
				xHP = (ControlPoint)xHeadPointArray[x];
				A = (NeighborDetailEntry)xDetailArray[x];
				if (xHP.number > y)
				{
					//System.out.print(xHP.number + ",");
					//System.out.println("\t" + xHP.number);
					for (z = x + 1; z < xHeadPointArray.length; z++)
					{
						zHP = (ControlPoint)xHeadPointArray[z];
						B = (NeighborDetailEntry)xDetailArray[z];
						//System.out.println("\t\t" + zHP.number);
						w = zHP.number;
						wHeadPointArray = (Object[])yHeadPointList.get(w);
						wDetailArray = (Object[])yDetailList.get(w);
						index = Arrays.binarySearch(wHeadPointArray,xHP);
						if (index >=0)
						{
							/**
							* possible 3pt patch found
							**/
							//System.out.println("3pt patch found");
							C = (NeighborDetailEntry)wDetailArray[index];
							c = A.trueA.curve;
							if (c != A.trueB.curve || c != B.trueA.curve || c != B.trueB.curve || c != C.trueA.curve || c != C.trueB.curve)
							{
								//System.out.println("A: " + A.trueA.number + " " + A.trueB.number + " - " + A.trueA.curve.hashCode() + " " + A.trueB.curve.hashCode() + " - " + A.trueA.position + " " + A.trueB.position);
								//System.out.println("B: " + B.trueA.number + " " + B.trueB.number + " - " + B.trueA.curve.hashCode() + " " + B.trueB.curve.hashCode() + " - " + B.trueA.position + " " + B.trueB.position);
								//System.out.println("C: " + C.trueA.number + " " + C.trueB.number + " - " + C.trueA.curve.hashCode() + " " + C.trueB.curve.hashCode() + " - " + C.trueA.position + " " + C.trueB.position);
								patch3++;
								Patch patch = new Patch();
								patch.add(A.trueA);
								patch.add(A.trueB);
								patch.add(C.trueB);
								patch.add(C.trueA);
								patch.add(B.trueB);
								patch.add(B.trueA);
								patches.add(patch);
							}
								
						}
						for (w = y + 1; w < yHeadPointList.size(); w++)
						{
							wHeadPointArray = (Object[])yHeadPointList.get(w);
							wDetailArray = (Object[])yDetailList.get(w);
							indexX = Arrays.binarySearch(wHeadPointArray,xHP);
							if (indexX >=0)
							{
								indexZ = Arrays.binarySearch(wHeadPointArray,zHP);
								if (indexZ >= 0)
								{
									/**
									* possible 4pt patch found
									**/
									C = (NeighborDetailEntry)wDetailArray[indexX];
									D = (NeighborDetailEntry)wDetailArray[indexZ];
									/*
									//System.out.println("4pt patch found in row " + w);
									//System.out.println("A: " + A.trueA.number + " " + A.trueB.number + " - " + A.trueA.curve.hashCode() + " " + A.trueB.curve.hashCode() + " - " + A.trueA.position + " " + A.trueB.position);
									//System.out.println("B: " + B.trueA.number + " " + B.trueB.number + " - " + B.trueA.curve.hashCode() + " " + B.trueB.curve.hashCode() + " - " + B.trueA.position + " " + B.trueB.position);
									//System.out.println("C: " + C.trueA.number + " " + C.trueB.number + " - " + C.trueA.curve.hashCode() + " " + C.trueB.curve.hashCode() + " - " + C.trueA.position + " " + C.trueB.position);
									//System.out.println("D: " + D.trueA.number + " " + D.trueB.number + " - " + D.trueA.curve.hashCode() + " " + D.trueB.curve.hashCode() + " - " + D.trueA.position + " " + D.trueB.position);
									*/
									c = A.trueA.curve;
									if (c != A.trueB.curve || c != B.trueA.curve || c != B.trueB.curve || c != C.trueA.curve || c != C.trueB.curve || c != D.trueA.curve || c != D.trueB.curve)
									{
										patch4++;
										Patch patch = new Patch();
										patch.add(B.trueA);
										patch.add(B.trueB);
										patch.add(D.trueB);
										patch.add(D.trueA);
										patch.add(C.trueA);
										patch.add(C.trueB);
										patch.add(A.trueB);
										patch.add(A.trueA);
										patches.add(patch);
									}	
								}
							}
						}
					}
				}
			}
			//System.out.println();
		}
		//System.out.println();
		//System.out.println(patch3 + " 3pt patches found");
		//System.out.println(patch4 + " 4pt patches found");
	}
		
		/**
		* find 3-point-patches...
		
		for (int y = 0; y < headPointNeighborList.size(); y++)
		{
			ControlPoint c1 = ControlPoint(HeadPointList.get(y));
			ArrayList row = (ArrayList)headPointNeighborList.get(y);
			for (int x = 0; x < row.size(); x++)
			{
				NeighborListEntry entryX = (NeighborListEntry)row.get(x);
				ControlPoint c2 = entryX.headPoint;
				int searchIndex = headPointList.indexOf(cp2);
				ArrayList searchRow = headPointNeighborList.get(searchIndex);
				if (headPointList.indexOf(c2) > y)
				{
					for (int z = x + 1; z < row.size(); z++)
					{
						NeighborListEntry entryZ = (NeighborListEntry)row.get(z);
						ControlPoint c3 = entryZ.headPoint;
						
	}
						
	/*	
	public void computePatches()
	{
				
		HashSet cps = new HashSet(); //contains all controlPoints
		HashSet cpNeighborCPs = new HashSet(); //contains a neighbor-list for each controlPoint, uses the same index as cps
		HashSet cpNeighbotHPs = new HashSet(); //contains a neighbor-headpoint-list for each controlPoint, uses the same index as cps
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				cps.add(cp);
				ArrayList weldedTo =  getWeldedCPs(cp);
				ArrayList neighbors = new ArrayList();
				ArrayList neighborHPs = new ArrayList();
				for (int w = 0; w < weldedTo.size(); w++)
				{
					ControlPoint neighbor;
					ControlPoint wcp = (ControlPoint)weldedTo.get(w);
					neighbor = wcp.curve.getPrevControlPoint(wcp);
					if (neighbor != null)
					{
						neighbors.add(neighbor);
						neighborHPs.add(neighbor.getHead());
					}
					neighbor = wcp.curve.getNextControlPoint(wcp);
					if (neighbor != null)
				 	{
				 		neighbors.add(neighbor);
				 		neighborHPs.add(neighbor.getHead());
				 	}
				}
				cpNeighborCPs.add(neighbors);
				cpNeighbotHPs.add(neighborHPs);
			}
		}
	}	
	*/
	/*
	public ArrayList getFullClone(ArrayList toClone)
	{
		ArrayList headPoints = new ArrayList();
		ArrayList clonedHeadPoints = new ArrayList();
		ArrayList clone = new ArrayList();
		for (int c = 0; c < toClone.size(); c++)
		{
			Curve curve = (Curve)toClone.get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.weldedTo == null)
				{
					ControlPoint clonedCP = cp.clone(null);
					headPoints.add(cp);
					clonedHeadPoints.add(clonedCP);
				}
			}
		}	
		for (int c = 0; c < toClone.size(); c++)
		{
			Curve curve = (Curve)toClone.get(c);
			Curve clonedCurve = new Curve();
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.weldedTo == null)
				{
					int index = headPoints.indexOf(cp);
					ControlPoint clonedCP = (ControlPoint)clonedHeadPoints.get(index);
					clonedCurve.add(clonedCP);
				}
				else
				{
					int index = headPoints.indexOf(cp.getHead());
					ControlPoint clonedHeadPoint = (ControlPoint)clonedHeadPoints.get(index);
					ControlPoint clonedCP = cp.clone(clonedHeadPoint);
					clonedCurve.add(clonedCP);
				}
			}
			clone.add(clonedCurve);
		}
		return clone;
	}
	public void copyToUndoBuffer()
	{
		undoBuffer[undoPosition++] = getFullClone(this);
		undoPosition %= undoMax;
		undoFill++;
		if (undoFill > undoMax) undoFill = undoMax;
	}
	public void undo()
	{
		int p = --undoPosition;
		copyFromUndoBuffer(p);
		validate();
	}
	public void copyFromUndoBuffer(int pos)
	{
		clear();
		ArrayList clone = getFullClone(undoBuffer[pos]);
		addAll(clone);
	}	
	
	*/
	
	public String ribExport(String filename)
	{
		String errorString = "";
		BufferedWriter file;
		try
		{
			validate();
			patches.clear();
			computePatches();
			boolean[] voidLayers = new boolean[Config.layers];
			file = new BufferedWriter(new FileWriter(filename));
			file.write("#");file.newLine();
			file.write("# RenderMan interface bytestream");file.newLine();
			file.write("# generated by JPatch");file.newLine();
			file.write("#");file.newLine();
			file.newLine();
			file.write("# scene definitions");file.newLine();
			file.newLine();
			file.write("Projection \"perspective\" \"fov\" [45]");file.newLine();
			file.write("Translate 0 0 20");file.newLine();
			file.write("Rotate -30 1 0 0");file.newLine();
			file.write("Rotate -30 0 1 0");file.newLine();
			file.write("FrameBegin 1");file.newLine();
			file.write("LightSource \"ambientlight\" 1 \"intensity\" [0.2]");file.newLine();
			file.write("LightSource \"distantlight\" 2 \"intensity\" [1] \"lightcolor\" [0.5 0.2 0.0] \"from\" [-1000 500 1000] \"to\" [0 0 0]");file.newLine();
			file.write("LightSource \"distantlight\" 3 \"intensity\" [1] \"lightcolor\" [0.5 0.2 0.0] \"from\" [1000 500 1000] \"to\" [0 0 0]");file.newLine();
			file.write("LightSource \"distantlight\" 4 \"intensity\" [1] \"lightcolor\" [1.0 1.0 1.0] \"from\" [0 500 -1000] \"to\" [0 0 0]");file.newLine();
			file.write("WorldBegin");file.newLine();
			file.newLine();
			file.write("# object definitions");file.newLine();
			file.newLine();
			for (int l = 0; l < Config.layers; l++)
			{
				boolean layerVoid = true;
				for (int p = 0; p < patches.size(); p++)
				{
					Patch patch = (Patch)patches.get(p);
					if (patch.getLayer() == l)
					{
						if (layerVoid)
						{
							file.write("# --- Layer_" + (l + 1) + " ---");file.newLine();
							file.write("AttributeBegin");file.newLine();
							file.write("Color [" + Config.layerColors[l].r + " " + Config.layerColors[l].g + " " + Config.layerColors[l].b + "]");file.newLine();
							file.write("Surface \"plastic\" \"Ka\" [1] \"Kd\" [0.8] \"Ks\" [1] \"roughness\" [0.05]");file.newLine();
							file.write("TransformBegin");file.newLine();
							file.write("Basis \"bezier\" 3 \"bezier\" 3");file.newLine();
							layerVoid = false;
						} 
						file.write(patch.renderer(Patch.RENDERMAN));file.newLine();
					}
				}
				if (!layerVoid)
				{
					file.write("TransformEnd");file.newLine();
					file.write("AttributeEnd");file.newLine();
				}
				voidLayers[l] = layerVoid;
			}
			file.write("WorldEnd");file.newLine();
			file.write("FrameEnd");file.newLine();
			file.close();
		}
		catch (IOException e)
		{
			errorString = e.getMessage();
		}
		return errorString;
	}
	
	public String povExport(String filename)
	{
		String errorString = "";
		BufferedWriter file;
		try
		{
			validate();
			patches.clear();
			computePatches();
			boolean[] voidLayers = new boolean[Config.layers];
			file = new BufferedWriter(new FileWriter(filename));
			file.write("/*");file.newLine();
			file.write("*  POV-Ray scene description file");file.newLine();
			file.write("*  generated by JPatch");file.newLine();
			file.write("*/");file.newLine();
			file.newLine();
			file.write("// object definitions");file.newLine();
			file.newLine();
			for (int l = 0; l < Config.layers; l++)
			{
				boolean layerVoid = true;
				for (int p = 0; p < patches.size(); p++)
				{
					Patch patch = (Patch)patches.get(p);
					if (patch.getLayer() == l)
					{
						if (layerVoid)
						{
							file.write("#declare layer_" + (l + 1) + "_object = union");file.newLine();
							file.write("{");file.newLine();
							file.write("\t#declare layer_flatness = 0.0;");file.newLine();
							file.write("\t#declare layer_u_steps = 3;");file.newLine();
							file.write("\t#declare layer_v_steps = 3;");file.newLine();
							layerVoid = false;
						} 
						file.write("\t" + patch.renderer(Patch.POVRAY));file.newLine();
					}
				}
				if (!layerVoid)
				{
					file.write("}");file.newLine();
				}
				voidLayers[l] = layerVoid;
			}
			file.newLine();
			file.write("// material definitions");file.newLine();
			file.newLine();
			file.write("#default");file.newLine();
			file.write("{");file.newLine();
			file.write("\tpigment { color rgb <0.5,0.5,0.5> }");file.newLine();
			file.write("\tfinish { ambient 1 diffuse 0.8 phong 1 phong_size 50 }");file.newLine();
			file.write("}");file.newLine();
			for (int l = 0; l < Config.layers; l++)
			{
				if (!voidLayers[l])
				{
					file.write("#declare layer_" + (l + 1) + "_material = material { texture { pigment { color rgb <" + Config.layerColors[l].r + "," + Config.layerColors[l].g + "," + Config.layerColors[l].b + "> } } }");file.newLine();
				}
			}
			file.newLine();
			file.write("// scene definitions");file.newLine();
			file.newLine();
			file.write("global_settings { ambient_light rgb <0.2,0.2,0.2> }");file.newLine();
			file.write("camera { location <0,0,-20> look_at <0,0,0> rotate <30,30,0> angle 50}");file.newLine();
			file.write("light_source { <-1000,500, 1000> color <0.5,0.2,0.0> parallel }");file.newLine();
			file.write("light_source { < 1000,500, 1000> color <0.5,0.2,0.0> parallel }");file.newLine();
			file.write("light_source { <    0,500,-1000> color <1.0,1.0,1.0> parallel }");file.newLine();
			for (int l = 0; l < Config.layers; l++)
			{
				if (!voidLayers[l])
				{
					file.write("object { layer_" + (l + 1) + "_object material { layer_" + (l + 1) + "_material } }");file.newLine();
				}
			}
			file.close();
		}
		catch (IOException e)
		{
			errorString = e.getMessage();
		}
		return errorString;
	}
	
	/*
	
	public Shape3D getShape()
	{
		computePatches();
		return new J3DMesh();
	}
	
	public class J3DMesh
	extends Shape3D
	{
		//int uSteps = 4;
		//int vSteps = 4;
		String s = JOptionPane.showInputDialog("Number of subdivisions:","4");
		int uSteps = (new Integer(s)).intValue();
		int vSteps = uSteps;			
		public J3DMesh()
		{
			PolygonAttributes p = new PolygonAttributes();
			p.setCullFace(PolygonAttributes.CULL_NONE);
			p.setBackFaceNormalFlip(true);
			Material m = new Material();
			m.setDiffuseColor(0.0f,0.0f,1.0f);
			m.setSpecularColor(0.0f,0.0f,0.0f);
			m.setShininess(1.0f);
			Appearance a = new Appearance();
			a.setPolygonAttributes(p);
			a.setMaterial(m);
			setAppearance(a);
			setGeometry(meshGeometry());
		}
		private Geometry meshGeometry()
		{
			TriangleArray triangleArray;
			ArrayList polygons = new ArrayList();
			ArrayList colors = new ArrayList();
			ArrayList normals = new ArrayList();
			for (int p = 0; p < patches.size(); p++)
			{
				Patch patch = (Patch)patches.get(p);
				ArrayList bcpl = patch.getBicubicPatches();
				for (int b = 0; b < bcpl.size(); b++)
				{
					Vector3f[] cps = (Vector3f[])bcpl.get(b);
					//Point3f displayCP = new Point3f();
					
					
					
					Point3f[][] pos = new Point3f[uSteps + 1][vSteps + 1]; 
					Vector3f[][] normal = new Vector3f[uSteps + 1][vSteps + 1];
					for (int u = 0; u <= uSteps; u++)
					{
						float uu = (float)u/(float)uSteps;
						for (int v = 0; v <= vSteps; v++)
						{
							float vv = (float)v/(float)vSteps;
							pos[u][v] = patch.getPoint(uu,vv,cps);
							normal[u][v] = patch.getNormal(uu,vv,cps);
						}
					}
					Color3f color = Config.layerColors[patch.getLayer()].getColor3f();
					for (int u = 0; u < uSteps; u++)
					{
						for (int v = 0; v < vSteps; v++)
						{
							polygons.add(pos[u][v]);
							polygons.add(pos[u+1][v]);
							polygons.add(pos[u+1][v+1]);
							polygons.add(pos[u+1][v+1]);
							polygons.add(pos[u][v+1]);
							polygons.add(pos[u][v]);
							Vector3f s = new Vector3f(pos[u+1][v]);
							Vector3f t = new Vector3f(pos[u][v+1]);
							s.sub(pos[u][v]);
							t.sub(pos[u][v]);
							Vector3f n = new Vector3f();
							n.cross(s,t);
							colors.add(color);
							colors.add(color);
							colors.add(color);
							colors.add(color);
							colors.add(color);
							colors.add(color);
							Vector3f uv = new Vector3f(normal[u][v]);
							Vector3f u1v = new Vector3f(normal[u+1][v]);
							Vector3f u1v1 = new Vector3f(normal[u+1][v+1]);
							Vector3f uv1 = new Vector3f(normal[u][v+1]);
							//uv.scale(-1);
							//u1v.scale(-1);
							//u1v1.scale(-1);
							//uv1.scale(-1);
							normals.add(uv);
							normals.add(u1v);
							normals.add(u1v1);
							normals.add(u1v1);
							normals.add(uv1);
							normals.add(uv);
							
						
							
							
						}
					}	
				}
			}
			Point3f[] coords = new Point3f[polygons.size()];
			Color3f[] cols = new Color3f[colors.size()];
			Vector3f[] norms = new Vector3f[normals.size()];
			
			for (int p = 0; p < polygons.size(); p++)
			{
				coords[p] = (Point3f)polygons.get(p);
				cols[p] = (Color3f)colors.get(p);
				norms[p] = (Vector3f)normals.get(p);
			}
			System.out.println(coords.length);
			triangleArray = new TriangleArray (coords.length,
			                        TriangleArray.COORDINATES
			                    	| TriangleArray.COLOR_3
			                        | TriangleArray.NORMALS
			                        );
			
			triangleArray.setCoordinates(0, coords);
			triangleArray.setColors(0,cols);
			triangleArray.setNormals(0,norms);
			
			return triangleArray;	
			
		}
	}
	public class J3DMeshTest
	extends Shape3D
	{
		public J3DMeshTest()
		{
			setGeometry(meshGeometry());
		}
		private Geometry meshGeometry()
		{
			TriangleFanArray tfa;
			int     N = 17;
			int     totalN = 4*(N+1);
			Point3f coords[] = new Point3f[totalN];
			Color3f colors[] = new Color3f[totalN];
			Color3f red = new Color3f(1.0f, 0.0f, 0.0f);
			Color3f yellow = new Color3f(0.7f, 0.5f, 0.0f);
			int     stripCounts[] = {N+1, N+1, N+1, N+1};
			float   r = 0.6f;
			float   w = 0.4f;
			int     n;
			double  a;
			float   x, y;
			
			// set the central points for four triangle fan strips
			coords[0*(N+1)] = new Point3f(0.0f, 0.0f, w);
			coords[1*(N+1)] = new Point3f(0.0f, 0.0f, 0.0f);
			coords[2*(N+1)] = new Point3f(0.0f, 0.0f, 0.0f);
			coords[3*(N+1)] = new Point3f(0.0f, 0.0f, -w);
			
			colors[0*(N+1)] = red;
			colors[1*(N+1)] = yellow;
			colors[2*(N+1)] = yellow;
			colors[3*(N+1)] = red;
			
			for(a = 0,n = 0; n < N; a = 2.0*Math.PI/(N-1) * ++n)
			{
			        x = (float) (r * Math.cos(a));
			        y = (float) (r * Math.sin(a));
			        coords[0*(N+1)+n+1] = new Point3f(x, y, w);
			        coords[1*(N+1)+N-n] = new Point3f(x, y, w);
			        coords[2*(N+1)+n+1] = new Point3f(x, y, -w);
			        coords[3*(N+1)+N-n] = new Point3f(x, y, -w);
			
			        colors[0*(N+1)+N-n] = red;
			        colors[1*(N+1)+n+1] = yellow;
			        colors[2*(N+1)+N-n] = yellow;
			        colors[3*(N+1)+n+1] = red;
			}
			
			tfa = new TriangleFanArray (totalN,
			                        TriangleFanArray.COORDINATES |
			                        TriangleFanArray.COLOR_3,
			                        stripCounts);
			
			tfa.setCoordinates(0, coords);
			tfa.setColors(0,colors);
			
			return tfa;	
			
		}
	}
	*/

	public void mirrorSelected(int direction) {
		String[] undoname={"x", "y", "z"};

		startUndoRecorder("mirror "+undoname[direction]);
		prepareUndoableTransform();
		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if ((cp.getHead()).isSelected)
				{
					switch (direction) {
						case Mesh.DIRECTION_X:
							cp.position.x=gw.cursor.x+gw.cursor.x-cp.position.x;
							cp.inTangent.x*=-1;
							cp.outTangent.x*=-1;
							break;
						case Mesh.DIRECTION_Z:
							cp.position.z=gw.cursor.z+gw.cursor.z-cp.position.z;
							cp.inTangent.z*=-1;
							cp.outTangent.z*=-1;
							break;
						default:
							cp.position.y=gw.cursor.y+gw.cursor.y-cp.position.y;
							cp.inTangent.y*=-1;
							cp.outTangent.y*=-1;

					}
				}
			}
		}
		recordUndoableTransform();
		stopUndoRecorder();
		validate();
	}

	public void selectConnected() {
		ArrayList selected=new ArrayList();

		for (int c = 0; c < size(); c++)
		{
			Curve curve = (Curve)get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				if (cp.getHead().isSelected) {
					selectConnectedFunc(curve, selected);
				}
			}
		}
	}

	public void selectConnectedFunc(Curve curve, ArrayList selected) {
			if (selected.contains(curve)) return;
			for (int p=0;p<curve.size();p++) {
				ControlPoint cp=(ControlPoint)curve.get(p);
				cp.isSelected=true;
				selected.add(curve);
				if (cp.weldedTo!=null) {
					Curve tmp=cp.weldedTo.curve;
					if ( (!tmp.equals(curve)) && (!selected.contains(tmp)) )
						selectConnectedFunc(tmp, selected);
				}
				ControlPoint tcp=cp.curve.getNextControlPoint(cp);
				if (tcp!=null) {
					Curve tmp=tcp.curve;
					if ( (!tmp.equals(curve)) && (!selected.contains(tmp)) )
						selectConnectedFunc(tmp, selected);
				}
				ControlPoint tcpp=cp.curve.getPrevControlPoint(cp);
				if (tcpp!=null) {
					Curve tmp=tcpp.curve;
					if ( (!tmp.equals(curve)) && (!selected.contains(tmp)) )
						selectConnectedFunc(tmp, selected);
				}
			}
	}

}	

