/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.util.*;
import javax.vecmath.*;

public class Patch
extends ArrayList
{
	static public boolean new_method = false;
	static public boolean blend = false;
	
	static final int POVRAY = 1;
	static final int RENDERMAN = 2;
	
	public int getLayer()
	{
		int layer = -1;
		for (int p = 0; p < size(); p++)
		{
			ControlPoint cp = (ControlPoint)get(p);
			if (layer == -1 || layer > cp.layer) layer = cp.layer;
		}
		return layer;
	}
	
	/**
	* returns 3,4,5 for 3-,4- or 5-point-patches, -1 for invalid patches
	**/
	public int getType()
	{
		int p = size();
		if (p != 6 && p!= 8 && p != 10) p = -2;
		p/=2;
		return p;
	}
	
	public ArrayList getCoonCPs()
	{
		//System.out.println(getType() + "pt patch");
		Vector3f tangentA;
		Vector3f tangentB;
		ArrayList coonCPs = new ArrayList();
		int p = 0;
		while (p < size())
		{
			//System.out.println(p);
			ControlPoint cpA = (ControlPoint)get(p++);
			ControlPoint cpB = (ControlPoint)get(p++);
			//System.out.println(cpA.curve.hashCode() + " / " + cpB.curve.hashCode());
			assert cpA.curve == cpB.curve : "patch edge controlpoints not on same curve";
			assert (cpA.pointOnCurve == cpB.pointOnCurve - 1 ||
				cpA.pointOnCurve == cpB.pointOnCurve + 1 ||
				cpA.pointOnCurve == 0 && cpB.pointOnCurve == cpB.curve.size() - 1 && cpB.curve.closed ||
				cpB.pointOnCurve == 0 && cpA.pointOnCurve == cpB.curve.size() - 1 && cpB.curve.closed) : "patch edge controlpoints not neighbors";
			coonCPs.add(cpA.position);
			tangentA = new Vector3f(cpA.position);
			tangentB = new Vector3f(cpB.position);
			//System.out.println(tangentA + " " + tangentB);
			if (cpA.pointOnCurve == cpB.pointOnCurve - 1 || cpA.pointOnCurve == cpA.curve.size() - 1 && cpB.pointOnCurve == 0 && cpA.curve.closed)
			{
				tangentA.add(cpA.outTangent);
				tangentB.add(cpB.inTangent);
			}
			else
			{
				tangentA.add(cpA.inTangent);
				tangentB.add(cpB.outTangent);
			}
			coonCPs.add(tangentA);
			coonCPs.add(tangentB);
		}
		return coonCPs;
	}
	
	public Vector3f parallelogram(Vector3f a, Vector3f b, Vector3f c)
	{
		Vector3f result = new Vector3f(b);
		result.add(c);
		result.sub(a);
		return result;
	}
	
	// returns (1-c)*a + c*b
	public Vector3f blend(Vector3f a, Vector3f b, float c)
	{
		Vector3f result = new Vector3f(a);
		Vector3f bb = new Vector3f(b);
		result.scale(1 - c);
		bb.scale(c);
		result.add(bb);
		return result;
	}
	
	private float b0(float t) { return (1-t)*(1-t)*(1-t); }
	private float b1(float t) { return 3*t*(1-t)*(1-t); }
	private float b2(float t) { return 3*t*t*(1-t); }
	private float b3(float t) { return t*t*t; }
	
	private float db0(float t) { return -3*(1-t)*(1-t); }
	private float db1(float t) { return -6*t*(1-t) + 3*(1-t)*(1-t); }
	private float db2(float t) { return -3*t*t + 6*t*(1-t); }
	private float db3(float t) { return 3*t*t; }
	
	
	public Point3f getPoint(float t, float s, Vector3f[] cp)
	{
		Point3f result = new Point3f();
		result.x = b0(s)*(cp[0].x*b0(t) + cp[1].x*b1(t) + cp[2].x*b2(t) + cp[3].x*b3(t))
			 + b1(s)*(cp[4].x*b0(t) + cp[5].x*b1(t) + cp[6].x*b2(t) + cp[7].x*b3(t))
			 + b2(s)*(cp[8].x*b0(t) + cp[9].x*b1(t) + cp[10].x*b2(t) + cp[11].x*b3(t))
			 + b3(s)*(cp[12].x*b0(t) + cp[13].x*b1(t) + cp[14].x*b2(t) + cp[15].x*b3(t));
		result.y = b0(s)*(cp[0].y*b0(t) + cp[1].y*b1(t) + cp[2].y*b2(t) + cp[3].y*b3(t))
			 + b1(s)*(cp[4].y*b0(t) + cp[5].y*b1(t) + cp[6].y*b2(t) + cp[7].y*b3(t))
			 + b2(s)*(cp[8].y*b0(t) + cp[9].y*b1(t) + cp[10].y*b2(t) + cp[11].y*b3(t))
			 + b3(s)*(cp[12].y*b0(t) + cp[13].y*b1(t) + cp[14].y*b2(t) + cp[15].y*b3(t));
		result.z = b0(s)*(cp[0].z*b0(t) + cp[1].z*b1(t) + cp[2].z*b2(t) + cp[3].z*b3(t))
			 + b1(s)*(cp[4].z*b0(t) + cp[5].z*b1(t) + cp[6].z*b2(t) + cp[7].z*b3(t))
			 + b2(s)*(cp[8].z*b0(t) + cp[9].z*b1(t) + cp[10].z*b2(t) + cp[11].z*b3(t))
			 + b3(s)*(cp[12].z*b0(t) + cp[13].z*b1(t) + cp[14].z*b2(t) + cp[15].z*b3(t));	 
		return result;
	}
	
	
	/*
	public Vector3f getNormal(float t, float s, Vector3f[] cp)
	{
		Vector3f result = new Vector3f();
		result.x = db0(s)*(cp[0].x*db0(t) + cp[1].x*db1(t) + cp[2].x*db2(t) + cp[3].x*db3(t))
			 + db1(s)*(cp[4].x*db0(t) + cp[5].x*db1(t) + cp[6].x*db2(t) + cp[7].x*db3(t))
			 + db2(s)*(cp[8].x*db0(t) + cp[9].x*db1(t) + cp[10].x*db2(t) + cp[11].x*db3(t))
			 + db3(s)*(cp[12].x*db0(t) + cp[13].x*db1(t) + cp[14].x*db2(t) + cp[15].x*db3(t));
		result.y = db0(s)*(cp[0].y*db0(t) + cp[1].y*db1(t) + cp[2].y*db2(t) + cp[3].y*db3(t))
			 + db1(s)*(cp[4].y*db0(t) + cp[5].y*db1(t) + cp[6].y*db2(t) + cp[7].y*db3(t))
			 + db2(s)*(cp[8].y*db0(t) + cp[9].y*db1(t) + cp[10].y*db2(t) + cp[11].y*db3(t))
			 + db3(s)*(cp[12].y*db0(t) + cp[13].y*db1(t) + cp[14].y*db2(t) + cp[15].y*db3(t));
		result.z = db0(s)*(cp[0].z*db0(t) + cp[1].z*db1(t) + cp[2].z*db2(t) + cp[3].z*db3(t))
			 + db1(s)*(cp[4].z*db0(t) + cp[5].z*db1(t) + cp[6].z*db2(t) + cp[7].z*db3(t))
			 + db2(s)*(cp[8].z*db0(t) + cp[9].z*db1(t) + cp[10].z*db2(t) + cp[11].z*db3(t))
			 + db3(s)*(cp[12].z*db0(t) + cp[13].z*db1(t) + cp[14].z*db2(t) + cp[15].z*db3(t));	
		result.normalize(); 
		return result;
	}
	*/
	
	
	public Vector3f getNormal(float t, float s, Vector3f[] cp)
	{
		/*
		if (s == 0) s=0.0001f;
		if (s == 1) s=0.9999f;
		if (t == 0) t=0.0001f;
		if (t == 1) t=0.9999f;
		*/
		Vector3f dt = new Vector3f();
		dt.x = b0(s)*(cp[0].x*db0(t) + cp[1].x*db1(t) + cp[2].x*db2(t) + cp[3].x*db3(t))
			 + b1(s)*(cp[4].x*db0(t) + cp[5].x*db1(t) + cp[6].x*db2(t) + cp[7].x*db3(t))
			 + b2(s)*(cp[8].x*db0(t) + cp[9].x*db1(t) + cp[10].x*db2(t) + cp[11].x*db3(t))
			 + b3(s)*(cp[12].x*db0(t) + cp[13].x*db1(t) + cp[14].x*db2(t) + cp[15].x*db3(t));
		dt.y = b0(s)*(cp[0].y*db0(t) + cp[1].y*db1(t) + cp[2].y*db2(t) + cp[3].y*db3(t))
			 + b1(s)*(cp[4].y*db0(t) + cp[5].y*db1(t) + cp[6].y*db2(t) + cp[7].y*db3(t))
			 + b2(s)*(cp[8].y*db0(t) + cp[9].y*db1(t) + cp[10].y*db2(t) + cp[11].y*db3(t))
			 + b3(s)*(cp[12].y*db0(t) + cp[13].y*db1(t) + cp[14].y*db2(t) + cp[15].y*db3(t));
		dt.z = b0(s)*(cp[0].z*db0(t) + cp[1].z*db1(t) + cp[2].z*db2(t) + cp[3].z*db3(t))
			 + b1(s)*(cp[4].z*db0(t) + cp[5].z*db1(t) + cp[6].z*db2(t) + cp[7].z*db3(t))
			 + b2(s)*(cp[8].z*db0(t) + cp[9].z*db1(t) + cp[10].z*db2(t) + cp[11].z*db3(t))
			 + b3(s)*(cp[12].z*db0(t) + cp[13].z*db1(t) + cp[14].z*db2(t) + cp[15].z*db3(t));
			 
		Vector3f ds = new Vector3f();
		ds.x = db0(s)*(cp[0].x*b0(t) + cp[1].x*b1(t) + cp[2].x*b2(t) + cp[3].x*b3(t))
			 + db1(s)*(cp[4].x*b0(t) + cp[5].x*b1(t) + cp[6].x*b2(t) + cp[7].x*b3(t))
			 + db2(s)*(cp[8].x*b0(t) + cp[9].x*b1(t) + cp[10].x*b2(t) + cp[11].x*b3(t))
			 + db3(s)*(cp[12].x*b0(t) + cp[13].x*b1(t) + cp[14].x*b2(t) + cp[15].x*b3(t));
		ds.y = db0(s)*(cp[0].y*b0(t) + cp[1].y*b1(t) + cp[2].y*b2(t) + cp[3].y*b3(t))
			 + db1(s)*(cp[4].y*b0(t) + cp[5].y*b1(t) + cp[6].y*b2(t) + cp[7].y*b3(t))
			 + db2(s)*(cp[8].y*b0(t) + cp[9].y*b1(t) + cp[10].y*b2(t) + cp[11].y*b3(t))
			 + db3(s)*(cp[12].y*b0(t) + cp[13].y*b1(t) + cp[14].y*b2(t) + cp[15].y*b3(t));
		ds.z = db0(s)*(cp[0].z*b0(t) + cp[1].z*b1(t) + cp[2].z*b2(t) + cp[3].z*b3(t))
			 + db1(s)*(cp[4].z*b0(t) + cp[5].z*b1(t) + cp[6].z*b2(t) + cp[7].z*b3(t))
			 + db2(s)*(cp[8].z*b0(t) + cp[9].z*b1(t) + cp[10].z*b2(t) + cp[11].z*b3(t))
			 + db3(s)*(cp[12].z*b0(t) + cp[13].z*b1(t) + cp[14].z*b2(t) + cp[15].z*b3(t));	 
		
		Vector3f normal = new Vector3f();
		//normal.set(ds);
		normal.cross(dt,ds);
		normal.normalize();
		return normal;
	}
		
		 
	/*	
	public Point3f getPoint(float u, float v, Vector3f[] cps)
	{
		Point3f result = new Point3f();
		float u1 = 1 - u;
		float v1 = 1 - v;
		float vs = v*v;
		float vc = vs*v;
		float v1s = v1*v1;
		float v1c = v1s*v1;
		float us = u*u;
		float uc = us*u;
		float u1s = u1*u1;
		float u1c = u1s*u1;
		result.x = u1c*(cps[0].x*v1c + 3*cps[1].x*v1s*v + 3*cps[2].x * v1*vs + cps[3].x*vc)
			 + 3*u1s*u*(cps[4].x*v1c + 3*cps[5].x*v1s*v + 3*cps[6].x * v1*vs + cps[7].x*vc)
			 + 3*u1*us*(cps[8].x*v1c + 3*cps[9].x*v1s*v + 3*cps[10].x * v1*vs + cps[11].x*vc)
			 + uc*(cps[12].x*v1c + 3*cps[13].x*v1s*v + 3*cps[14].x * v1*vs + cps[15].x*vc);
		result.y = u1c*(cps[0].y*v1c + 3*cps[1].y*v1s*v + 3*cps[2].y * v1*vs + cps[3].y*vc)
			 + 3*u1s*u*(cps[4].y*v1c + 3*cps[5].y*v1s*v + 3*cps[6].y * v1*vs + cps[7].y*vc)
			 + 3*u1*us*(cps[8].y*v1c + 3*cps[9].y*v1s*v + 3*cps[10].y * v1*vs + cps[11].y*vc)
			 + uc*(cps[12].y*v1c + 3*cps[13].y*v1s*v + 3*cps[14].y * v1*vs + cps[15].y*vc);
		result.z = u1c*(cps[0].z*v1c + 3*cps[1].z*v1s*v + 3*cps[2].z * v1*vs + cps[3].z*vc)
			 + 3*u1s*u*(cps[4].z*v1c + 3*cps[5].z*v1s*v + 3*cps[6].z * v1*vs + cps[7].z*vc)
			 + 3*u1*us*(cps[8].z*v1c + 3*cps[9].z*v1s*v + 3*cps[10].z * v1*vs + cps[11].z*vc)
			 + uc*(cps[12].z*v1c + 3*cps[13].z*v1s*v + 3*cps[14].z * v1*vs + cps[15].z*vc);
		return result;
	}
	*/
			 
			 
	public ArrayList getBicubicPatches()
	{
		ArrayList bicubicPatches = new ArrayList();
		Vector3f[] cps = new Vector3f[16];
		ArrayList coonCPs = getCoonCPs();
		switch(size())
		{
			case 8:
				/**
				* four point patch
				**/
				
				//Vector3f[] cps = new Vector3f[16];
				//coonCPs = getCoonCPs();
				/*
				for (int p = 0; p < coonCPs.size(); p++)
				{
					//System.out.println(p + " " + (Vector3f)coonCPs.get(p));
				}
				*/
				cps[0] = new Vector3f((Vector3f)coonCPs.get(0));
				cps[1] = new Vector3f((Vector3f)coonCPs.get(1));
				cps[2] = new Vector3f((Vector3f)coonCPs.get(2));
				cps[3] = new Vector3f((Vector3f)coonCPs.get(3));
				cps[7] = new Vector3f((Vector3f)coonCPs.get(4));
				cps[11] = new Vector3f((Vector3f)coonCPs.get(5));
				cps[15] = new Vector3f((Vector3f)coonCPs.get(6));
				cps[14] = new Vector3f((Vector3f)coonCPs.get(7));
				cps[13] = new Vector3f((Vector3f)coonCPs.get(8));
				cps[12] = new Vector3f((Vector3f)coonCPs.get(9));
				cps[8] = new Vector3f((Vector3f)coonCPs.get(10));
				cps[4] = new Vector3f((Vector3f)coonCPs.get(11));

				if (!blend)
				{
				/**
				* calculate inner controlpoints with parallelogram
				**/
				cps[5] = parallelogram(cps[0],cps[1],cps[4]);
				cps[6] = parallelogram(cps[3],cps[2],cps[7]);
				cps[9] = parallelogram(cps[12],cps[8],cps[13]);
				cps[10] = parallelogram(cps[15],cps[11],cps[14]);
				}
				
				else
				{
				/**
				* calculate blended inner controlpoints (gregory?)
				**/
				Vector3f v0_4 = new Vector3f(cps[4]); v0_4.sub(cps[0]);
				Vector3f v0_1 = new Vector3f(cps[1]); v0_1.sub(cps[0]);
				Vector3f v3_2 = new Vector3f(cps[2]); v3_2.sub(cps[3]);
				Vector3f v3_7 = new Vector3f(cps[7]); v3_7.sub(cps[3]);
				Vector3f v12_8 = new Vector3f(cps[8]); v12_8.sub(cps[12]);
				Vector3f v12_13 = new Vector3f(cps[13]); v12_13.sub(cps[12]);
				Vector3f v15_11 = new Vector3f(cps[11]); v15_11.sub(cps[15]);
				Vector3f v15_14 = new Vector3f(cps[14]); v15_14.sub(cps[15]);
				
				Vector3f v1_5 = blend(v0_4,v3_7,1f/3f);
				Vector3f v2_6 = blend(v0_4,v3_7,2f/3f);
				Vector3f v13_9 = blend(v12_8,v15_11,1f/3f);
				Vector3f v14_10 = blend(v12_8,v15_11,2f/3f);
				Vector3f v4_5 = blend(v0_1,v12_13,1f/3f);
				Vector3f v8_9 = blend(v0_1,v12_13,2f/3f);
				Vector3f v7_6 = blend(v3_2,v15_14,1f/3f);
				Vector3f v11_10 = blend(v3_2,v15_14,2f/3f);
				/*
				Vector3f v5v = new Vector3f(cps[1]); v5v.add(v1_5);
				Vector3f v5h = new Vector3f(cps[4]); v5v.add(v4_5);
				Vector3f v6v = new Vector3f(cps[2]); v5v.add(v2_6);
				Vector3f v6h = new Vector3f(cps[7]); v5v.add(v7_6);
				Vector3f v9v = new Vector3f(cps[13]); v5v.add(v13_9);
				Vector3f v9h = new Vector3f(cps[8]); v5v.add(v8_9);
				Vector3f v10v = new Vector3f(cps[14]); v5v.add(v14_10);
				Vector3f v10h = new Vector3f(cps[11]); v5v.add(v11_10);
				*/
				Vector3f v5v = new Vector3f(cps[1]); v5v.add(v1_5);
				Vector3f v5h = new Vector3f(cps[4]); v5h.add(v4_5);
				Vector3f v6v = new Vector3f(cps[2]); v6v.add(v2_6);
				Vector3f v6h = new Vector3f(cps[7]); v6h.add(v7_6);
				Vector3f v9v = new Vector3f(cps[13]); v9v.add(v13_9);
				Vector3f v9h = new Vector3f(cps[8]); v9h.add(v8_9);
				Vector3f v10v = new Vector3f(cps[14]); v10v.add(v14_10);
				Vector3f v10h = new Vector3f(cps[11]); v10h.add(v11_10);
				
				cps[5] = blend(v5v,v5h,0.5f);
				cps[6] = blend(v6v,v6h,0.5f);
				cps[9] = blend(v9v,v9h,0.5f);
				cps[10] = blend(v10v,v10h,0.5f);
				}
			
				bicubicPatches.add(cps);
			break;
			case 6:
			if (!new_method)
			{
				/**
				* three point patch
				* ugly hack - returns just one bicubic patch
				**/
				
				//Vector3f[] cps = new Vector3f[16];
				coonCPs = getCoonCPs();
				
				cps[0] = new Vector3f((Vector3f)coonCPs.get(0));
				cps[1] = new Vector3f((Vector3f)coonCPs.get(1));
				cps[2] = new Vector3f((Vector3f)coonCPs.get(2));
				cps[3] = new Vector3f((Vector3f)coonCPs.get(3));
				cps[7] = new Vector3f((Vector3f)coonCPs.get(4));
				cps[11] = new Vector3f((Vector3f)coonCPs.get(5));
				cps[15] = new Vector3f((Vector3f)coonCPs.get(6));
				cps[14] = new Vector3f((Vector3f)coonCPs.get(6));
				cps[13] = new Vector3f((Vector3f)coonCPs.get(6));
				cps[12] = new Vector3f((Vector3f)coonCPs.get(6));
				cps[8] = new Vector3f((Vector3f)coonCPs.get(7));
				cps[4] = new Vector3f((Vector3f)coonCPs.get(8));
				cps[5] = parallelogram(cps[0],cps[1],cps[4]);
				cps[6] = parallelogram(cps[3],cps[2],cps[7]);
				cps[9] = parallelogram(cps[12],cps[8],cps[13]);
				cps[10] = parallelogram(cps[15],cps[11],cps[14]);
				bicubicPatches.add(cps);
			}
			else
			{
				
				/**
				* returns four bezier-patches
				**/
				//dfsdf
				//coonCPs = getCoonCPs();
				
				Point3f p003 = new Point3f((Vector3f)coonCPs.get(0));
				Point3f p102 = new Point3f((Vector3f)coonCPs.get(1));
				Point3f p201 = new Point3f((Vector3f)coonCPs.get(2));
				Point3f p300 = new Point3f((Vector3f)coonCPs.get(3));
				Point3f p210 = new Point3f((Vector3f)coonCPs.get(4));
				Point3f p120 = new Point3f((Vector3f)coonCPs.get(5));
				Point3f p030 = new Point3f((Vector3f)coonCPs.get(6));
				Point3f p021 = new Point3f((Vector3f)coonCPs.get(7));
				Point3f p012 = new Point3f((Vector3f)coonCPs.get(8));
				BezierTriangle bt = new BezierTriangle(p003,p102,p201,p300,p210,p120,p030,p021,p012);
				bt.compute();
				
				cps = new Vector3f[16];
				cps[0] = new Vector3f(bt.cp[0]);
				cps[1] = new Vector3f(bt.cp[1]);
				cps[2] = new Vector3f(bt.cp[2]);
				cps[3] = new Vector3f(bt.cp[3]);
				cps[7] = new Vector3f(bt.cp[24]);
				cps[11] = new Vector3f(bt.cp[23]);
				cps[15] = new Vector3f(bt.cp[18]);
				cps[14] = new Vector3f(bt.cp[21]);
				cps[13] = new Vector3f(bt.cp[22]);
				cps[12] = new Vector3f(bt.cp[15]);
				cps[8] = new Vector3f(bt.cp[16]);
				cps[4] = new Vector3f(bt.cp[17]);
				cps[5] = parallelogram(cps[0],cps[1],cps[4]);
				cps[6] = parallelogram(cps[3],cps[2],cps[7]);
				cps[9] = parallelogram(cps[12],cps[8],cps[13]);
				cps[10] = parallelogram(cps[15],cps[11],cps[14]);
				bicubicPatches.add(cps);
				
				cps = new Vector3f[16];
				cps[0] = new Vector3f(bt.cp[3]);
				cps[1] = new Vector3f(bt.cp[4]);
				cps[2] = new Vector3f(bt.cp[5]);
				cps[3] = new Vector3f(bt.cp[6]);
				cps[7] = new Vector3f(bt.cp[7]);
				cps[11] = new Vector3f(bt.cp[8]);
				cps[15] = new Vector3f(bt.cp[9]);
				cps[14] = new Vector3f(bt.cp[20]);
				cps[13] = new Vector3f(bt.cp[19]);
				cps[12] = new Vector3f(bt.cp[18]);
				cps[8] = new Vector3f(bt.cp[23]);
				cps[4] = new Vector3f(bt.cp[24]);
				cps[5] = parallelogram(cps[0],cps[1],cps[4]);
				cps[6] = parallelogram(cps[3],cps[2],cps[7]);
				cps[9] = parallelogram(cps[12],cps[8],cps[13]);
				cps[10] = parallelogram(cps[15],cps[11],cps[14]);
				bicubicPatches.add(cps);
				
				cps = new Vector3f[16];
				cps[0] = new Vector3f(bt.cp[15]);
				cps[1] = new Vector3f(bt.cp[22]);
				cps[2] = new Vector3f(bt.cp[21]);
				cps[3] = new Vector3f(bt.cp[18]);
				cps[7] = new Vector3f(bt.cp[19]);
				cps[11] = new Vector3f(bt.cp[20]);
				cps[15] = new Vector3f(bt.cp[9]);
				cps[14] = new Vector3f(bt.cp[10]);
				cps[13] = new Vector3f(bt.cp[11]);
				cps[12] = new Vector3f(bt.cp[12]);
				cps[8] = new Vector3f(bt.cp[13]);
				cps[4] = new Vector3f(bt.cp[14]);
				cps[5] = parallelogram(cps[0],cps[1],cps[4]);
				cps[6] = parallelogram(cps[3],cps[2],cps[7]);
				cps[9] = parallelogram(cps[12],cps[8],cps[13]);
				cps[10] = parallelogram(cps[15],cps[11],cps[14]);
				bicubicPatches.add(cps);
			}	
				
			break;
		}		
		return bicubicPatches;
	}
	public String renderer(int type)
	{
		ArrayList bicubicPatches;
		String result = "";
		switch(type)
		{
			case POVRAY:
				bicubicPatches = getBicubicPatches();
				for (int p = 0; p < bicubicPatches.size(); p++)
				{
					Vector3f[] cps = (Vector3f[])bicubicPatches.get(p);
					result =  "bicubic_patch { type 1 flatness layer_flatness u_steps layer_u_steps v_steps layer_v_steps ";
					for (int c = 0; c < 16; c++)
					{
						result += ("<" + (-cps[c].x) + "," + cps[c].y + "," + cps[c].z + ">");
						if (c != 15) result += (",");
						else result +="}";
					}
				}
			break;
			case RENDERMAN:
				bicubicPatches = getBicubicPatches();
				for (int p = 0; p < bicubicPatches.size(); p++)
				{
					Vector3f[] cps = (Vector3f[])bicubicPatches.get(p);
					result =  "Patch \"bicubic\" \"P\" [";
					for (int c = 0; c < 16; c++)
					{
						result += ((-cps[c].x) + " " + cps[c].y + " " + cps[c].z);
						if (c != 15) result += (" ");
						else result +="]";
					}
				}
			break;
		}
		return result;
	}			 
}
