/**
 *    Copyright (c)2003 Sascha Ledinsky
 *
 *    This file is part of JPatch.
 *
 *    JPatch is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JPatch is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JPatch; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author     Developer : Dan Bishop
 * @version    $Id: RotoscopeInput.java,v 1.4 2003/03/30 23:47:46 danbishop Exp $ $Date: 2003/03/30 23:47:46 $
 */
package jpatch;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.io.File;

/**
 *  Description of the Class
 *
 * @author     Dan Bishop
 * @created    March 30, 2003
 */
class RotoscopeInput
		 extends JDialog
		 implements ActionListener, CaretListener, FocusListener {
	/**
	 *  Description of the Fields
	 */
	public JTextField field = new JTextField();
	private static boolean code;
	private JButton fileButton = new JButton("File");
	private JButton okButton = new JButton("Ok");
	private JButton cancelButton = new JButton("Cancel");
	private JButton resetButton = new JButton("Reset");
	private static RotoscopeInput rotoInput;
	private Frame frame;
	private Image scaledImage;
	private String[] strFile = new String[1];
	private GraphicsPanel gp;


	/**
	 *Constructor for the RotoscopeInput object
	 *
	 * @param  frame  Frame associated with the view
	 * @param  v      Background image file
	 * @param  gp     GraphicsPanel
	 */
	public RotoscopeInput(Frame frame, String[] v, GraphicsPanel gp) {

		super(frame, "Background Image Information", true);
		this.strFile = v;
		this.gp = gp;
		this.frame = frame;
		setSize(450, 100);
		Container cp = getContentPane();
		cp.setLayout(new FlowLayout());
		field = new JTextField(strFile[0], 30);
		cp.add(field);
		field.addCaretListener(this);
		cp.add(fileButton);
		fileButton.addActionListener(this);
		okButton.addActionListener(this);
		cp.add(okButton);
		cancelButton.addActionListener(this);
		cp.add(cancelButton);
		resetButton.addActionListener(this);
		cp.add(resetButton);
		setLocationRelativeTo(frame);
		setVisible(true);
	}


	/**
	 *  Description of the Method
	 *
	 *  Returns the code value depending on which button is chosen
	 * @param  event  Description of the Parameter
	 */
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == okButton) {
			if (strFile[0].compareTo("") == 0) {
				code = false;
			} else {
				code = true;
			}
			dispose();
		}
		if (event.getSource() == cancelButton) {
			code = false;
			dispose();
		}
		if (event.getSource() == resetButton) {
			strFile[0] = "";
			gp.gw.backgroundInfo[gp.viewMode].backgroundImage = null;
			gp.gw.backgroundInfo[gp.viewMode].fscale = 1.0f;
			gp.gw.backgroundInfo[gp.viewMode].frotoscopeXoffset = 0;
			gp.gw.backgroundInfo[gp.viewMode].frotoscopeYoffset = 0;
			field.setText(strFile[0]);

		}
		if (event.getSource() == fileButton) {
			setBackgroundImage();
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  event  Description of the Parameter
	 */
	public void focusGained(FocusEvent event) {
		if (event.getSource() == field) {
			field.selectAll();
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  event  Description of the Parameter
	 */
	public void focusLost(FocusEvent event) {
	}


	/**
	 *  Description of the Method
	 *
	 * @param  event  CaretEvent
	 */
	public void caretUpdate(CaretEvent event) {
		try {
			if (event.getSource() == field) {
				strFile[0] = field.getText();
			}
		} catch (Exception e) {
			System.err.println(e);
		}

	}


	/**
	 *  Sets the backgroundImage attribute of the RotoscopeInput object
	 */
	public void setBackgroundImage() {

		JFileChooser fc = new JFileChooser();
		fc.addChoosableFileFilter(new ImageFilter());
		File file = null;

		try {
			int returnValue = fc.showOpenDialog(this);
			if (returnValue == JFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
			}
			field.setText(file.getPath());
			strFile[0] = file.getPath();
			frame.repaint();

		} catch (Exception e) {
			System.err.println(e);
		}

	}


	/**
	 *  Gets the values attribute of the RotoscopeInput class
	 *
	 * @param  frame    frame associated with the view
	 * @param  strFile  File for the background image
	 * @param  gp       GraphicsPanel
	 * @return          The values value
	 */
	public static boolean getValues(Frame frame, String[] strFile, GraphicsPanel gp) {
		rotoInput = new RotoscopeInput(frame, strFile, gp);
		return code;
	}

}
