/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;

public class SelectMoveML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected int button;
	protected MouseMotionAdapter motionListener;
	protected MouseAdapter weldListener;
	//protected MoverMML moveListener;
	protected int mode = -1;
	static final int SELECTOR = 1;
	static final int POINT = 2;
	
	public SelectMoveML(GraphicsWindow gw,GraphicsPanel gp,int button)
	{
		this.gw = gw;
		this.gp = gp;
		this.button = button;
		gp.removeAllMouseMotionListeners();
	}	

	public void mousePressed(MouseEvent event)
	{
		if (button == event.getButton())
		{
			Point2D.Float mouse = new Point2D.Float(event.getX(),event.getY());
			gp.setPopupEnable(false);
			gp.mesh.initHelper();
			////System.out.println(mouseX + "/" + mouseY);
			ControlPoint closestPoint = gp.getClosestPoint(mouse,8);
			float mouseX = mouse.x;
			float mouseY = mouse.y;
			////System.out.println(mouseX + "/" + mouseY);
			////System.out.println("---");
			if (closestPoint != null)
			{
				mode = POINT;
				//gp.activeCP = closestPoint;
				gp.mesh.startUndoRecorder("translation");
				gp.mesh.prepareUndoableTransform();
				Curve curve = closestPoint.curve;
				ControlPoint cp = closestPoint;
				if (!event.isShiftDown() && !event.isControlDown() && !cp.isSelected)
				{
					gp.mesh.unselectAll();
					cp.isSelected = true;
				}
				if (!event.isShiftDown() && !event.isControlDown()) cp.isSelected = true;
				else cp.isSelected = !cp.isSelected;
				gw.repaint();
				motionListener = new MoverMML(gw,gp,mouseX,mouseY,cp);
				gp.addMouseMotionListener(motionListener);
				gp.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
				weldListener = new WeldML(gp,MouseEvent.BUTTON3,cp,GraphicsWindow.ADD_SEGMENT);
				gp.addMouseListener(weldListener);
			}
			else
			{
				//gp.activeCP = null;
				if (!event.isShiftDown() && !event.isControlDown()) gp.mesh.unselectAll();
				mode = SELECTOR;
				gp.setSelectorA(mouseX,mouseY);
				motionListener = new SelectorMML(gp);
				gp.addMouseMotionListener(motionListener);
				gp.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
				gw.repaint();
			}
		}
	}

	public void mouseReleased(MouseEvent event)
	{
		if (button == event.getButton())
		{
			if (mode == POINT)
			{
				gp.mesh.recordUndoableTransform();
				gp.mesh.stopUndoRecorder();
			}
			gp.setPopupEnable(true);
			gp.removeMouseMotionListener(motionListener);
			gp.removeMouseListener(weldListener);
			gp.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			//gp.clearSelector();
			if (mode == SELECTOR && gp.selectorRectangle.visible)
			{
				gp.selectRectangle(!event.isShiftDown() && !event.isControlDown());
				gw.repaint();
			}
			/*
			if (mode == POINT)
			{
				gp.mesh.unselectAll();
			}
			*/
		}
	}

	class SelectorMML
	extends MouseMotionAdapter
	{
		protected GraphicsPanel gp;
		
		public SelectorMML(GraphicsPanel gp)
		{
			this.gp = gp;
		}

		public void mouseDragged(MouseEvent event)
		{
			float mouseX = event.getX();
			float mouseY = event.getY();
			gp.setSelectorB(mouseX,mouseY);
			//gp.repaint();
		}
	}
	class MoverMML
	extends MouseMotionAdapter
	{
		/*
		protected float mouseX;
		protected float mouseY;
		protected float dragX;
		protected float dragY;
		*/
		protected GraphicsPanel gp;
		protected GraphicsWindow gw;
		protected Point3f mouse;
		protected ControlPoint cp;
		
		public MoverMML(GraphicsWindow gw,GraphicsPanel gp, float mouseX, float mouseY, ControlPoint cp)
		{
			this.cp = cp;
			this.gw = gw;
			this.gp = gp;
			mouse = new Point3f(cp.position);
			gw.cursor.set(cp.position);
			//gp.snapToClosestPoint(mouse,8,null);
		}

		public void mouseDragged(MouseEvent event)
		{
			gp = (GraphicsPanel)event.getSource();
			Point3f vector = new Point3f(gp.get3dCoords(event.getX(),event.getY()));
			//vector.z = cp.position.z;
			
			if (gw.snapToGrid) gp.snapToGrid(vector);
			//if (!gp.snapToClosestPoint(vector,8,cp)) if (gw.snapToGrid) gp.snapToGrid(vector);
			/*
			gp.iView.set(gp.view);
			gp.iView.invert();
			gp.iView.transform(vector);
			*/
			Point3f dummy = new Point3f(vector);
			vector.sub(mouse);
			mouse.set(dummy);
			gp.mesh.moveSelected(vector);
			gw.repaint();
		}
	}
}