/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;

public class SetCursorML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected int button;
	protected Point3f oldCursor;
	protected Point3f newCursor;
	protected MouseMotionAdapter motionListener;
	//protected MoverMML moveListener;
	
	public SetCursorML(GraphicsPanel gp,int button)
	{
		this.gp = gp;
		this.button = button;
		gp.removeAllMouseMotionListeners();
	}	

	public void mousePressed(MouseEvent event)
	{
		if (button == event.getButton())
		{
			gp.setPopupEnable(false);
			float mouseX = event.getX();
			float mouseY = event.getY();
			motionListener = new MoveCursorMML(gp);
			gp.addMouseMotionListener(motionListener);
			Point3f c = new Point3f(gp.get3dCoords(mouseX,mouseY));
			if (!gp.snapToClosestPoint(c,8,null)) if (gp.gw.snapToGrid) gp.snapToGrid(c);;
			gp.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
			gp.gw.cursor.set(c);	
			gp.gw.repaint();
		}
	}

	public void mouseReleased(MouseEvent event)
	{
		if (button == event.getButton())
		{
			gp.setPopupEnable(true);
			gp.removeMouseMotionListener(motionListener);
			gp.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	}
	
	class MoveCursorMML
	extends MouseMotionAdapter
	{
		protected GraphicsPanel gp;
		
		public MoveCursorMML(GraphicsPanel gp)
		{
			this.gp = gp;
		}

		public void mouseDragged(MouseEvent event)
		{
			float mouseX = event.getX();
			float mouseY = event.getY();
			Point3f c = new Point3f(gp.get3dCoords(mouseX,mouseY));
			if (gp.gw.snapToGrid) gp.snapToGrid(c);;
			gp.gw.cursor.set(c);	
			gp.gw.repaint();
		}
	}
}

	
