/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;

public class TangentML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected int button;
	protected MouseMotionAdapter motionListener;
	protected int segment = 0;
	protected float curvature;
	
	public TangentML(GraphicsPanel gp,int button)
	{
		this.gw = gp.gw;
		this.gp = gp;
		this.button = button;
		gp.removeAllMouseMotionListeners();
	}	

	public void mousePressed(MouseEvent event)
	{
		if (button == event.getButton())
		{
			Point2D.Float mouse = new Point2D.Float(event.getX(),event.getY());
			gp.setPopupEnable(false);
			ControlPoint closestPoint = gp.getClosestPoint(mouse,8);
			if (closestPoint != null)
			{
				if (gw.activeCP != null && closestPoint == gw.activeCP.getHead())
				{
					gw.curveSelector = !gw.curveSelector;
					if (gw.curveSelector)
					{
						ArrayList tail = closestPoint.tail;
						if (segment >= tail.size())
						{
							segment = 0;
							gw.activeCP = closestPoint;
						}
						else gw.activeCP = (ControlPoint)tail.get(segment++);
						//segment %= tail.size();
						//gp.activeCP =  % tail.size());
					}
				}
				else gw.activeCP = closestPoint;
				curvature = gw.activeCP.inCurvature;
				motionListener = new TangentMML(gp,event.getX(),event.getY());
				gp.addMouseMotionListener(motionListener);
			}
			else gw.activeCP = null;
			gw.repaint();
			//motionListener = new TangentMML(gp,event.getX(),event.getY());
			//gp.addMouseMotionListener(motionListener);
			//gp.mesh.startUndoRecorder(modeName[mode]);
			//gp.mesh.prepareUndoableTransform();
		}
	}

	public void mouseReleased(MouseEvent event)
	{
		if (button == event.getButton())
		{
			//gp.mesh.recordUndoableTransform();
			//gp.mesh.stopUndoRecorder();
			gp.removeMouseMotionListener(motionListener);
			gp.setPopupEnable(true);
			if (gw.activeCP != null && gw.activeCP.inCurvature != curvature)
			{
				gp.mesh.startUndoRecorder("change tangent");
				gp.mesh.recordAtomicAction(new TangentChangeAtomicAction(gw.activeCP,gw.activeCP.peak,curvature));
				gp.mesh.stopUndoRecorder();
			}
			//System.out.println(curvature);
			//gw.changeAction(gw.DEFAULT);
		}
	}

	class TangentMML
	extends MouseMotionAdapter
	{
		/*
		protected float mouseX;
		protected float mouseY;
		protected float dragX;
		protected float dragY;
		*/
		protected GraphicsPanel gp;
		protected GraphicsWindow gw;
		protected Point3f mouse;
		protected ControlPoint cp;
		protected int mode;
		protected float mouseX;
		protected float mouseY;
		protected boolean valid = false;
		
		public TangentMML(GraphicsPanel gp, float mouseX, float mouseY)
		{
			this.gw = gp.gw;
			this.gp = gp;
			this.mouseX = mouseX;
		}

		public void mouseDragged(MouseEvent event)
		{
			float deltaX = (event.getX() - mouseX);
			mouseX = event.getX();
			if (valid)
			{
				float c = gw.activeCP.inCurvature;
				c = Math.round(c*1000f + deltaX)/1000f;
				if (c < 0) c = 0;
				gw.activeCP.inCurvature = c;
				gw.activeCP.outCurvature = c;
				gw.activeCP.curve.validate();
				gw.repaint();
			}
			else valid = true;
			/*
			case SCALE:
				{
					float deltaX = event.getX() - mouseX;
					float scale = (1 + deltaX/100);
					if (event.isControlDown() || event.isShiftDown()) scale = Math.round(scale);
					Matrix4f transform = new Matrix4f();
					transform.setIdentity();
					Matrix4f dummy = new Matrix4f();
					Vector3f cv = new Vector3f(gw.cursor);
					transform.set(cv);
					dummy.set(scale);
					transform.mul(dummy);
					cv.scale(-1);
					dummy.set(cv);
					transform.mul(dummy);
					if (scale != 0) gp.mesh.transformSelected(transform);
					break;
				}
			*/
					
			//gw.repaint();
		}
	}
}