/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;

public class TransformML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected int button;
	protected MouseMotionAdapter motionListener;
	
	protected int mode = -1;
	static final int SCALE = 1;
	static final int ROTATE = 2;
	static final String[] modeName = { "","scale","rotation" };
	
	public TransformML(GraphicsPanel gp,int button, int mode)
	{
		this.mode = mode;
		this.gw = gp.gw;
		this.gp = gp;
		this.button = button;
		gp.removeAllMouseMotionListeners();
	}	

	public void mousePressed(MouseEvent event)
	{
		if (button == event.getButton())
		{
			gp.setPopupEnable(false);
			motionListener = new TransformMML(gp,event.getX(),event.getY(),mode);
			gp.addMouseMotionListener(motionListener);
			gp.mesh.startUndoRecorder(modeName[mode]);
			gp.mesh.prepareUndoableTransform();
		}
	}

	public void mouseReleased(MouseEvent event)
	{
		if (button == event.getButton())
		{
			gp.mesh.recordUndoableTransform();
			gp.mesh.stopUndoRecorder();
			gp.removeMouseMotionListener(motionListener);
			gp.setPopupEnable(true);
			gw.changeAction(gw.DEFAULT);
		}
	}

	class TransformMML
	extends MouseMotionAdapter
	{
		/*
		protected float mouseX;
		protected float mouseY;
		protected float dragX;
		protected float dragY;
		*/
		protected GraphicsPanel gp;
		protected GraphicsWindow gw;
		protected Point3f mouse;
		protected ControlPoint cp;
		protected int mode;
		protected float mouseX;
		protected float mouseY;
		
		public TransformMML(GraphicsPanel gp, float mouseX, float mouseY, int mode)
		{
			this.mode = mode;
			this.gw = gp.gw;
			this.gp = gp;
			this.mouseX = mouseX;
			this.mouseY = mouseY;
			gp.mesh.initHelper();
		}

		public void mouseDragged(MouseEvent event)
		{
			switch(mode)
			{
				case SCALE:
				{
					float deltaX = event.getX() - mouseX;
					float scale = (1 + deltaX/100);
					if (event.isControlDown() || event.isShiftDown()) scale = Math.round(scale);
					Matrix4f transform = new Matrix4f();
					transform.setIdentity();
					Matrix4f dummy = new Matrix4f();
					Vector3f cv = new Vector3f(gw.cursor);
					transform.set(cv);
					dummy.set(scale);
					transform.mul(dummy);
					cv.scale(-1);
					dummy.set(cv);
					transform.mul(dummy);
					if (scale != 0) gp.mesh.transformSelected(transform);
					break;
				}
				case ROTATE:
				{
					float deltaX = event.getX() - mouseX;
					float rotation = -deltaX/2;
					Vector3f axis = new Vector3f(0,0,1);
					gp.iView.set(gp.view);
					gp.iView.invert();
					gp.iView.transform(axis);
					if (event.isControlDown()) rotation = Math.round(rotation/5)*15;
					if (event.isShiftDown()) rotation = Math.round(rotation/15)*45;
					//System.out.println(rotation);
					Matrix4f transform = new Matrix4f();
					transform.setIdentity();
					Matrix4f dummy = new Matrix4f();
					Vector3f cv = new Vector3f(gw.cursor);
					transform.set(cv);
					dummy.setIdentity();
					dummy.set(new AxisAngle4f(axis,rotation/180*(float)Math.PI));
					transform.mul(dummy);
					cv.scale(-1);
					dummy.set(cv);
					transform.mul(dummy);
					gp.mesh.transformSelected(transform);
					break;
				}
			}		
			gw.repaint();
		}
	}
}