/**
 *    Copyright (c)2003 Sascha Ledinsky
 *
 *    This file is part of JPatch.
 *
 *    JPatch is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JPatch is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JPatch; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author     developer: Dan Bishop
 * @version    $Id: TranslateImageML.java,v 1.1 2003/03/31 00:59:17 danbishop Exp $ $Date: 2003/03/31 00:59:17 $
 */

package jpatch;

import java.awt.*;
import java.awt.event.*;

/**
 *  Description of the Class
 *
 *  Adds a MouseListener to the graphics panel
 * @author     Dan Bishop
 * @created    March 30, 2003
 */
public class TranslateImageML
		 extends TranslateViewML {
	/**
	 *Constructor for the TranslateImageML object
	 *
	 * @param  gp      GraphicsPanel associated with the Image
	 * @param  button  Description of the Parameter
	 */
	public TranslateImageML(GraphicsPanel gp, int button) {
		super(gp, button);
	}


	/**
	 *  Description of the Method
	 *
	 *  Overrides the mousePressed method so that a MouseMotionListener can be added
	 * @param  event  Description of the Parameter
	 */
	public void mousePressed(MouseEvent event) {
		if (button == event.getButton()) {
			gp.setPopupEnable(false);
			motionListener = new TranslateImageMML(gp, event.getX(), event.getY());
			gp.addMouseMotionListener(motionListener);
			gp.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		}
	}


	/**
	 *  Description of the Class
	 *
	 *  Adds MouseMotionListener to the graphics panel
	 * @author     dbishop
	 * @created    March 30, 2003
	 */
	public class TranslateImageMML
			 extends TranslateViewMML {
		/**
		 *Constructor for the TranslateImageMML object
		 *
		 * @param  gp      GraphicsPanel associated with the view
		 * @param  mouseX  x coordinate of the mouse
		 * @param  mouseY  y coordinate of the mouse
		 */
		public TranslateImageMML(GraphicsPanel gp, int mouseX, int mouseY) {
			super(gp, mouseX, mouseY);
		}


		/**
		 *  Description of the Method
		 *  Overrides the mouseDragged method to get the x and y offsets
		 *  for the image
		 *
		 * @param  event  MouseEvent
		 */
		public void mouseDragged(MouseEvent event) {
			int deltaX = event.getX() - mouseX;
			int deltaY = event.getY() - mouseY;
			mouseX = event.getX();
			mouseY = event.getY();
			gp.gw.backgroundInfo[gp.viewMode].frotoscopeXoffset += deltaX / gp.windowScale;
			gp.gw.backgroundInfo[gp.viewMode].frotoscopeYoffset += deltaY / gp.windowScale;
			gp.gw.repaint();
		}
	}
}

