/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.util.*;

/**
* the undo stack stores undoable user actions on a stack
**/
public class UndoStack
extends ArrayList
{
	protected int position = -1;
	protected int capacity;
	
	public UndoStack(int capacity)
	{
		this.capacity = capacity;
	}
	
	public String getUndoName()
	{
		if (position < size() && position >= 0) return ((UndoAction)get(position)).getName();
		else return "";
	}
	
	public String getRedoName()
	{
		if ((position + 1) < size() && position >= -1) return ((UndoAction)get(position + 1)).getName();
		else return "";
	}
	
	public void add(UndoAction undoAction)
	{
		clearRedoList();
		super.add(undoAction);
		if (size() > capacity) super.remove(0);
		position = size()-1;
	}
	
	protected void clearRedoList()
	{
		if ((position + 1) < size() && position >= -1) removeRange(position + 1, size());
	}
	
	public void undo()
	{
		if (position < size() && position >=0) ((UndoAction)get(position--)).undo();
	}
	
	public void redo()
	{
		if ((position + 1) < size() && position >= -1) ((UndoAction)get(++position)).redo();
	}
}		