/**
*    Copyright (c)2003 Sascha Ledinsky
*
*    This file is part of JPatch.
*
*    JPatch is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    JPatch is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with JPatch; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

package jpatch;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.vecmath.*;
import java.util.*;

public class WeldML
extends MouseAdapter
{
	protected GraphicsPanel gp;
	protected GraphicsWindow gw;
	protected Mesh mesh;
	protected int button;
	protected ControlPoint cp;
	protected Curve curve;
	protected int action;
	
	public WeldML(GraphicsPanel gp,int button, ControlPoint cp, int action)
	{
		this.gp = gp;
		this.gw = gp.gw;
		this.mesh = gw.mesh;
		this.button = button;
		this.cp = cp;
		curve = cp.curve;
		this.action = action;
	}	

	public void mousePressed(MouseEvent event)
	{
		////System.out.println("weld button pressed");
		if (button == event.getButton())
		{
			Point mouse = event.getPoint();
			
			////System.out.println(mouseX + "/" + mouseY);
			//ControlPoint closestPoint = gp.getClosestPoint(mouse,8,cp);
			//float mouseX = mouse.x;
			//float mouseY = mouse.y;
			
			
			
			float mouseX = event.getX();
			float mouseY = event.getY();
			Point3f coord = new Point3f(gp.get3dCoords(mouseX,mouseY));
			if (!gp.snapToClosestPoint(coord,8,cp)) if (gw.snapToGrid) gp.snapToGrid(coord);;
			gp.view.transform(coord);
			mouseX = coord.x;
			mouseY = coord.y;
			ControlPoint closestPoint = gp.getClosestPoint(new Point2D.Float(mouseX,mouseY),8,cp);
			
			////System.out.println("curve: " + closestCurveID + " point: " + closestPointID);
			check:
			if (closestPoint != null)
			{
				/**
				* change 24.4.2003 author: sascha ledinsky
				* if we're on a welded point, search and select a
				* curve end if possible
				* change 1.6.2003 author: sascha ledinsky
				* added a check for closed curves
				**/
				ArrayList weldedPoints = mesh.getWeldedCPs(closestPoint);
				for (int i = 0; i < weldedPoints.size(); i++) {
					ControlPoint cp = (ControlPoint)weldedPoints.get(i);
					if (!cp.curve.closed && (cp.pointOnCurve == 0 || cp.pointOnCurve == cp.curve.size() - 1)) {
						closestPoint = cp;
					}
				} 
	
				Curve closestCurve = closestPoint.curve;
				
				/**
				* check if we should close this curve
				**/
				if ((curve == closestCurve) && (curve.size() > 3))
				{
					if ((cp.pointOnCurve == 0 && closestPoint.pointOnCurve == (curve.size() - 1)) || (cp.pointOnCurve == (curve.size() -1) && closestPoint.pointOnCurve == 0))
					{
						mesh.recordUndoableTransform();
						mesh.recordAtomicAction(curve.remove(cp));
						curve.remove(cp);
						curve.closed = true;
						mesh.recordAtomicAction(new CurveCloseOpenAtomicAction(curve,false,true));
						mesh.stopUndoRecorder("close curve");
						mesh.validate();
						gw.repaint();
						gw.changeAction(GraphicsWindow.DEFAULT);
						break check;
					}
				}
				
				/**
				* check if we've hit a non-end of a curve
				**/
				if (closestCurve.closed || (closestPoint.pointOnCurve != 0 && closestPoint.pointOnCurve != (closestCurve.size() - 1)))
				{
					//ControlPoint closestPoint = (ControlPoint)closestCurve.get(closestPointID);
					
					cp.position.set(closestPoint.position);
					mesh.recordUndoableTransform();
					mesh.recordAtomicAction(new WeldUnweldControlPointAtomicAction(closestPoint,closestPoint.weldedTo,cp));
					// change 1.6.2003 author: sascha ledinsky
					// was closestPoint.weldedTo = cp;
					// (now we wled the head)
					closestPoint.getHead().weldedTo = cp;
					mesh.stopUndoRecorder("weld");
					mesh.validate();
					gw.repaint();
					gw.changeAction(GraphicsWindow.DEFAULT);
					break check;
				}
				
				/**
				* check if we should weld to curves together
				**/
				if (!closestCurve.closed && (closestPoint.pointOnCurve == 0 || closestPoint.pointOnCurve == (closestCurve.size() - 1)))
				{
					if (!curve.closed && (cp.pointOnCurve == 0 || cp.pointOnCurve ==  curve.size() - 1))
					{
						mesh.recordUndoableTransform();
						mesh.recordAtomicAction(mesh.weldCurves(curve,closestCurve,cp.pointOnCurve,closestPoint.pointOnCurve));
						mesh.stopUndoRecorder("weld");
						gw.repaint();
						gw.changeAction(GraphicsWindow.DEFAULT);
						break check;
					}
					else
					{
						cp.position.set(closestPoint.position);
						mesh.recordUndoableTransform();
						mesh.recordAtomicAction(new WeldUnweldControlPointAtomicAction(closestPoint,closestPoint.weldedTo,cp));
						closestPoint.weldedTo = cp;
						mesh.stopUndoRecorder("weld");
						mesh.validate();
						gw.repaint();
						gw.changeAction(GraphicsWindow.DEFAULT);
						break check;
						
					}
				}	
					
			}		
		}
	}
}