/**
 *    Copyright (c)2003 Sascha Ledinsky
 *
 *    This file is part of JPatch.
 *
 *    JPatch is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    JPatch is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with JPatch; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author     Developer: Dan Bishop
 * @version    $Id: ZoomImageML.java,v 1.1 2003/03/31 01:00:24 danbishop Exp $ $Date: 2003/03/31 01:00:24 $
 */

package jpatch;

import java.awt.event.*;

/**
 *  Description of the Class
 *  Adds Mouse Listener for the background image
 *
 * @author     Dan Bishop
 * @created    March 30, 2003
 */
public class ZoomImageML
		 extends ZoomViewML {
	/**
	 *Constructor for the ZoomImageML object
	 *
	 * @param  gp      GraphicsPanel associated with the image
	 * @param  button  Mouse button selected
	 */
	public ZoomImageML(GraphicsPanel gp, int button) {
		super(gp, button);
	}


	/**
	 *  Description of the Method
	 *  Overrides the mousePressed method so that a MouseMotionListener is added to the GraphicsPanel
	 *
	 * @param  event  Mouse Event
	 */
	public void mousePressed(MouseEvent event) {
		if (button == event.getButton()) {
			motionListener = new ZoomImageMML(gp, event.getX(), event.getY());
			gp.addMouseMotionListener(motionListener);

		}
	}


	/**
	 *  Description of the Class
	 *  Adds a MouseMotionListener for the background image
	 *
	 * @author     dbishop
	 * @created    March 30, 2003
	 */
	public class ZoomImageMML
			 extends ZoomViewMML {
		/**
		 *Constructor for the ZoomImageMML object
		 *
		 * @param  gp      GraphicsPanel for the image
		 * @param  mouseX  x coordinate for the mouse
		 * @param  mouseY  y coordinate for the mouse
		 */
		public ZoomImageMML(GraphicsPanel gp, int mouseX, int mouseY) {
			super(gp, mouseX, mouseY);
		}


		/**
		 *  Description of the Method
		 *  Method overrides the mouseDragged method so that it can
		 *  get the scale for the image
		 *
		 * @param  event  Mouse Event
		 */
		public void mouseDragged(MouseEvent event) {
			int deltaX = event.getX() - mouseX;
			int deltaY = event.getY() - mouseY;
			mouseX = event.getX();
			mouseY = event.getY();
			float ds = (1 + ((float) deltaX - (float) deltaY) / 200);
			if (ds < 0.2) {
				ds = (float) 0.2;
			}
			if (ds > 5) {
				ds = (float) 5;
			}
			gp.gw.backgroundInfo[gp.viewMode].fscale *= ds;
			gp.gw.repaint();
		}
	}
}

