/**
 *         ===============================
 *          JPatch open source 3D modeler
 *         ===============================
 *      Copyright (c)2003 JPatch developer team
 *         see http://jpatch.sourceforge.net
 *
 * This file is part of JPatch.
 *
 * JPatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JPatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JPatch; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Heiko Irrgang
 * @version $Id: Plugin.java,v 1.4 2003/06/04 07:14:01 god64 Exp $ $Date: 2003/06/04 07:14:01 $
 * last changed by $Author: god64 $
 *
 * Class: Plugin
 * Hungarian notation prefix: *not yet... my brain dont works in hungarian notation... will go over it when finished*
 *
 **/

package jpatch.plugin;

import jpatch.*;
import java.lang.reflect.Method;
import javax.vecmath.*;

public class Plugin {
	public String name;
	private GraphicsWindow gw;
	public static boolean isJPatchPlugin=true;

	public Plugin() {
	}

	public Plugin(GraphicsWindow gw, String name) {
		this.gw=gw;
		this.name=name;
	}

	public GraphicsWindow getGraphicsWindow() {
		return gw;
	}

	public GraphicsPanel getGraphicsPanel(int num) {
		return this.gw.graphicsPanels[num];
	}

	public GraphicsPanel getActiveGraphicsPanel() {
		return this.gw.graphicsPanels[this.gw.selectedPanel];
	}

	public int getActiveGraphicsPanelIndex() {
		return this.gw.selectedPanel;
	}

	public Mesh getMesh() {
		return getActiveGraphicsPanel().mesh;
	}

	public Point3f getCursor() {
		return this.gw.cursor;
	}

	public void withAllControlPoints(String method) {
		withAllControlPoints(method, true);
	}

	public void withAllControlPoints(String method, boolean recordUndo) {
		Method m;
		try {
			ControlPoint dummy=new ControlPoint(new Vector3f(0,0,0), false, 0);
			m=getClass().getMethod(method, new Class[] {dummy.getClass()});
		} catch (Exception e) {
			System.out.println("Plugin "+this.name+": withAllControlPoints: error initializing method "+method);
			return;
		}
		Mesh mesh=getMesh();

		if (recordUndo) {
			mesh.startUndoRecorder(this.name);
			mesh.prepareUndoableTransform();
		}
		for (int c = 0; c < mesh.size(); c++)
		{
			Curve curve = (Curve)mesh.get(c);
			for (int p = 0; p < curve.size(); p++)
			{
				ControlPoint cp = (ControlPoint)curve.get(p);
				try {
					m.invoke(this, new Object[]{cp});
				} catch (Exception e) {
					System.err.println("Plugin "+this.name+": withAllControlPoints(\""+method+"\") execution error");
				}
			}
		}
		if (recordUndo) {
			mesh.recordUndoableTransform();
			mesh.stopUndoRecorder();
		}
		mesh.validate();
		this.gw.repaint();
	}

	public void withAllCurves(String method) {
		withAllCurves(method, true);
	}

	public void withAllCurves(String method, boolean recordUndo) {
		Method m;
		try {
			Curve dummy=new Curve();
			m=getClass().getMethod(method, new Class[] {dummy.getClass()});
		} catch (Exception e) {
			System.out.println("Plugin "+this.name+": withAllCurves: error initializing method "+method);
			return;
		}
		Mesh mesh=getMesh();

		if (recordUndo) {
			mesh.startUndoRecorder(this.name);
			mesh.prepareUndoableTransform();
		}
		for (int c = 0; c < mesh.size(); c++)
		{
			Curve curve = (Curve)mesh.get(c);
			try {
				m.invoke(this, new Object[]{curve});
			} catch (Exception e) {
				System.err.println("Plugin "+this.name+": withAllCurves(\""+method+"\") execution error");
			}
		}
		if (recordUndo) {
			mesh.recordUndoableTransform();
			mesh.stopUndoRecorder();
		}
		mesh.validate();
		this.gw.repaint();
	}

	public void requestUndo(String name) {
		Mesh mesh=getMesh();
		mesh.startUndoRecorder(name);
		mesh.prepareUndoableTransform();
	}

	public void freeUndo() {
		Mesh mesh=getMesh();
		mesh.recordUndoableTransform();
		mesh.stopUndoRecorder();
	}

	public float getCameraXRotation(int gp) {
		return getGraphicsPanel(gp).viewDef.rotateX;
	}

	public float getCameraXRotation() {
		return getActiveGraphicsPanel().viewDef.rotateX;
	}

	public float getCameraYRotation(int gp) {
		return getGraphicsPanel(gp).viewDef.rotateY;
	}

	public float getCameraYRotation() {
		return getActiveGraphicsPanel().viewDef.rotateY;
	}

	public float getCameraXTranslation(int gp) {
		return getGraphicsPanel(gp).viewDef.translateX;
	}

	public float getCameraXTranslation() {
		return getActiveGraphicsPanel().viewDef.translateX;
	}

	public float getCameraYTranslation(int gp) {
		return getGraphicsPanel(gp).viewDef.translateY;
	}

	public float getCameraYTranslation() {
		return getActiveGraphicsPanel().viewDef.translateY;
	}

	public float getCameraScale(int gp) {
		return getGraphicsPanel(gp).viewDef.scale;
	}

	public float getCameraScale() {
		return getActiveGraphicsPanel().viewDef.scale;
	}

}
