/**
 *         ===============================
 *          JPatch open source 3D modeler
 *         ===============================
 *      Copyright (c)2003 JPatch developer team
 *         see http://jpatch.sourceforge.net
 *
 * This file is part of JPatch.
 *
 * JPatch is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JPatch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JPatch; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Heiko Irrgang
 * @version $Id: PluginManager.java,v 1.5 2003/06/03 07:46:16 god64 Exp $ $Date: 2003/06/03 07:46:16 $
 * last changed by $Author: god64 $
 *
 * Class: PluginManager
 * Hungarian notation prefix: *not yet... my brain dont works in hungarian notation... will go over it when finished*
 *
 **/

package jpatch.plugin;

import java.util.*;
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import jpatch.*;

public class PluginManager extends AbstractAction {
	private ArrayList io;
	private ArrayList modifiers;
	private PluginLoader pl;
	private GraphicsWindow gw;
	private boolean yetWarned=false;

	public PluginManager(GraphicsWindow gw) {
		this.pl=new PluginLoader();
		this.gw=gw;
		this.io=new ArrayList();
		this.modifiers=new ArrayList();
	}

	public void execModifier(int i) {
		PluginModifier p=(PluginModifier)this.modifiers.get(i);
		p.exec();
	}

	public void writeIO(int i, String filename) {
		PluginIO p=(PluginIO)this.io.get(i);
		p.write(filename);
	}

	public void readIO(int i, String filename) {
		PluginIO p=(PluginIO)this.io.get(i);
		p.read(filename);
	}

	public String getJPatchHomeVar() {
		String ret=System.getProperty("JPATCH_HOME");
		if (ret==null) ret=new String("");
		if (ret.length()>0) {
			ret+="/";
		}
		return ret;
	}
	public void loadPluginType(String type) {
		String prefix=getJPatchHomeVar();
		File f=new File(prefix+"plugins/"+type);
		File[] plg=f.listFiles();
		if (plg==null) {
			if (!this.yetWarned) {
				System.err.println("\n* WARNING: Plugin directory not found.\n* Please set your JPATCH_HOME environment variable.\n");
				this.yetWarned=true;
			}
			return;
		}
		for (int i=0;i<plg.length;i++) {
			String name=plg[i].getName();
			if (name.matches(".*\\.class$")) {
				if (type.equals("modifiers")) {
					loadModifier(name.replaceAll("\\.class$",""));
				} else {
					loadIO(name.replaceAll("\\.class$",""));
				}
			}
		}
	}

	public void loadPlugins() {
		loadPluginType("modifiers");
		loadPluginType("io");
	}

	public String[] getModifiers() {
		String[] ret=new String[modifiers.size()];
		for (int i=0;i<modifiers.size();i++) {
			PluginModifier plug=(PluginModifier)modifiers.get(i);
			ret[i]=plug.getName();
		}
		return ret;
	}

	public Object getValue(Class c, String name) {
		try {
			return c.getField(name).get(c);
		} catch (java.lang.Exception e) {}
		return null;
	}

	private void loadModifier(String name) {
		PluginModifier mod=new PluginModifier();
		try {
			pl.setFilePrefix("modifiers/");
			pl.setPathPrefix(getJPatchHomeVar());
			Class c=pl.loadClass(name);
			if (getValue(c,"isJPatchPlugin")!=null) {
				Method m=c.getMethod("exec", new Class[] {});
				String plugname=(String)getValue(c, "name");
				if (plugname!=null) {
					Constructor constr=c.getConstructor(new Class[] {gw.getClass()});
					Object o=constr.newInstance(new Object[] {gw});
					mod.setClass(c);
					mod.setMethod(m);
					mod.setInstance(o);
					mod.setName(plugname);
					modifiers.add(mod);
					// System.out.println("Plugin loaded: "+name);
				} else {
					System.err.println("WARNING: Illegal plugin format: modifiers/"+name+".plg (Ignored)");
				}
			}
		} catch (Exception e) {
			System.err.println("Plugin "+name+": error loading");
			System.err.println(e);
		}
	}

	private void loadIO(String name) {
		PluginIO mod=new PluginIO();
		String dummy=new String();
		try {
			pl.setFilePrefix("io/");
			pl.setPathPrefix(getJPatchHomeVar());
			Class c=pl.loadClass(name);
			if (getValue(c,"isJPatchPlugin")!=null) {
				Method mWrite=c.getMethod("write", new Class[] {dummy.getClass()});
				Method mRead=c.getMethod("read", new Class[] {dummy.getClass()});
				String plugname=(String)getValue(c, "name");
				if (plugname!=null) {
					Constructor constr=c.getConstructor(new Class[] {gw.getClass()});
					Object o=constr.newInstance(new Object[] {gw});
					mod.setClass(c);
					mod.setWriteMethod(mWrite);
					mod.setReadMethod(mRead);
					mod.setInstance(o);
					mod.setName(plugname);
					io.add(mod);
					// System.out.println("Plugin loaded: "+name);
				} else {
					System.err.println("WARNING: Illegal plugin format: io/"+name+".plg (Ignored)");
				}
			} else {
				System.err.println("no plugin: "+name);
			}
		} catch (Exception e) {
			System.err.println("Plugin "+name+": error loading");
		}
	}

	public void actionPerformed(ActionEvent event) {
		String cmd=event.getActionCommand();
		String[] mod=getModifiers();
		boolean found=false;
		for (int i=0;i<mod.length;i++) {
			if (mod[i].equals(cmd)) {
				execModifier(i);
				found=true;
				break;
			}
		}
		if (!found) {
			// TODO: exec io plugin
		}
	}
}
